=== modified file '.bzrignore'
--- .bzrignore	2010-09-30 06:24:20 +0000
+++ .bzrignore	2012-05-17 01:55:58 +0000
@@ -3,6 +3,7 @@
 *.8mandos
 confdir
 keydir
+statedir
 man
 plugin-runner
 plugins.d/askpass-fifo

=== modified file 'DBUS-API'
--- DBUS-API	2011-11-26 23:08:17 +0000
+++ DBUS-API	2012-01-15 20:27:28 +0000
@@ -93,10 +93,11 @@
    | Interval (a)            | t    | Read/Write | interval            |
    | LastApprovalRequest (g) | s    | Read       | N/A                 |
    | LastCheckedOK (h)       | s    | Read/Write | N/A                 |
-   | LastEnabled (i)         | s    | Read       | N/A                 |
+   | LastCheckerStatus (i)   | n    | Read       | N/A                 |
+   | LastEnabled (j)         | s    | Read       | N/A                 |
    | Name                    | s    | Read       | (Section name)      |
    | ObjectPath              | o    | Read       | N/A                 |
-   | Secret (j)              | ay   | Write      | secret (or secfile) |
+   | Secret (k)              | ay   | Write      | secret (or secfile) |
    | Timeout (a)             | t    | Read/Write | timeout             |
    
    a) Represented as milliseconds.
@@ -124,10 +125,13 @@
       always use an empty string when setting this property, to allow
       for possible future expansion.
    
-   i) The date and time this client was last enabled, as an RFC 3339
+   i) The exit status of the last checker, -1 if it did not exit
+      cleanly, -2 if a checker has not yet returned.
+   
+   j) The date and time this client was last enabled, as an RFC 3339
       string, or an empty string if this has not happened.
    
-   j) A raw byte array, not hexadecimal digits.
+   k) A raw byte array, not hexadecimal digits.
 
 ** Signals
 *** CheckerCompleted(n: Exitcode, x: Waitstatus, s: Command)
@@ -152,13 +156,10 @@
 *** Rejected(s: Reason)
     This client was not given its secret for a specified Reason.
 
-*** NewRequest(s: IP address)
-    A client at IP adress has asked for its password.
-
 * Copyright
 
-    Copyright © 2010-2011 Teddy Hogeborn
-    Copyright © 2010-2011 Björn Påhlsson
+    Copyright © 2010-2012 Teddy Hogeborn
+    Copyright © 2010-2012 Björn Påhlsson
   
 ** License:
    

=== modified file 'INSTALL'
--- INSTALL	2011-03-08 19:09:03 +0000
+++ INSTALL	2013-06-23 15:13:06 +0000
@@ -12,7 +12,7 @@
    server and client programs themselves *could* be run in other
    distributions, but they *are* specific to GNU/Linux systems, and
    are not written with portabillity to other Unixes in mind.
-  
+   
 ** Libraries
    
    The following libraries and packages are needed.  (It is possible
@@ -35,7 +35,7 @@
     To build just the documentation, run the command "make doc".  Then
     the manual page "mandos.8", for example, can be read by running
     "man -l mandos.8".
-   
+    
 *** Mandos Server
     + GnuTLS 2.4          http://www.gnu.org/software/gnutls/
     + Avahi 0.6.16        http://www.avahi.org/
@@ -51,7 +51,7 @@
     Package names:
     python-gnutls avahi-daemon python python-avahi python-dbus
     python-gobject python-urwid
-   
+    
 *** Mandos Client
     + initramfs-tools 0.85i
                   http://packages.qa.debian.org/i/initramfs-tools.html

=== modified file 'Makefile'
--- Makefile	2011-12-21 00:33:12 +0000
+++ Makefile	2012-06-22 23:33:56 +0000
@@ -23,7 +23,7 @@
 OPTIMIZE=-Os
 LANGUAGE=-std=gnu99
 htmldir=man
-version=1.4.1
+version=1.6.0
 SED=sed
 
 USER=$(firstword $(subst :, ,$(shell getent passwd _mandos || getent passwd nobody || echo 65534)))
@@ -68,11 +68,13 @@
 	--param make.single.year.ranges		1 \
 	--param man.output.quietly		1 \
 	--param man.authors.section.enabled	0 \
-	 /usr/share/xml/docbook/stylesheet/nwalsh/manpages/docbook.xsl \
+	/usr/share/xml/docbook/stylesheet/nwalsh/manpages/docbook.xsl \
 	$(notdir $<); \
 	$(MANPOST) $(notdir $@);\
-	LANG=en_US.UTF-8 MANWIDTH=80 man --warnings --encoding=UTF-8 \
-	--local-file $(notdir $@) >/dev/null)
+	if locale --all 2>/dev/null | grep --regexp='^en_US\.utf8$$' \
+	&& type man 2>/dev/null; then LANG=en_US.UTF-8 MANWIDTH=80 \
+	man --warnings --encoding=UTF-8 --local-file $(notdir $@); \
+	fi >/dev/null)
 # DocBook-to-man post-processing to fix a '\n' escape bug
 MANPOST=$(SED) --in-place --expression='s,\\\\en,\\en,g;s,\\n,\\en,g'
 
@@ -239,6 +241,7 @@
 
 check:	all
 	./mandos --check
+	./mandos-ctl --check
 
 # Run the client with a local config and key
 run-client: all keydir/seckey.txt keydir/pubkey.txt
@@ -264,7 +267,7 @@
 	./mandos-keygen --dir keydir --force
 
 # Run the server with a local config
-run-server: confdir/mandos.conf confdir/clients.conf
+run-server: confdir/mandos.conf confdir/clients.conf statedir
 	./mandos --debug --no-dbus --configdir=confdir \
 		--statedir=statedir $(SERVERARGS)
 
@@ -319,6 +322,8 @@
 		> $(MANDIR)/man5/mandos.conf.5.gz
 	gzip --best --to-stdout mandos-clients.conf.5 \
 		> $(MANDIR)/man5/mandos-clients.conf.5.gz
+	gzip --best --to-stdout intro.8mandos \
+		> $(MANDIR)/man8/intro.8mandos.gz
 
 install-client-nokey: all doc
 	install --directory $(PREFIX)/lib/mandos $(CONFDIR)

=== modified file 'NEWS'
--- NEWS	2011-10-15 16:48:03 +0000
+++ NEWS	2012-06-17 22:26:40 +0000
@@ -1,6 +1,68 @@
 This NEWS file records noteworthy changes, very tersely.
 See the manual for detailed information.
 
+Version 1.6.0 (2012-06-18)
+* Server
+** Takes new --foreground option
+** Init script supports new "status" action.
+* Client
+** Now uses all interfaces by default; the --interface option can
+   still be used to restrict it, and the argument to --interface (as
+   well as the $DEVICE environment variable for the network hooks) is
+   now a comma-separated list of interfaces to use.
+
+Version 1.5.5 (2012-06-01)
+* Server
+** Server takes new --socket option
+
+Version 1.5.4 (2012-05-20)
+* Server
+** Bug fix: Regression fix: Make non-zero approval timeout values work.
+** Bug fix: Regression fix: Allow changing the Timeout D-Bus property.
+** Fall back to not bind to an interface if an invalid interface name
+   is given.
+** Removed support for undocumented feature of using plain "%%s" in
+   "checker" client option.
+** Old D-Bus interface are now marked as deprecated.
+** mandos-monitor: Bug fix: show approval timers correctly.
+** mandos-ctl: Show "Extended Timeout" correctly, not as milliseconds.
+
+Version 1.5.3 (2012-01-15)
+* Server
+** Add D-Bus property se.recompile.Client.LastCheckerStatus and use it
+   in mandos-monitor.
+* Client
+** Fix bugs in the example "bridge" network hook.
+
+Version 1.5.2 (2012-01-08)
+* Server
+** Removed D-Bus signal se.recompile.Mandos.NewRequest() added in
+   1.5.0.  It was buggy and was of questionable utility.
+
+Version 1.5.1 (2012-01-01)
+* Server
+** Include intro(8mandos) manual page, missing since migration from
+   README file in version 1.4.0.
+
+Version 1.5.0 (2012-01-01)
+* Client
+** Network hooks.  The Mandos client can now run custom scripts to take
+   up a network interface before the client is run.  Three example
+   scripts are provided: "wireless", "openvpn", and "bridge".
+   To facilitate this, the client now prefers network interfaces which
+   are up (if any) over all other interfaces.
+* Server
+** Persistent state.  Client state is now saved between server
+   restarts.
+** clients.conf file can now contain "enabled" setting for clients.
+** Bug fix: Fix rare crash bug.
+** Bug fix: Send corrent D-Bus type in PropertyChanged for
+   "ApprovalDelay", "ApprovalDuration", "Timeout", and
+   "ExtendedTimeout".
+** mandos-ctl: Bare numbers as arguments are taken to be milliseconds.
+** Bug fix: mandos-ctl --secret option now works.
+** New D-Bus signal: se.recompile.Mandos.NewRequest(s).
+
 Version 1.4.1 (2011-10-15)
 * Server
 ** Make D-Bus properties settable again, and handle checkers

=== modified file 'README'
--- README	2011-10-05 16:00:56 +0000
+++ README	2012-01-01 20:45:53 +0000
@@ -1,7 +1,8 @@
 Please see: http://www.recompile.se/mandos/man/intro.8mandos
 
 This information previously in this file has been moved to the
-intro(8mandos) manual page.  Go to the above URL, or run this command:
+intro(8mandos) manual page.  Go to the above URL, or install the
+Mandos server and run this command:
 
 	man 8mandos intro
 

=== modified file 'TODO'
--- TODO	2011-12-25 00:43:51 +0000
+++ TODO	2013-06-23 15:30:34 +0000
@@ -2,15 +2,18 @@
 
 * [[http://www.undeadly.org/cgi?action=article&sid=20110530221728][OpenBSD]]
 
+* Testing
+** python-nemu
+
 * mandos-applet
 
 * mandos-client
-** TODO [#A] Wireless network hook
 ** TODO [#B] Use capabilities instead of seteuid().
 ** TODO [#B] Use struct sockaddr_storage instead of a union
 ** TODO [#B] Use getaddrinfo(hints=AI_NUMERICHOST) instead of inet_pton()
 ** TODO [#B] Use getnameinfo(serv=NULL, NI_NUMERICHOST) instead of inet_ntop()
 ** TODO [#B] Prefer /run/tmp over /tmp, if it exists
+** TODO [#C] Make start_mandos_communication() take "struct server".
 
 * splashy
 ** TODO [#B] use scandir(3) instead of readdir(3)
@@ -18,7 +21,6 @@
 * usplash (Deprecated)
 ** TODO [#A] Make it work again
 ** TODO [#B] use scandir(3) instead of readdir(3)
-** TODO Use [[info:libc:Argz%20Functions][argz_extract]]
 
 * askpass-fifo
 ** TODO [#B] Drop privileges after opening FIFO.
@@ -32,30 +34,23 @@
 
 * plugin-runner
 ** TODO handle printing for errors for plugins
-*** Hook up stderr of plugins, buffer them, and prepend mandos pluig [plugin name]
+*** Hook up stderr of plugins, buffer them, and prepend "Mandos Plugin [plugin name]"
 ** TODO [#B] use scandir(3) instead of readdir(3)
 ** TODO [#C] use same file name rules as run-parts(8)
 ** kernel command line option for debug info
 ** TODO [#B] Use openat()
 
 * mandos (server)
-** TODO Document why we ignore sigint
 ** TODO [#B] Log level							  :BUGS:
 *** TODO /etc/mandos/clients.d/*.conf
     Watch this directory and add/remove/update clients?
 ** TODO [#C] config for TXT record
 ** TODO Log level dbus option
    SetLogLevel D-Bus call
-** TODO Implement --foreground						  :BUGS:
-   [[info:standards:Option%20Table][Table of Long Options]]
-** TODO Implement --socket
-   [[info:standards:Option%20Table][Table of Long Options]]
 ** TODO [#C] DBusServiceObjectUsingSuper
 ** TODO [#B] Global enable/disable flag
 ** TODO [#B] By-client countdown on number of secrets given
-** TODO [#B] Support RFC 3339 time duration syntax
-** More D-Bus methods
-*** NeedsPassword(50) - Timeout, default disapprove
+** D-Bus Client method NeedsPassword(50) - Timeout, default disapprove
     + SetPass(u"gazonk", True) -> Approval, persistent
     + Approve(False) -> Close client connection immediately
 ** TODO [#C] python-parsedatetime
@@ -69,6 +64,20 @@
 ** TODO Allow %%(checker)s as a runtime expansion
 ** TODO Use python-tlslite?
 ** TODO D-Bus AddClient() method on server object
+** TODO Use org.freedesktop.DBus.Method.NoReply annotation on async methods. :2:
+** TODO Emit [[http://dbus.freedesktop.org/doc/dbus-specification.html#standard-interfaces-properties][org.freedesktop.DBus.Properties.PropertiesChanged]] signal	  :2:
+   TODO Deprecate se.recompile.Mandos.Client.PropertyChanged - annotate!
+   TODO Can use "invalidates" annotation to also emit on changed secret.
+** TODO Support [[http://dbus.freedesktop.org/doc/dbus-specification.html#standard-interfaces-objectmanager][org.freedesktop.DBus.ObjectManager]] interface on server object :2:
+   Deprecate methods GetAllClients(), GetAllClientsWithProperties()
+   and signals ClientAdded and ClientRemoved.
+** TODO Save state periodically to recover better from hard shutdowns
+** TODO CheckerCompleted method, deprecate CheckedOK
+** TODO Secret Service API?
+   http://standards.freedesktop.org/secret-service/
+** TODO Remove D-Bus interfaces with old domain name			  :2:
+** TODO Remove old string_to_delta format				  :2:
+** TODO --no-zeroconf (only valid if port or socket is set)
 
 * mandos.xml
 ** Add mandos contact info in manual pages
@@ -76,7 +85,7 @@
 * mandos-ctl
 *** Handle "no D-Bus server" and/or "no Mandos server found" better
 *** [#B] --dump option
-** TODO Support RFC 3339 time duration syntax
+** TODO Remove old string_to_delta format				  :2:
 
 * TODO mandos-dispatch
   Listens for specified D-Bus signals and spawns shell commands with
@@ -87,9 +96,8 @@
 ** Urwid client data displayer
    Better view of client data in the listing
 *** Properties popup
-** Nicer crashes. Stack traces Messes up shell.
-*** Print a nice "We are sorry" message, save stack trace to log.
-** Show timeout countdown for approval
+** Print a nice "We are sorry" message, save stack trace to log.
+** Rename module "gobject" to "GObject".
 
 * mandos-keygen
 ** TODO "--secfile" option

=== modified file 'clients.conf'
--- clients.conf	2011-11-26 23:08:17 +0000
+++ clients.conf	2012-06-23 00:58:49 +0000
@@ -4,19 +4,19 @@
 
 # How long until a client is disabled and not be allowed to get the
 # data this server holds.
-;timeout = 5m
+;timeout = PT5M
 
 # How often to run the checker to confirm that a client is still up.
 # Note: a new checker will not be started if an old one is still
 # running.  The server will wait for a checker to complete until the
 # above "timeout" occurs, at which time the client will be disabled,
 # and any running checker killed.
-;interval = 2m
+;interval = PT2M
 
 # Extended timeout is an added timeout that is given once after a
 # password has been sent sucessfully to a client.  This allows for
 # additional delays caused by file system checks and quota checks.
-;extended_timeout = 15m
+;extended_timeout = PT15M
 
 # What command to run as "the checker".
 ;checker = fping -q -- %%(host)s
@@ -25,10 +25,10 @@
 ;approved_by_default = True
 
 # How long to wait for approval.
-;approval_delay = 0s
+;approval_delay = PT0S
 
 # How long one approval will last.
-;approval_duration = 1s
+;approval_duration = PT1S
 
 # Whether this client is enabled by default
 ;enabled = True
@@ -78,10 +78,10 @@
 ;host = 192.0.2.3
 ;
 ;# Parameters from the [DEFAULT] section can be overridden per client.
-;interval = 1m
+;interval = PT1M
 ;
 ;# This client requires manual approval before it receives its secret.
 ;approved_by_default = False
 ;# Require approval within 30 seconds.
-;approval_delay = 30s
+;approval_delay = PT30S
 ;####

=== modified file 'common.ent'
--- common.ent	2011-10-15 16:48:03 +0000
+++ common.ent	2012-06-17 22:26:40 +0000
@@ -1,3 +1,3 @@
 <?xml version="1.0" encoding="UTF-8"?>
 <!-- Edited by Makefile -->
-<!ENTITY version "1.4.1">
+<!ENTITY version "1.6.0">

=== modified file 'debian/changelog'
--- debian/changelog	2011-10-15 16:48:03 +0000
+++ debian/changelog	2012-06-17 22:26:40 +0000
@@ -1,3 +1,84 @@
+mandos (1.6.0-1) unstable; urgency=low
+
+  * New upstream release.
+  * debian/copyright (Copyright): Join the two lines to a single line.
+  * debian/mandos-client.README.Debian: Update to refer to the new
+    location of the example network hooks, and the new feature of using
+    all network interfaces.
+  * debian/mandos-client.docs (network-hooks.d): Removed.
+  * debian/mandos-client.examples (network-hooks.d): New.
+  * debian/rules (binary-common): Added "dh_installexamples".
+    (binary-common/dh_fixperms): Exclude new location of
+    "network-hooks.d".
+
+ -- Teddy Hogeborn <teddy@recompile.se>  Mon, 18 Jun 2012 00:15:23 +0200
+
+mandos (1.5.5-1) unstable; urgency=low
+
+  * New upstream release.
+  * debian/copyright (Format): Updated to
+    "http://www.debian.org/doc/packaging-manuals/copyright-format/1.0/".
+  * debian/control (Build-Depends): Removed "man, locales-all".
+
+ -- Teddy Hogeborn <teddy@recompile.se>  Fri, 01 Jun 2012 20:30:41 +0200
+
+mandos (1.5.4-1) unstable; urgency=low
+
+  * New upstream release.
+
+ -- Teddy Hogeborn <teddy@recompile.se>  Sun, 20 May 2012 15:38:34 +0200
+
+mandos (1.5.3-1.2) unstable; urgency=low
+
+  * Non-maintainer upload.
+  * Set Architecture to linux-any.  (Closes: #647670)
+
+ -- Robert Millan <rmh@debian.org>  Sun, 22 Apr 2012 16:22:01 +0200
+
+mandos (1.5.3-1.1) unstable; urgency=low
+
+  * Non-maintainer upload.
+  * Fix "mandos FTBFS on buildds": add build-dependency on locales-all and
+    pass LC_ALL to dh_auto_build to make sure we have and use the en_US.UTF-8
+    locale for manpage creation.
+    (Closes: #656178)
+
+ -- gregor herrmann <gregoa@debian.org>  Tue, 31 Jan 2012 17:56:05 +0100
+
+mandos (1.5.3-1) unstable; urgency=low
+
+  * New upstream release.
+
+ -- Teddy Hogeborn <teddy@recompile.se>  Sun, 15 Jan 2012 22:05:54 +0100
+
+mandos (1.5.2-1) unstable; urgency=low
+
+  * New upstream release.
+
+ -- Teddy Hogeborn <teddy@recompile.se>  Sun, 08 Jan 2012 11:17:20 +0100
+
+mandos (1.5.1-1) unstable; urgency=low
+
+  * New upstream release.
+
+ -- Teddy Hogeborn <teddy@recompile.se>  Sun, 01 Jan 2012 21:53:31 +0100
+
+mandos (1.5.0-1) unstable; urgency=low
+
+  * New upstream release.
+  * debian/control (mandos-client/Depends): Added "initramfs-tools".
+  * debian/mandos-client.README.Debian: Corrected mail address and adjust
+                                        wording.
+  * debian/rules (binary-common): Exclude new nework-hooks.d directory
+                                  from dh_fixperms.
+  * debian/mandos-client.README.Debian: Document network hook facility.
+  * debian/mandos-client.docs (network-hooks.d): Added.
+  * debian/mandos.dirs (var/lib/mandos): Added.
+  * debian/mandos.postinst: Fix ownership of /var/lib/mandos.
+  * debian/control (mandos/Depends): Added "python-gnupginterface".
+
+ -- Teddy Hogeborn <teddy@recompile.se>  Sun, 01 Jan 2012 05:58:11 +0100
+
 mandos (1.4.1-1) unstable; urgency=low
 
   * New upstream release.

=== modified file 'debian/control'
--- debian/control	2011-12-21 17:09:12 +0000
+++ debian/control	2013-10-05 19:34:40 +0000
@@ -4,10 +4,10 @@
 Maintainer: Mandos Maintainers <mandos@recompile.se>
 Uploaders: Teddy Hogeborn <teddy@recompile.se>,
 	   Björn Påhlsson <belorn@recompile.se>
-Build-Depends: debhelper (>= 7), docbook-xml, docbook-xsl,
+Build-Depends: debhelper (>= 8.9.7), docbook-xml, docbook-xsl,
 	libavahi-core-dev, libgpgme11-dev, libgnutls-dev, xsltproc,
-	pkg-config, man
-Standards-Version: 3.9.2
+	pkg-config
+Standards-Version: 3.9.3
 Vcs-Bzr: http://ftp.recompile.se/pub/mandos/trunk
 Vcs-Browser: http://bzr.recompile.se/loggerhead/mandos/trunk/files
 Homepage: http://www.recompile.se/mandos
@@ -17,8 +17,7 @@
 Architecture: all
 Depends: ${misc:Depends}, python (>=2.6), python-gnutls, python-dbus,
 	python-avahi, python-gobject, avahi-daemon, adduser,
-	python-urwid, python (>=2.7) | python-argparse,
-	python-gnupginterface
+	python-urwid, python (>=2.7) | python-argparse, gnupg (<< 2)
 Recommends: fping
 Description: server giving encrypted passwords to Mandos clients
  This is the server part of the Mandos system, which allows
@@ -36,7 +35,7 @@
  whereupon the computers can continue booting normally.
 
 Package: mandos-client
-Architecture: any
+Architecture: linux-any
 Depends: ${shlibs:Depends}, ${misc:Depends}, adduser, cryptsetup,
 	gnupg (<< 2), initramfs-tools
 Breaks: dropbear (<= 0.53.1-1)

=== modified file 'debian/copyright'
--- debian/copyright	2011-10-10 20:29:58 +0000
+++ debian/copyright	2012-06-01 21:48:12 +0000
@@ -1,11 +1,11 @@
-Format: http://anonscm.debian.org/viewvc/dep/web/deps/dep5.mdwn?revision=202
+Format: http://www.debian.org/doc/packaging-manuals/copyright-format/1.0/
 Upstream-Name: Mandos
 Upstream-Contact: Mandos <mandos@recompile.se>
 Source: <http://www.recompile.se/mandos>
 
 Files: *
-Copyright: Copyright © 2008-2011 Teddy Hogeborn
-Copyright: Copyright © 2008-2011 Björn Påhlsson
+Copyright: Copyright © 2008-2012 Teddy Hogeborn
+	   Copyright © 2008-2012 Björn Påhlsson
 License: GPL-3+
     This program is free software: you can redistribute it and/or
     modify it under the terms of the GNU General Public License as

=== modified file 'debian/mandos-client.README.Debian'
--- debian/mandos-client.README.Debian	2011-11-28 22:15:04 +0000
+++ debian/mandos-client.README.Debian	2013-09-29 15:52:19 +0000
@@ -32,30 +32,31 @@
 
 * Specifying a Client Network Interface
   
-  At boot time the network interface to use will by default be
-  automatically detected.  If should result in an incorrect interface,
-  edit the DEVICE setting in the "/etc/initramfs-tools/initramfs.conf"
-  file.  (The default setting is empty, meaning to autodetect the
-  interface.)  *If* the DEVICE setting is changed, it will be
-  necessary to update the initrd image by running the command
+  At boot time the network interfaces to use will by default be
+  automatically detected.  If this should result in incorrect
+  interfaces, edit the DEVICE setting in the
+  "/etc/initramfs-tools/initramfs.conf" file.  (The default setting is
+  empty, meaning it will autodetect the interface.)  *If* the DEVICE
+  setting is changed, it will be necessary to update the initrd image
+  by running the command
   
 	update-initramfs -k all -u
   
-  The device can be overridden at boot time on the Linux kernel
+  The device can also be overridden at boot time on the Linux kernel
   command line using the sixth colon-separated field of the "ip="
   option; for exact syntax, read the documentation in the file
-  "/usr/share/doc/linux-doc-*/Documentation/filesystems/nfsroot.txt",
+  "/usr/share/doc/linux-doc-*/Documentation/filesystems/nfs/nfsroot.txt",
   available in the "linux-doc-*" package.
   
-  Note that since this network interface is used in the initial RAM
-  disk environment, the network interface *must* exist at that stage.
-  Thus, the interface can *not* be a pseudo-interface such as "br0" or
-  "tun0"; instead, only real interface (such as "eth0") can be used.
+  Note that since the network interfaces are used in the initial RAM
+  disk environment, the network interfaces *must* exist at that stage.
+  Thus, an interface can *not* be a pseudo-interface such as "br0" or
+  "tun0"; instead, only real interfaces (such as "eth0") can be used.
   This can be overcome by writing a "network hook" program to create
-  the interface (see mandos-client(8mandos)) and placing it in
+  an interface (see mandos-client(8mandos)) and placing it in
   "/etc/mandos/network-hooks.d", from where it will be copied into the
   initial RAM disk.  Example network hook scripts can be found in
-  "/usr/share/doc/mandos-client/network-hooks.d".
+  "/usr/share/doc/mandos-client/examples/network-hooks.d".
 
 * User-Supplied Plugins
   
@@ -89,4 +90,4 @@
   work, "--options-for=mandos-client:--connect=<ADDRESS>:<PORT>" needs
   to be manually added to the file "/etc/mandos/plugin-runner.conf".
 
- -- Teddy Hogeborn <teddy@recompile.se>, Mon, 28 Nov 2011 23:07:22 +0100
+ -- Teddy Hogeborn <teddy@recompile.se>, Sun, 23 Jun 2013 17:31:53 +0200

=== modified file 'debian/mandos-client.docs'
--- debian/mandos-client.docs	2011-11-27 02:32:20 +0000
+++ debian/mandos-client.docs	2012-06-01 21:48:12 +0000
@@ -1,4 +1,3 @@
 NEWS
 README
 TODO
-network-hooks.d

=== added file 'debian/mandos-client.examples'
--- debian/mandos-client.examples	1970-01-01 00:00:00 +0000
+++ debian/mandos-client.examples	2012-06-01 21:48:12 +0000
@@ -0,0 +1,1 @@
+network-hooks.d

=== modified file 'debian/rules'
--- debian/rules	2011-11-27 14:44:28 +0000
+++ debian/rules	2013-10-05 19:34:40 +0000
@@ -1,107 +1,29 @@
 #!/usr/bin/make -f
-# Sample debian/rules that uses debhelper.
-#
-# This file was originally written by Joey Hess and Craig Small.
-# As a special exception, when this file is copied by dh-make into a
-# dh-make output file, you may use that output file without restriction.
-# This special exception was added by Craig Small in version 0.37 of dh-make.
-#
-# Modified to make a template file for a multi-binary package with separated
-# build-arch and build-indep targets  by Bill Allombert 2001
-
-# Uncomment this to turn on verbose mode.
-#export DH_VERBOSE=1
-
-# This has to be exported to make some magic below work.
-export DH_OPTIONS
-
-# -pie was broken briefly on the mips and mipsel architectures, see
-# <http://bugs.debian.org/526961>
-BINUTILS_V := $(shell dpkg-query --showformat='$${Version}' \
-			--show binutils)
-ifeq (yes,$(shell dpkg --compare-versions $(BINUTILS_V) lt 2.20-3 \
-		&& dpkg --compare-versions $(BINUTILS_V) ge 2.19.1-1 \
-		&& echo yes))
-  ifneq (,$(strip $(findstring :$(DEB_HOST_ARCH):,:mips:mipsel:) \
-		$(findstring :$(DEB_BUILD_ARCH):,:mips:mipsel:)))
-    BROKEN_PIE := yes
-    export BROKEN_PIE
-  endif
-endif
-
-configure: configure-stamp
-configure-stamp:
-	dh_testdir
-	touch configure-stamp
-
-build: build-arch build-indep
-
-build-arch: build-arch-stamp
-build-arch-stamp: configure-stamp
-	dh_auto_build -- all doc
-	touch $@
-
-build-indep: build-indep-stamp
-build-indep-stamp: configure-stamp
-	dh_auto_build -- doc
-	touch $@
-
-clean:
-	dh_testdir
-	dh_testroot
-	rm -f build-arch-stamp build-indep-stamp configure-stamp
-	dh_auto_clean
-	dh_clean
-
-install: install-indep install-arch
-install-indep:
-	dh_testdir
-	dh_testroot
-	dh_prep
-	dh_installdirs --indep
-	$(MAKE) DESTDIR=$(CURDIR)/debian/mandos install-server
-	dh_lintian
+%:
+	dh $@
+
+override_dh_auto_build-arch:
+	LC_ALL=en_US.utf8 dh_auto_build -- all doc
+
+override_dh_auto_build-indep:
+	LC_ALL=en_US.utf8 dh_auto_build -- doc
+
+override_dh_installinit-indep:
 	dh_installinit --onlyscripts \
 		--update-rcd-params="defaults 25 15"
-	dh_install --indep
-
-install-arch:
-	dh_testdir
-	dh_testroot
-	dh_prep
-	dh_installdirs --same-arch
-	$(MAKE) DESTDIR=$(CURDIR)/debian/mandos-client install-client-nokey
-	dh_lintian
-	dh_install --same-arch
-
-binary-common:
-	dh_testdir
-	dh_testroot
-	dh_installchangelogs
-	dh_installdocs
-	dh_link
-	dh_strip
-	dh_compress
+
+override_dh_auto_install-indep:
+	$(MAKE) DESTDIR=$(CURDIR)/debian/mandos install-server
+
+override_dh_auto_install-arch:
+	$(MAKE) DESTDIR=$(CURDIR)/debian/mandos-client \
+		install-client-nokey
+
+override_dh_fixperms:
 	dh_fixperms --exclude etc/keys/mandos \
 		--exclude etc/mandos/clients.conf \
 		--exclude etc/mandos/plugins.d \
 		--exclude usr/lib/mandos/plugins.d \
-		--exclude /usr/share/doc/mandos-client/network-hooks.d
-	dh_installdeb
-	dh_shlibdeps
-	dh_gencontrol
-	dh_md5sums
-	dh_builddeb
-
-# Build architecture independant packages using the common target.
-binary-indep: build-indep install-indep
-	$(MAKE) -f debian/rules DH_OPTIONS=--indep binary-common
-
-# Build architecture dependant packages using the common target.
-binary-arch: build-arch install-arch
-	$(MAKE) -f debian/rules DH_OPTIONS=--same-arch binary-common
-
-binary: binary-arch binary-indep
-
-.PHONY: build clean binary-indep binary-arch binary install \
-		install-indep install-arch configure
+		--exclude usr/share/doc/mandos-client/examples/network-hooks.d
+	chmod --recursive g-w -- \
+	"$(CURDIR)/debian/mandos-client/usr/share/doc/mandos-client/examples/network-hooks.d"

=== modified file 'init.d-mandos'
--- init.d-mandos	2011-10-05 16:00:56 +0000
+++ init.d-mandos	2012-06-01 21:48:12 +0000
@@ -149,6 +149,9 @@
 		;;
 	esac
 	;;
+  status)
+	status_of_proc "$DAEMON" "$NAME" -p "$PIDFILE"
+	;;
   *)
 	#echo "Usage: $SCRIPTNAME {start|stop|restart|reload|force-reload}" >&2
 	echo "Usage: $SCRIPTNAME {start|stop|restart|force-reload}" >&2

=== modified file 'intro.xml'
--- intro.xml	2011-10-05 16:00:56 +0000
+++ intro.xml	2011-12-31 23:05:34 +0000
@@ -1,7 +1,7 @@
 <?xml version="1.0" encoding="UTF-8"?>
 <!DOCTYPE refentry PUBLIC "-//OASIS//DTD DocBook XML V4.5//EN"
 "http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd" [
-<!ENTITY TIMESTAMP "2011-10-03">
+<!ENTITY TIMESTAMP "2012-01-01">
 <!ENTITY % common SYSTEM "common.ent">
 %common;
 ]>
@@ -31,6 +31,7 @@
     </authorgroup>
     <copyright>
       <year>2011</year>
+      <year>2012</year>
       <holder>Teddy Hogeborn</holder>
       <holder>Björn Påhlsson</holder>
     </copyright>

=== modified file 'mandos'
--- mandos	2011-12-23 20:00:52 +0000
+++ mandos	2013-06-23 15:13:06 +0000
@@ -11,8 +11,8 @@
 # "AvahiService" class, and some lines in "main".
 # 
 # Everything else is
-# Copyright © 2008-2011 Teddy Hogeborn
-# Copyright © 2008-2011 Björn Påhlsson
+# Copyright © 2008-2012 Teddy Hogeborn
+# Copyright © 2008-2012 Björn Påhlsson
 # 
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -34,6 +34,8 @@
 from __future__ import (division, absolute_import, print_function,
                         unicode_literals)
 
+from future_builtins import *
+
 import SocketServer as socketserver
 import socket
 import argparse
@@ -65,6 +67,8 @@
 import types
 import binascii
 import tempfile
+import itertools
+import collections
 
 import dbus
 import dbus.service
@@ -75,7 +79,6 @@
 import ctypes.util
 import xml.dom.minidom
 import inspect
-import GnuPGInterface
 
 try:
     SO_BINDTODEVICE = socket.SO_BINDTODEVICE
@@ -85,7 +88,7 @@
     except ImportError:
         SO_BINDTODEVICE = None
 
-version = "1.4.1"
+version = "1.6.0"
 stored_state_file = "clients.pickle"
 
 logger = logging.getLogger()
@@ -136,18 +139,17 @@
 class PGPEngine(object):
     """A simple class for OpenPGP symmetric encryption & decryption"""
     def __init__(self):
-        self.gnupg = GnuPGInterface.GnuPG()
         self.tempdir = tempfile.mkdtemp(prefix="mandos-")
-        self.gnupg = GnuPGInterface.GnuPG()
-        self.gnupg.options.meta_interactive = False
-        self.gnupg.options.homedir = self.tempdir
-        self.gnupg.options.extra_args.extend(['--force-mdc',
-                                              '--quiet'])
+        self.gnupgargs = ['--batch',
+                          '--home', self.tempdir,
+                          '--force-mdc',
+                          '--quiet',
+                          '--no-use-agent']
     
     def __enter__(self):
         return self
     
-    def __exit__ (self, exc_type, exc_value, traceback):
+    def __exit__(self, exc_type, exc_value, traceback):
         self._cleanup()
         return False
     
@@ -173,41 +175,43 @@
         return b"mandos" + binascii.hexlify(password)
     
     def encrypt(self, data, password):
-        self.gnupg.passphrase = self.password_encode(password)
-        with open(os.devnull) as devnull:
-            try:
-                proc = self.gnupg.run(['--symmetric'],
-                                      create_fhs=['stdin', 'stdout'],
-                                      attach_fhs={'stderr': devnull})
-                with contextlib.closing(proc.handles['stdin']) as f:
-                    f.write(data)
-                with contextlib.closing(proc.handles['stdout']) as f:
-                    ciphertext = f.read()
-                proc.wait()
-            except IOError as e:
-                raise PGPError(e)
-        self.gnupg.passphrase = None
+        passphrase = self.password_encode(password)
+        with tempfile.NamedTemporaryFile(dir=self.tempdir
+                                         ) as passfile:
+            passfile.write(passphrase)
+            passfile.flush()
+            proc = subprocess.Popen(['gpg', '--symmetric',
+                                     '--passphrase-file',
+                                     passfile.name]
+                                    + self.gnupgargs,
+                                    stdin = subprocess.PIPE,
+                                    stdout = subprocess.PIPE,
+                                    stderr = subprocess.PIPE)
+            ciphertext, err = proc.communicate(input = data)
+        if proc.returncode != 0:
+            raise PGPError(err)
         return ciphertext
     
     def decrypt(self, data, password):
-        self.gnupg.passphrase = self.password_encode(password)
-        with open(os.devnull) as devnull:
-            try:
-                proc = self.gnupg.run(['--decrypt'],
-                                      create_fhs=['stdin', 'stdout'],
-                                      attach_fhs={'stderr': devnull})
-                with contextlib.closing(proc.handles['stdin'] ) as f:
-                    f.write(data)
-                with contextlib.closing(proc.handles['stdout']) as f:
-                    decrypted_plaintext = f.read()
-                proc.wait()
-            except IOError as e:
-                raise PGPError(e)
-        self.gnupg.passphrase = None
+        passphrase = self.password_encode(password)
+        with tempfile.NamedTemporaryFile(dir = self.tempdir
+                                         ) as passfile:
+            passfile.write(passphrase)
+            passfile.flush()
+            proc = subprocess.Popen(['gpg', '--decrypt',
+                                     '--passphrase-file',
+                                     passfile.name]
+                                    + self.gnupgargs,
+                                    stdin = subprocess.PIPE,
+                                    stdout = subprocess.PIPE,
+                                    stderr = subprocess.PIPE)
+            decrypted_plaintext, err = proc.communicate(input
+                                                        = data)
+        if proc.returncode != 0:
+            raise PGPError(err)
         return decrypted_plaintext
 
 
-
 class AvahiError(Exception):
     def __init__(self, value, *args, **kwargs):
         self.value = value
@@ -230,7 +234,7 @@
                Used to optionally bind to the specified interface.
     name: string; Example: 'Mandos'
     type: string; Example: '_mandos._tcp'.
-                  See <http://www.dns-sd.org/ServiceTypes.html>
+     See <https://www.iana.org/assignments/service-names-port-numbers>
     port: integer; what port to announce
     TXT: list of strings; TXT record for the service
     domain: string; Domain to publish on, default to .local if empty.
@@ -242,6 +246,7 @@
     server: D-Bus Server
     bus: dbus.SystemBus()
     """
+    
     def __init__(self, interface = avahi.IF_UNSPEC, name = None,
                  servicetype = None, port = None, TXT = None,
                  domain = "", host = "", max_renames = 32768,
@@ -260,6 +265,7 @@
         self.server = None
         self.bus = bus
         self.entry_group_state_changed_match = None
+    
     def rename(self):
         """Derived from the Avahi example code"""
         if self.rename_count >= self.max_renames:
@@ -275,10 +281,11 @@
         try:
             self.add()
         except dbus.exceptions.DBusException as error:
-            logger.critical("DBusException: %s", error)
+            logger.critical("D-Bus Exception", exc_info=error)
             self.cleanup()
             os._exit(1)
         self.rename_count += 1
+    
     def remove(self):
         """Derived from the Avahi example code"""
         if self.entry_group_state_changed_match is not None:
@@ -286,6 +293,7 @@
             self.entry_group_state_changed_match = None
         if self.group is not None:
             self.group.Reset()
+    
     def add(self):
         """Derived from the Avahi example code"""
         self.remove()
@@ -308,6 +316,7 @@
             dbus.UInt16(self.port),
             avahi.string_array_to_txt_array(self.TXT))
         self.group.Commit()
+    
     def entry_group_state_changed(self, state, error):
         """Derived from the Avahi example code"""
         logger.debug("Avahi entry group state change: %i", state)
@@ -320,8 +329,9 @@
         elif state == avahi.ENTRY_GROUP_FAILURE:
             logger.critical("Avahi: Error in group state changed %s",
                             unicode(error))
-            raise AvahiGroupError("State changed: %s"
-                                  % unicode(error))
+            raise AvahiGroupError("State changed: {0!s}"
+                                  .format(error))
+    
     def cleanup(self):
         """Derived from the Avahi example code"""
         if self.group is not None:
@@ -332,6 +342,7 @@
                 pass
             self.group = None
         self.remove()
+    
     def server_state_changed(self, state, error=None):
         """Derived from the Avahi example code"""
         logger.debug("Avahi server state change: %i", state)
@@ -356,6 +367,7 @@
                 logger.debug("Unknown state: %r", state)
             else:
                 logger.debug("Unknown state: %r: %r", state, error)
+    
     def activate(self):
         """Derived from the Avahi example code"""
         if self.server is None:
@@ -368,22 +380,25 @@
                                  self.server_state_changed)
         self.server_state_changed(self.server.GetState())
 
+
 class AvahiServiceToSyslog(AvahiService):
     def rename(self):
         """Add the new name to the syslog messages"""
         ret = AvahiService.rename(self)
         syslogger.setFormatter(logging.Formatter
-                               ('Mandos (%s) [%%(process)d]:'
-                                ' %%(levelname)s: %%(message)s'
-                                % self.name))
+                               ('Mandos ({0}) [%(process)d]:'
+                                ' %(levelname)s: %(message)s'
+                                .format(self.name)))
         return ret
 
+
 def timedelta_to_milliseconds(td):
     "Convert a datetime.timedelta() to milliseconds"
     return ((td.days * 24 * 60 * 60 * 1000)
             + (td.seconds * 1000)
             + (td.microseconds // 1000))
-        
+
+
 class Client(object):
     """A representation of a client host served by this server.
     
@@ -414,30 +429,32 @@
     last_checked_ok: datetime.datetime(); (UTC) or None
     last_checker_status: integer between 0 and 255 reflecting exit
                          status of last checker. -1 reflects crashed
-                         checker, or None.
+                         checker, -2 means no checker completed yet.
     last_enabled: datetime.datetime(); (UTC) or None
     name:       string; from the config file, used in log messages and
                         D-Bus identifiers
     secret:     bytestring; sent verbatim (over TLS) to client
     timeout:    datetime.timedelta(); How long from last_checked_ok
                                       until this client is disabled
-    extended_timeout:   extra long timeout when password has been sent
+    extended_timeout:   extra long timeout when secret has been sent
     runtime_expansions: Allowed attributes for runtime expansion.
     expires:    datetime.datetime(); time (UTC) when a client will be
                 disabled, or None
+    server_settings: The server_settings dict from main()
     """
     
     runtime_expansions = ("approval_delay", "approval_duration",
-                          "created", "enabled", "fingerprint",
-                          "host", "interval", "last_checked_ok",
+                          "created", "enabled", "expires",
+                          "fingerprint", "host", "interval",
+                          "last_approval_request", "last_checked_ok",
                           "last_enabled", "name", "timeout")
-    client_defaults = { "timeout": "5m",
-                        "extended_timeout": "15m",
-                        "interval": "2m",
+    client_defaults = { "timeout": "PT5M",
+                        "extended_timeout": "PT15M",
+                        "interval": "PT2M",
                         "checker": "fping -q -- %%(host)s",
                         "host": "",
-                        "approval_delay": "0s",
-                        "approval_duration": "1s",
+                        "approval_delay": "PT0S",
+                        "approval_duration": "PT1S",
                         "approved_by_default": "True",
                         "enabled": "True",
                         }
@@ -456,12 +473,15 @@
     
     def approval_delay_milliseconds(self):
         return timedelta_to_milliseconds(self.approval_delay)
-
+    
     @staticmethod
     def config_parser(config):
-        """ Construct a new dict of client settings of this form:
+        """Construct a new dict of client settings of this form:
         { client_name: {setting_name: value, ...}, ...}
-        with exceptions for any special settings as defined above"""
+        with exceptions for any special settings as defined above.
+        NOTE: Must be a pure function. Must return the same result
+        value given the same arguments.
+        """
         settings = {}
         for client_name in config.sections():
             section = dict(config.items(client_name))
@@ -471,7 +491,8 @@
             # Reformat values from string types to Python types
             client["approved_by_default"] = config.getboolean(
                 client_name, "approved_by_default")
-            client["enabled"] = config.getboolean(client_name, "enabled")
+            client["enabled"] = config.getboolean(client_name,
+                                                  "enabled")
             
             client["fingerprint"] = (section["fingerprint"].upper()
                                      .replace(" ", ""))
@@ -483,8 +504,8 @@
                           "rb") as secfile:
                     client["secret"] = secfile.read()
             else:
-                raise TypeError("No secret or secfile for section %s"
-                                % section)
+                raise TypeError("No secret or secfile for section {0}"
+                                .format(section))
             client["timeout"] = string_to_delta(section["timeout"])
             client["extended_timeout"] = string_to_delta(
                 section["extended_timeout"])
@@ -496,34 +517,37 @@
             client["checker_command"] = section["checker"]
             client["last_approval_request"] = None
             client["last_checked_ok"] = None
-            client["last_checker_status"] = None
-            if client["enabled"]:
-                client["last_enabled"] = datetime.datetime.utcnow()
-                client["expires"] = (datetime.datetime.utcnow()
-                                     + client["timeout"])
-            else:
-                client["last_enabled"] = None
-                client["expires"] = None
-
+            client["last_checker_status"] = -2
+        
         return settings
-        
-        
-    def __init__(self, settings, name = None):
-        """Note: the 'checker' key in 'config' sets the
-        'checker_command' attribute and *not* the 'checker'
-        attribute."""
+    
+    def __init__(self, settings, name = None, server_settings=None):
         self.name = name
+        if server_settings is None:
+            server_settings = {}
+        self.server_settings = server_settings
         # adding all client settings
         for setting, value in settings.iteritems():
             setattr(self, setting, value)
         
+        if self.enabled:
+            if not hasattr(self, "last_enabled"):
+                self.last_enabled = datetime.datetime.utcnow()
+            if not hasattr(self, "expires"):
+                self.expires = (datetime.datetime.utcnow()
+                                + self.timeout)
+        else:
+            self.last_enabled = None
+            self.expires = None
+        
         logger.debug("Creating client %r", self.name)
         # Uppercase and remove spaces from fingerprint for later
         # comparison purposes with return value from the fingerprint()
         # function
         logger.debug("  Fingerprint: %s", self.fingerprint)
-        self.created = settings.get("created", datetime.datetime.utcnow())
-
+        self.created = settings.get("created",
+                                    datetime.datetime.utcnow())
+        
         # attributes specific for this server instance
         self.checker = None
         self.checker_initiator_tag = None
@@ -557,29 +581,29 @@
         if getattr(self, "enabled", False):
             # Already enabled
             return
-        self.send_changedstate()
         self.expires = datetime.datetime.utcnow() + self.timeout
         self.enabled = True
         self.last_enabled = datetime.datetime.utcnow()
         self.init_checker()
+        self.send_changedstate()
     
     def disable(self, quiet=True):
         """Disable this client."""
         if not getattr(self, "enabled", False):
             return False
         if not quiet:
-            self.send_changedstate()
-        if not quiet:
             logger.info("Disabling client %s", self.name)
-        if getattr(self, "disable_initiator_tag", False):
+        if getattr(self, "disable_initiator_tag", None) is not None:
             gobject.source_remove(self.disable_initiator_tag)
             self.disable_initiator_tag = None
         self.expires = None
-        if getattr(self, "checker_initiator_tag", False):
+        if getattr(self, "checker_initiator_tag", None) is not None:
             gobject.source_remove(self.checker_initiator_tag)
             self.checker_initiator_tag = None
         self.stop_checker()
         self.enabled = False
+        if not quiet:
+            self.send_changedstate()
         # Do not run this again if called by a gobject.timeout_add
         return False
     
@@ -589,10 +613,14 @@
     def init_checker(self):
         # Schedule a new checker to be started an 'interval' from now,
         # and every interval from then on.
+        if self.checker_initiator_tag is not None:
+            gobject.source_remove(self.checker_initiator_tag)
         self.checker_initiator_tag = (gobject.timeout_add
                                       (self.interval_milliseconds(),
                                        self.start_checker))
         # Schedule a disable() when 'timeout' has passed
+        if self.disable_initiator_tag is not None:
+            gobject.source_remove(self.disable_initiator_tag)
         self.disable_initiator_tag = (gobject.timeout_add
                                    (self.timeout_milliseconds(),
                                     self.disable))
@@ -617,17 +645,19 @@
             logger.warning("Checker for %(name)s crashed?",
                            vars(self))
     
-    def checked_ok(self, timeout=None):
-        """Bump up the timeout for this client.
-        
-        This should only be called when the client has been seen,
-        alive and well.
-        """
+    def checked_ok(self):
+        """Assert that the client has been seen, alive and well."""
+        self.last_checked_ok = datetime.datetime.utcnow()
+        self.last_checker_status = 0
+        self.bump_timeout()
+    
+    def bump_timeout(self, timeout=None):
+        """Bump up the timeout for this client."""
         if timeout is None:
             timeout = self.timeout
-        self.last_checked_ok = datetime.datetime.utcnow()
         if self.disable_initiator_tag is not None:
             gobject.source_remove(self.disable_initiator_tag)
+            self.disable_initiator_tag = None
         if getattr(self, "enabled", False):
             self.disable_initiator_tag = (gobject.timeout_add
                                           (timedelta_to_milliseconds
@@ -643,10 +673,10 @@
         If a checker already exists, leave it running and do
         nothing."""
         # The reason for not killing a running checker is that if we
-        # did that, then if a checker (for some reason) started
-        # running slowly and taking more than 'interval' time, the
-        # client would inevitably timeout, since no checker would get
-        # a chance to run to completion.  If we instead leave running
+        # did that, and if a checker (for some reason) started running
+        # slowly and taking more than 'interval' time, then the client
+        # would inevitably timeout, since no checker would get a
+        # chance to run to completion.  If we instead leave running
         # checkers alone, the checker would have to take more time
         # than 'timeout' for the client to be disabled, which is as it
         # should be.
@@ -666,25 +696,17 @@
                                       self.current_checker_command)
         # Start a new checker if needed
         if self.checker is None:
+            # Escape attributes for the shell
+            escaped_attrs = dict(
+                (attr, re.escape(unicode(getattr(self, attr))))
+                for attr in
+                self.runtime_expansions)
             try:
-                # In case checker_command has exactly one % operator
-                command = self.checker_command % self.host
-            except TypeError:
-                # Escape attributes for the shell
-                escaped_attrs = dict(
-                    (attr,
-                     re.escape(unicode(str(getattr(self, attr, "")),
-                                       errors=
-                                       'replace')))
-                    for attr in
-                    self.runtime_expansions)
-                
-                try:
-                    command = self.checker_command % escaped_attrs
-                except TypeError as error:
-                    logger.error('Could not format string "%s":'
-                                 ' %s', self.checker_command, error)
-                    return True # Try again later
+                command = self.checker_command % escaped_attrs
+            except TypeError as error:
+                logger.error('Could not format string "%s"',
+                             self.checker_command, exc_info=error)
+                return True # Try again later
             self.current_checker_command = command
             try:
                 logger.info("Starting checker %r for %s",
@@ -693,22 +715,40 @@
                 # in normal mode, that is already done by daemon(),
                 # and in debug mode we don't want to.  (Stdin is
                 # always replaced by /dev/null.)
+                # The exception is when not debugging but nevertheless
+                # running in the foreground; use the previously
+                # created wnull.
+                popen_args = {}
+                if (not self.server_settings["debug"]
+                    and self.server_settings["foreground"]):
+                    popen_args.update({"stdout": wnull,
+                                       "stderr": wnull })
                 self.checker = subprocess.Popen(command,
                                                 close_fds=True,
-                                                shell=True, cwd="/")
-                self.checker_callback_tag = (gobject.child_watch_add
-                                             (self.checker.pid,
-                                              self.checker_callback,
-                                              data=command))
-                # The checker may have completed before the gobject
-                # watch was added.  Check for this.
+                                                shell=True, cwd="/",
+                                                **popen_args)
+            except OSError as error:
+                logger.error("Failed to start subprocess",
+                             exc_info=error)
+                return True
+            self.checker_callback_tag = (gobject.child_watch_add
+                                         (self.checker.pid,
+                                          self.checker_callback,
+                                          data=command))
+            # The checker may have completed before the gobject
+            # watch was added.  Check for this.
+            try:
                 pid, status = os.waitpid(self.checker.pid, os.WNOHANG)
-                if pid:
-                    gobject.source_remove(self.checker_callback_tag)
-                    self.checker_callback(pid, status, command)
             except OSError as error:
-                logger.error("Failed to start subprocess: %s",
-                             error)
+                if error.errno == errno.ECHILD:
+                    # This should never happen
+                    logger.error("Child process vanished",
+                                 exc_info=error)
+                    return True
+                raise
+            if pid:
+                gobject.source_remove(self.checker_callback_tag)
+                self.checker_callback(pid, status, command)
         # Re-run this periodically if run by gobject.timeout_add
         return True
     
@@ -721,10 +761,10 @@
             return
         logger.debug("Stopping checker for %(name)s", vars(self))
         try:
-            os.kill(self.checker.pid, signal.SIGTERM)
+            self.checker.terminate()
             #time.sleep(0.5)
             #if self.checker.poll() is None:
-            #    os.kill(self.checker.pid, signal.SIGKILL)
+            #    self.checker.kill()
         except OSError as error:
             if error.errno != errno.ESRCH: # No such process
                 raise
@@ -747,7 +787,7 @@
     # "Set" method, so we fail early here:
     if byte_arrays and signature != "ay":
         raise ValueError("Byte arrays not supported for non-'ay'"
-                         " signature %r" % signature)
+                         " signature {0!r}".format(signature))
     def decorator(func):
         func._dbus_is_property = True
         func._dbus_interface = dbus_interface
@@ -761,6 +801,43 @@
     return decorator
 
 
+def dbus_interface_annotations(dbus_interface):
+    """Decorator for marking functions returning interface annotations
+    
+    Usage:
+    
+    @dbus_interface_annotations("org.example.Interface")
+    def _foo(self):  # Function name does not matter
+        return {"org.freedesktop.DBus.Deprecated": "true",
+                "org.freedesktop.DBus.Property.EmitsChangedSignal":
+                    "false"}
+    """
+    def decorator(func):
+        func._dbus_is_interface = True
+        func._dbus_interface = dbus_interface
+        func._dbus_name = dbus_interface
+        return func
+    return decorator
+
+
+def dbus_annotations(annotations):
+    """Decorator to annotate D-Bus methods, signals or properties
+    Usage:
+    
+    @dbus_service_property("org.example.Interface", signature="b",
+                           access="r")
+    @dbus_annotations({{"org.freedesktop.DBus.Deprecated": "true",
+                        "org.freedesktop.DBus.Property."
+                        "EmitsChangedSignal": "false"})
+    def Property_dbus_property(self):
+        return dbus.Boolean(False)
+    """
+    def decorator(func):
+        func._dbus_annotations = annotations
+        return func
+    return decorator
+
+
 class DBusPropertyException(dbus.exceptions.DBusException):
     """A base class for D-Bus property-related exceptions
     """
@@ -789,16 +866,25 @@
     """
     
     @staticmethod
-    def _is_dbus_property(obj):
-        return getattr(obj, "_dbus_is_property", False)
+    def _is_dbus_thing(thing):
+        """Returns a function testing if an attribute is a D-Bus thing
+        
+        If called like _is_dbus_thing("method") it returns a function
+        suitable for use as predicate to inspect.getmembers().
+        """
+        return lambda obj: getattr(obj, "_dbus_is_{0}".format(thing),
+                                   False)
     
-    def _get_all_dbus_properties(self):
+    def _get_all_dbus_things(self, thing):
         """Returns a generator of (name, attribute) pairs
         """
-        return ((prop.__get__(self)._dbus_name, prop.__get__(self))
+        return ((getattr(athing.__get__(self), "_dbus_name",
+                         name),
+                 athing.__get__(self))
                 for cls in self.__class__.__mro__
-                for name, prop in
-                inspect.getmembers(cls, self._is_dbus_property))
+                for name, athing in
+                inspect.getmembers(cls,
+                                   self._is_dbus_thing(thing)))
     
     def _get_dbus_property(self, interface_name, property_name):
         """Returns a bound method if one exists which is a D-Bus
@@ -806,7 +892,8 @@
         """
         for cls in  self.__class__.__mro__:
             for name, value in (inspect.getmembers
-                                (cls, self._is_dbus_property)):
+                                (cls,
+                                 self._is_dbus_thing("property"))):
                 if (value._dbus_name == property_name
                     and value._dbus_interface == interface_name):
                     return value.__get__(self)
@@ -841,8 +928,8 @@
             # signatures other than "ay".
             if prop._dbus_signature != "ay":
                 raise ValueError
-            value = dbus.ByteArray(''.join(unichr(byte)
-                                           for byte in value))
+            value = dbus.ByteArray(b''.join(chr(byte)
+                                            for byte in value))
         prop(value)
     
     @dbus.service.method(dbus.PROPERTIES_IFACE, in_signature="s",
@@ -854,7 +941,7 @@
         Note: Will not include properties with access="write".
         """
         properties = {}
-        for name, prop in self._get_all_dbus_properties():
+        for name, prop in self._get_all_dbus_things("property"):
             if (interface_name
                 and interface_name != prop._dbus_interface):
                 # Interface non-empty but did not match
@@ -875,7 +962,9 @@
                          path_keyword='object_path',
                          connection_keyword='connection')
     def Introspect(self, object_path, connection):
-        """Standard D-Bus method, overloaded to insert property tags.
+        """Overloading of standard D-Bus method.
+        
+        Inserts property tags and interface annotation tags.
         """
         xmlstring = dbus.service.Object.Introspect(self, object_path,
                                                    connection)
@@ -888,12 +977,44 @@
                 e.setAttribute("access", prop._dbus_access)
                 return e
             for if_tag in document.getElementsByTagName("interface"):
+                # Add property tags
                 for tag in (make_tag(document, name, prop)
                             for name, prop
-                            in self._get_all_dbus_properties()
+                            in self._get_all_dbus_things("property")
                             if prop._dbus_interface
                             == if_tag.getAttribute("name")):
                     if_tag.appendChild(tag)
+                # Add annotation tags
+                for typ in ("method", "signal", "property"):
+                    for tag in if_tag.getElementsByTagName(typ):
+                        annots = dict()
+                        for name, prop in (self.
+                                           _get_all_dbus_things(typ)):
+                            if (name == tag.getAttribute("name")
+                                and prop._dbus_interface
+                                == if_tag.getAttribute("name")):
+                                annots.update(getattr
+                                              (prop,
+                                               "_dbus_annotations",
+                                               {}))
+                        for name, value in annots.iteritems():
+                            ann_tag = document.createElement(
+                                "annotation")
+                            ann_tag.setAttribute("name", name)
+                            ann_tag.setAttribute("value", value)
+                            tag.appendChild(ann_tag)
+                # Add interface annotation tags
+                for annotation, value in dict(
+                    itertools.chain.from_iterable(
+                        annotations().iteritems()
+                        for name, annotations in
+                        self._get_all_dbus_things("interface")
+                        if name == if_tag.getAttribute("name")
+                        )).iteritems():
+                    ann_tag = document.createElement("annotation")
+                    ann_tag.setAttribute("name", annotation)
+                    ann_tag.setAttribute("value", value)
+                    if_tag.appendChild(ann_tag)
                 # Add the names to the return values for the
                 # "org.freedesktop.DBus.Properties" methods
                 if (if_tag.getAttribute("name")
@@ -914,11 +1035,11 @@
         except (AttributeError, xml.dom.DOMException,
                 xml.parsers.expat.ExpatError) as error:
             logger.error("Failed to override Introspection method",
-                         error)
+                         exc_info=error)
         return xmlstring
 
 
-def datetime_to_dbus (dt, variant_level=0):
+def datetime_to_dbus(dt, variant_level=0):
     """Convert a UTC datetime.datetime() to a D-Bus type."""
     if dt is None:
         return dbus.String("", variant_level = variant_level)
@@ -926,33 +1047,52 @@
                        variant_level=variant_level)
 
 
-class AlternateDBusNamesMetaclass(DBusObjectWithProperties
-                                  .__metaclass__):
-    """Applied to an empty subclass of a D-Bus object, this metaclass
-    will add additional D-Bus attributes matching a certain pattern.
+def alternate_dbus_interfaces(alt_interface_names, deprecate=True):
+    """A class decorator; applied to a subclass of
+    dbus.service.Object, it will add alternate D-Bus attributes with
+    interface names according to the "alt_interface_names" mapping.
+    Usage:
+    
+    @alternate_dbus_interfaces({"org.example.Interface":
+                                    "net.example.AlternateInterface"})
+    class SampleDBusObject(dbus.service.Object):
+        @dbus.service.method("org.example.Interface")
+        def SampleDBusMethod():
+            pass
+    
+    The above "SampleDBusMethod" on "SampleDBusObject" will be
+    reachable via two interfaces: "org.example.Interface" and
+    "net.example.AlternateInterface", the latter of which will have
+    its D-Bus annotation "org.freedesktop.DBus.Deprecated" set to
+    "true", unless "deprecate" is passed with a False value.
+    
+    This works for methods and signals, and also for D-Bus properties
+    (from DBusObjectWithProperties) and interfaces (from the
+    dbus_interface_annotations decorator).
     """
-    def __new__(mcs, name, bases, attr):
-        # Go through all the base classes which could have D-Bus
-        # methods, signals, or properties in them
-        for base in (b for b in bases
-                     if issubclass(b, dbus.service.Object)):
-            # Go though all attributes of the base class
-            for attrname, attribute in inspect.getmembers(base):
+    def wrapper(cls):
+        for orig_interface_name, alt_interface_name in (
+            alt_interface_names.iteritems()):
+            attr = {}
+            interface_names = set()
+            # Go though all attributes of the class
+            for attrname, attribute in inspect.getmembers(cls):
                 # Ignore non-D-Bus attributes, and D-Bus attributes
                 # with the wrong interface name
                 if (not hasattr(attribute, "_dbus_interface")
                     or not attribute._dbus_interface
-                    .startswith("se.recompile.Mandos")):
+                    .startswith(orig_interface_name)):
                     continue
                 # Create an alternate D-Bus interface name based on
                 # the current name
                 alt_interface = (attribute._dbus_interface
-                                 .replace("se.recompile.Mandos",
-                                          "se.bsnet.fukt.Mandos"))
+                                 .replace(orig_interface_name,
+                                          alt_interface_name))
+                interface_names.add(alt_interface)
                 # Is this a D-Bus signal?
                 if getattr(attribute, "_dbus_is_signal", False):
-                    # Extract the original non-method function by
-                    # black magic
+                    # Extract the original non-method undecorated
+                    # function by black magic
                     nonmethod_func = (dict(
                             zip(attribute.func_code.co_freevars,
                                 attribute.__closure__))["func"]
@@ -969,9 +1109,16 @@
                                 nonmethod_func.func_name,
                                 nonmethod_func.func_defaults,
                                 nonmethod_func.func_closure)))
+                    # Copy annotations, if any
+                    try:
+                        new_function._dbus_annotations = (
+                            dict(attribute._dbus_annotations))
+                    except AttributeError:
+                        pass
                     # Define a creator of a function to call both the
-                    # old and new functions, so both the old and new
-                    # signals gets sent when the function is called
+                    # original and alternate functions, so both the
+                    # original and alternate signals gets sent when
+                    # the function is called
                     def fixscope(func1, func2):
                         """This function is a scope container to pass
                         func1 and func2 to the "call_both" function
@@ -984,8 +1131,7 @@
                         return call_both
                     # Create the "call_both" function and add it to
                     # the class
-                    attr[attrname] = fixscope(attribute,
-                                              new_function)
+                    attr[attrname] = fixscope(attribute, new_function)
                 # Is this a D-Bus method?
                 elif getattr(attribute, "_dbus_is_method", False):
                     # Create a new, but exactly alike, function
@@ -1002,6 +1148,12 @@
                                         attribute.func_name,
                                         attribute.func_defaults,
                                         attribute.func_closure)))
+                    # Copy annotations, if any
+                    try:
+                        attr[attrname]._dbus_annotations = (
+                            dict(attribute._dbus_annotations))
+                    except AttributeError:
+                        pass
                 # Is this a D-Bus property?
                 elif getattr(attribute, "_dbus_is_property", False):
                     # Create a new, but exactly alike, function
@@ -1021,9 +1173,51 @@
                                         attribute.func_name,
                                         attribute.func_defaults,
                                         attribute.func_closure)))
-        return type.__new__(mcs, name, bases, attr)
-
-
+                    # Copy annotations, if any
+                    try:
+                        attr[attrname]._dbus_annotations = (
+                            dict(attribute._dbus_annotations))
+                    except AttributeError:
+                        pass
+                # Is this a D-Bus interface?
+                elif getattr(attribute, "_dbus_is_interface", False):
+                    # Create a new, but exactly alike, function
+                    # object.  Decorate it to be a new D-Bus interface
+                    # with the alternate D-Bus interface name.  Add it
+                    # to the class.
+                    attr[attrname] = (dbus_interface_annotations
+                                      (alt_interface)
+                                      (types.FunctionType
+                                       (attribute.func_code,
+                                        attribute.func_globals,
+                                        attribute.func_name,
+                                        attribute.func_defaults,
+                                        attribute.func_closure)))
+            if deprecate:
+                # Deprecate all alternate interfaces
+                iname="_AlternateDBusNames_interface_annotation{0}"
+                for interface_name in interface_names:
+                    @dbus_interface_annotations(interface_name)
+                    def func(self):
+                        return { "org.freedesktop.DBus.Deprecated":
+                                     "true" }
+                    # Find an unused name
+                    for aname in (iname.format(i)
+                                  for i in itertools.count()):
+                        if aname not in attr:
+                            attr[aname] = func
+                            break
+            if interface_names:
+                # Replace the class with a new subclass of it with
+                # methods, signals, etc. as created above.
+                cls = type(b"{0}Alternate".format(cls.__name__),
+                           (cls,), attr)
+        return cls
+    return wrapper
+
+
+@alternate_dbus_interfaces({"se.recompile.Mandos":
+                                "se.bsnet.fukt.Mandos"})
 class ClientDBus(Client, DBusObjectWithProperties):
     """A Client class using D-Bus
     
@@ -1040,9 +1234,6 @@
     def __init__(self, bus = None, *args, **kwargs):
         self.bus = bus
         Client.__init__(self, *args, **kwargs)
-        self._approvals_pending = 0
-        
-        self._approvals_pending = 0
         # Only now, when this client is initialized, can it show up on
         # the D-Bus
         client_object_name = unicode(self.name).translate(
@@ -1052,7 +1243,7 @@
                                  ("/clients/" + client_object_name))
         DBusObjectWithProperties.__init__(self, self.bus,
                                           self.dbus_object_path)
-        
+    
     def notifychangeproperty(transform_func,
                              dbus_name, type_func=lambda x: x,
                              variant_level=1):
@@ -1081,7 +1272,6 @@
         
         return property(lambda self: getattr(self, attrname), setter)
     
-    
     expires = notifychangeproperty(datetime_to_dbus, "Expires")
     approvals_pending = notifychangeproperty(dbus.Boolean,
                                              "ApprovalPending",
@@ -1094,6 +1284,8 @@
                                        checker is not None)
     last_checked_ok = notifychangeproperty(datetime_to_dbus,
                                            "LastCheckedOK")
+    last_checker_status = notifychangeproperty(dbus.Int16,
+                                               "LastCheckerStatus")
     last_approval_request = notifychangeproperty(
         datetime_to_dbus, "LastApprovalRequest")
     approved_by_default = notifychangeproperty(dbus.Boolean,
@@ -1167,16 +1359,22 @@
         return False
     
     def approve(self, value=True):
-        self.send_changedstate()
         self.approved = value
         gobject.timeout_add(timedelta_to_milliseconds
                             (self.approval_duration),
                             self._reset_approved)
-    
+        self.send_changedstate()
     
     ## D-Bus methods, signals & properties
     _interface = "se.recompile.Mandos.Client"
     
+    ## Interfaces
+    
+    @dbus_interface_annotations(_interface)
+    def _foo(self):
+        return { "org.freedesktop.DBus.Property.EmitsChangedSignal":
+                     "false"}
+    
     ## Signals
     
     # CheckerCompleted - signal
@@ -1218,14 +1416,6 @@
         "D-Bus signal"
         return self.need_approval()
     
-    # NeRwequest - signal
-    @dbus.service.signal(_interface, signature="s")
-    def NewRequest(self, ip):
-        """D-Bus signal
-        Is sent after a client request a password.
-        """
-        pass
-    
     ## Methods
     
     # Approve - method
@@ -1341,6 +1531,12 @@
             return
         return datetime_to_dbus(self.last_checked_ok)
     
+    # LastCheckerStatus - property
+    @dbus_service_property(_interface, signature="n",
+                           access="read")
+    def LastCheckerStatus_dbus_property(self):
+        return dbus.Int16(self.last_checker_status)
+    
     # Expires - property
     @dbus_service_property(_interface, signature="s", access="read")
     def Expires_dbus_property(self):
@@ -1357,27 +1553,24 @@
     def Timeout_dbus_property(self, value=None):
         if value is None:       # get
             return dbus.UInt64(self.timeout_milliseconds())
+        old_timeout = self.timeout
         self.timeout = datetime.timedelta(0, 0, 0, value)
-        if getattr(self, "disable_initiator_tag", None) is None:
-            return
-        # Reschedule timeout
-        gobject.source_remove(self.disable_initiator_tag)
-        self.disable_initiator_tag = None
-        self.expires = None
-        time_to_die = timedelta_to_milliseconds((self
-                                                 .last_checked_ok
-                                                 + self.timeout)
-                                                - datetime.datetime
-                                                .utcnow())
-        if time_to_die <= 0:
-            # The timeout has passed
-            self.disable()
-        else:
-            self.expires = (datetime.datetime.utcnow()
-                            + datetime.timedelta(milliseconds =
-                                                 time_to_die))
-            self.disable_initiator_tag = (gobject.timeout_add
-                                          (time_to_die, self.disable))
+        # Reschedule disabling
+        if self.enabled:
+            now = datetime.datetime.utcnow()
+            self.expires += self.timeout - old_timeout
+            if self.expires <= now:
+                # The timeout has passed
+                self.disable()
+            else:
+                if (getattr(self, "disable_initiator_tag", None)
+                    is None):
+                    return
+                gobject.source_remove(self.disable_initiator_tag)
+                self.disable_initiator_tag = (
+                    gobject.timeout_add(
+                        timedelta_to_milliseconds(self.expires - now),
+                        self.disable))
     
     # ExtendedTimeout - property
     @dbus_service_property(_interface, signature="t",
@@ -1462,10 +1655,6 @@
         self._pipe.send(('setattr', name, value))
 
 
-class ClientDBusTransitional(ClientDBus):
-    __metaclass__ = AlternateDBusNamesMetaclass
-
-
 class ClientHandler(socketserver.BaseRequestHandler, object):
     """A class to handle client connections.
     
@@ -1539,10 +1728,6 @@
                 except KeyError:
                     return
                 
-                if self.server.use_dbus:
-                    # Emit D-Bus signal
-                    client.NewRequest(str(self.client_address))
-                
                 if client.approval_delay:
                     delay = client.approval_delay
                     client.approvals_pending += 1
@@ -1579,9 +1764,9 @@
                     #wait until timeout or approved
                     time = datetime.datetime.now()
                     client.changedstate.acquire()
-                    (client.changedstate.wait
-                     (float(client.timedelta_to_milliseconds(delay)
-                            / 1000)))
+                    client.changedstate.wait(
+                        float(timedelta_to_milliseconds(delay)
+                              / 1000))
                     client.changedstate.release()
                     time2 = datetime.datetime.now()
                     if (time2 - time) >= delay:
@@ -1603,7 +1788,8 @@
                     try:
                         sent = session.send(client.secret[sent_size:])
                     except gnutls.errors.GNUTLSError as error:
-                        logger.warning("gnutls send failed")
+                        logger.warning("gnutls send failed",
+                                       exc_info=error)
                         return
                     logger.debug("Sent: %d, remaining: %d",
                                  sent, len(client.secret)
@@ -1612,7 +1798,7 @@
                 
                 logger.info("Sending secret to %s", client.name)
                 # bump the timeout using extended_timeout
-                client.checked_ok(client.extended_timeout)
+                client.bump_timeout(client.extended_timeout)
                 if self.server.use_dbus:
                     # Emit D-Bus signal
                     client.GotSecret()
@@ -1623,7 +1809,8 @@
                 try:
                     session.bye()
                 except gnutls.errors.GNUTLSError as error:
-                    logger.warning("GnuTLS bye failed")
+                    logger.warning("GnuTLS bye failed",
+                                   exc_info=error)
     
     @staticmethod
     def peer_certificate(session):
@@ -1701,8 +1888,7 @@
     def process_request(self, request, address):
         """Start a new process to process the request."""
         proc = multiprocessing.Process(target = self.sub_process_main,
-                                       args = (request,
-                                               address))
+                                       args = (request, address))
         proc.start()
         return proc
 
@@ -1736,12 +1922,42 @@
         use_ipv6:       Boolean; to use IPv6 or not
     """
     def __init__(self, server_address, RequestHandlerClass,
-                 interface=None, use_ipv6=True):
+                 interface=None, use_ipv6=True, socketfd=None):
+        """If socketfd is set, use that file descriptor instead of
+        creating a new one with socket.socket().
+        """
         self.interface = interface
         if use_ipv6:
             self.address_family = socket.AF_INET6
+        if socketfd is not None:
+            # Save the file descriptor
+            self.socketfd = socketfd
+            # Save the original socket.socket() function
+            self.socket_socket = socket.socket
+            # To implement --socket, we monkey patch socket.socket.
+            # 
+            # (When socketserver.TCPServer is a new-style class, we
+            # could make self.socket into a property instead of monkey
+            # patching socket.socket.)
+            # 
+            # Create a one-time-only replacement for socket.socket()
+            @functools.wraps(socket.socket)
+            def socket_wrapper(*args, **kwargs):
+                # Restore original function so subsequent calls are
+                # not affected.
+                socket.socket = self.socket_socket
+                del self.socket_socket
+                # This time only, return a new socket object from the
+                # saved file descriptor.
+                return socket.fromfd(self.socketfd, *args, **kwargs)
+            # Replace socket.socket() function with wrapper
+            socket.socket = socket_wrapper
+        # The socketserver.TCPServer.__init__ will call
+        # socket.socket(), which might be our replacement,
+        # socket_wrapper(), if socketfd was set.
         socketserver.TCPServer.__init__(self, server_address,
                                         RequestHandlerClass)
+    
     def server_bind(self):
         """This overrides the normal server_bind() function
         to bind to an interface if one was specified, and also NOT to
@@ -1755,17 +1971,18 @@
                 try:
                     self.socket.setsockopt(socket.SOL_SOCKET,
                                            SO_BINDTODEVICE,
-                                           str(self.interface
-                                               + '\0'))
+                                           str(self.interface + '\0'))
                 except socket.error as error:
-                    if error[0] == errno.EPERM:
-                        logger.error("No permission to"
-                                     " bind to interface %s",
-                                     self.interface)
-                    elif error[0] == errno.ENOPROTOOPT:
+                    if error.errno == errno.EPERM:
+                        logger.error("No permission to bind to"
+                                     " interface %s", self.interface)
+                    elif error.errno == errno.ENOPROTOOPT:
                         logger.error("SO_BINDTODEVICE not available;"
                                      " cannot bind to interface %s",
                                      self.interface)
+                    elif error.errno == errno.ENODEV:
+                        logger.error("Interface %s does not exist,"
+                                     " cannot bind", self.interface)
                     else:
                         raise
         # Only bind(2) the socket if we really need to.
@@ -1774,7 +1991,7 @@
                 if self.address_family == socket.AF_INET6:
                     any_address = "::" # in6addr_any
                 else:
-                    any_address = socket.INADDR_ANY
+                    any_address = "0.0.0.0" # INADDR_ANY
                 self.server_address = (any_address,
                                        self.server_address[1])
             elif not self.server_address[1]:
@@ -1801,7 +2018,7 @@
     """
     def __init__(self, server_address, RequestHandlerClass,
                  interface=None, use_ipv6=True, clients=None,
-                 gnutls_priority=None, use_dbus=True):
+                 gnutls_priority=None, use_dbus=True, socketfd=None):
         self.enabled = False
         self.clients = clients
         if self.clients is None:
@@ -1811,7 +2028,8 @@
         IPv6_TCPServer.__init__(self, server_address,
                                 RequestHandlerClass,
                                 interface = interface,
-                                use_ipv6 = use_ipv6)
+                                use_ipv6 = use_ipv6,
+                                socketfd = socketfd)
     def server_activate(self):
         if self.enabled:
             return socketserver.TCPServer.server_activate(self)
@@ -1830,22 +2048,8 @@
     
     def handle_ipc(self, source, condition, parent_pipe=None,
                    proc = None, client_object=None):
-        condition_names = {
-            gobject.IO_IN: "IN",   # There is data to read.
-            gobject.IO_OUT: "OUT", # Data can be written (without
-                                    # blocking).
-            gobject.IO_PRI: "PRI", # There is urgent data to read.
-            gobject.IO_ERR: "ERR", # Error condition.
-            gobject.IO_HUP: "HUP"  # Hung up (the connection has been
-                                    # broken, usually for pipes and
-                                    # sockets).
-            }
-        conditions_string = ' | '.join(name
-                                       for cond, name in
-                                       condition_names.iteritems()
-                                       if cond & condition)
         # error, or the other end of multiprocessing.Pipe has closed
-        if condition & (gobject.IO_ERR | condition & gobject.IO_HUP):
+        if condition & (gobject.IO_ERR | gobject.IO_HUP):
             # Wait for other process to exit
             proc.join()
             return False
@@ -1909,6 +2113,105 @@
         return True
 
 
+def rfc3339_duration_to_delta(duration):
+    """Parse an RFC 3339 "duration" and return a datetime.timedelta
+    
+    >>> rfc3339_duration_to_delta("P7D")
+    datetime.timedelta(7)
+    >>> rfc3339_duration_to_delta("PT60S")
+    datetime.timedelta(0, 60)
+    >>> rfc3339_duration_to_delta("PT60M")
+    datetime.timedelta(0, 3600)
+    >>> rfc3339_duration_to_delta("PT24H")
+    datetime.timedelta(1)
+    >>> rfc3339_duration_to_delta("P1W")
+    datetime.timedelta(7)
+    >>> rfc3339_duration_to_delta("PT5M30S")
+    datetime.timedelta(0, 330)
+    >>> rfc3339_duration_to_delta("P1DT3M20S")
+    datetime.timedelta(1, 200)
+    """
+    
+    # Parsing an RFC 3339 duration with regular expressions is not
+    # possible - there would have to be multiple places for the same
+    # values, like seconds.  The current code, while more esoteric, is
+    # cleaner without depending on a parsing library.  If Python had a
+    # built-in library for parsing we would use it, but we'd like to
+    # avoid excessive use of external libraries.
+    
+    # New type for defining tokens, syntax, and semantics all-in-one
+    Token = collections.namedtuple("Token",
+                                   ("regexp", # To match token; if
+                                              # "value" is not None,
+                                              # must have a "group"
+                                              # containing digits
+                                    "value",  # datetime.timedelta or
+                                              # None
+                                    "followers")) # Tokens valid after
+                                                  # this token
+    # RFC 3339 "duration" tokens, syntax, and semantics; taken from
+    # the "duration" ABNF definition in RFC 3339, Appendix A.
+    token_end = Token(re.compile(r"$"), None, frozenset())
+    token_second = Token(re.compile(r"(\d+)S"),
+                         datetime.timedelta(seconds=1),
+                         frozenset((token_end,)))
+    token_minute = Token(re.compile(r"(\d+)M"),
+                         datetime.timedelta(minutes=1),
+                         frozenset((token_second, token_end)))
+    token_hour = Token(re.compile(r"(\d+)H"),
+                       datetime.timedelta(hours=1),
+                       frozenset((token_minute, token_end)))
+    token_time = Token(re.compile(r"T"),
+                       None,
+                       frozenset((token_hour, token_minute,
+                                  token_second)))
+    token_day = Token(re.compile(r"(\d+)D"),
+                      datetime.timedelta(days=1),
+                      frozenset((token_time, token_end)))
+    token_month = Token(re.compile(r"(\d+)M"),
+                        datetime.timedelta(weeks=4),
+                        frozenset((token_day, token_end)))
+    token_year = Token(re.compile(r"(\d+)Y"),
+                       datetime.timedelta(weeks=52),
+                       frozenset((token_month, token_end)))
+    token_week = Token(re.compile(r"(\d+)W"),
+                       datetime.timedelta(weeks=1),
+                       frozenset((token_end,)))
+    token_duration = Token(re.compile(r"P"), None,
+                           frozenset((token_year, token_month,
+                                      token_day, token_time,
+                                      token_week))),
+    # Define starting values
+    value = datetime.timedelta() # Value so far
+    found_token = None
+    followers = frozenset(token_duration,) # Following valid tokens
+    s = duration                # String left to parse
+    # Loop until end token is found
+    while found_token is not token_end:
+        # Search for any currently valid tokens
+        for token in followers:
+            match = token.regexp.match(s)
+            if match is not None:
+                # Token found
+                if token.value is not None:
+                    # Value found, parse digits
+                    factor = int(match.group(1), 10)
+                    # Add to value so far
+                    value += factor * token.value
+                # Strip token from string
+                s = token.regexp.sub("", s, 1)
+                # Go to found token
+                found_token = token
+                # Set valid next tokens
+                followers = found_token.followers
+                break
+        else:
+            # No currently valid tokens were found
+            raise ValueError("Invalid RFC 3339 duration")
+    # End token found
+    return value
+
+
 def string_to_delta(interval):
     """Parse a string and return a datetime.timedelta
     
@@ -1925,6 +2228,12 @@
     >>> string_to_delta('5m 30s')
     datetime.timedelta(0, 330)
     """
+    
+    try:
+        return rfc3339_duration_to_delta(interval)
+    except ValueError:
+        pass
+    
     timevalue = datetime.timedelta(0)
     for s in interval.split():
         try:
@@ -1941,7 +2250,8 @@
             elif suffix == "w":
                 delta = datetime.timedelta(0, 0, 0, 0, 0, 0, value)
             else:
-                raise ValueError("Unknown suffix %r" % suffix)
+                raise ValueError("Unknown suffix {0!r}"
+                                 .format(suffix))
         except (ValueError, IndexError) as e:
             raise ValueError(*(e.args))
         timevalue += delta
@@ -1961,11 +2271,11 @@
         sys.exit()
     if not noclose:
         # Close all standard open file descriptors
-        null = os.open(os.path.devnull, os.O_NOCTTY | os.O_RDWR)
+        null = os.open(os.devnull, os.O_NOCTTY | os.O_RDWR)
         if not stat.S_ISCHR(os.fstat(null).st_mode):
             raise OSError(errno.ENODEV,
-                          "%s not a character device"
-                          % os.path.devnull)
+                          "{0} not a character device"
+                          .format(os.devnull))
         os.dup2(null, sys.stdin.fileno())
         os.dup2(null, sys.stdout.fileno())
         os.dup2(null, sys.stderr.fileno())
@@ -1980,7 +2290,7 @@
     
     parser = argparse.ArgumentParser()
     parser.add_argument("-v", "--version", action="version",
-                        version = "%%(prog)s %s" % version,
+                        version = "%(prog)s {0}".format(version),
                         help="show version number and exit")
     parser.add_argument("-i", "--interface", metavar="IF",
                         help="Bind to interface IF")
@@ -1992,7 +2302,7 @@
                         help="Run self-test")
     parser.add_argument("--debug", action="store_true",
                         help="Debug mode; run in foreground and log"
-                        " to terminal")
+                        " to terminal", default=None)
     parser.add_argument("--debuglevel", metavar="LEVEL",
                         help="Debug level for stdout output")
     parser.add_argument("--priority", help="GnuTLS"
@@ -2005,14 +2315,20 @@
                         " files")
     parser.add_argument("--no-dbus", action="store_false",
                         dest="use_dbus", help="Do not provide D-Bus"
-                        " system bus interface")
+                        " system bus interface", default=None)
     parser.add_argument("--no-ipv6", action="store_false",
-                        dest="use_ipv6", help="Do not use IPv6")
+                        dest="use_ipv6", help="Do not use IPv6",
+                        default=None)
     parser.add_argument("--no-restore", action="store_false",
                         dest="restore", help="Do not restore stored"
-                        " state")
+                        " state", default=None)
+    parser.add_argument("--socket", type=int,
+                        help="Specify a file descriptor to a network"
+                        " socket to use instead of creating one")
     parser.add_argument("--statedir", metavar="DIR",
                         help="Directory to save/restore state in")
+    parser.add_argument("--foreground", action="store_true",
+                        help="Run in foreground", default=None)
     
     options = parser.parse_args()
     
@@ -2027,13 +2343,15 @@
                         "port": "",
                         "debug": "False",
                         "priority":
-                        "SECURE256:!CTYPE-X.509:+CTYPE-OPENPGP",
+                        "SECURE256:!CTYPE-X.509:+CTYPE-OPENPGP:+SIGN-RSA-SHA224",
                         "servicename": "Mandos",
                         "use_dbus": "True",
                         "use_ipv6": "True",
                         "debuglevel": "",
                         "restore": "True",
-                        "statedir": "/var/lib/mandos"
+                        "socket": "",
+                        "statedir": "/var/lib/mandos",
+                        "foreground": "False",
                         }
     
     # Parse config file for server-global settings
@@ -2044,12 +2362,21 @@
     # Convert the SafeConfigParser object to a dict
     server_settings = server_config.defaults()
     # Use the appropriate methods on the non-string config options
-    for option in ("debug", "use_dbus", "use_ipv6"):
+    for option in ("debug", "use_dbus", "use_ipv6", "foreground"):
         server_settings[option] = server_config.getboolean("DEFAULT",
                                                            option)
     if server_settings["port"]:
         server_settings["port"] = server_config.getint("DEFAULT",
                                                        "port")
+    if server_settings["socket"]:
+        server_settings["socket"] = server_config.getint("DEFAULT",
+                                                         "socket")
+        # Later, stdin will, and stdout and stderr might, be dup'ed
+        # over with an opened os.devnull.  But we don't want this to
+        # happen with a supplied network socket.
+        if 0 <= server_settings["socket"] <= 2:
+            server_settings["socket"] = os.dup(server_settings
+                                               ["socket"])
     del server_config
     
     # Override the settings from the config file with command line
@@ -2057,7 +2384,7 @@
     for option in ("interface", "address", "port", "debug",
                    "priority", "servicename", "configdir",
                    "use_dbus", "use_ipv6", "debuglevel", "restore",
-                   "statedir"):
+                   "statedir", "socket", "foreground"):
         value = getattr(options, option)
         if value is not None:
             server_settings[option] = value
@@ -2066,6 +2393,13 @@
     for option in server_settings.keys():
         if type(server_settings[option]) is str:
             server_settings[option] = unicode(server_settings[option])
+    # Force all boolean options to be boolean
+    for option in ("debug", "use_dbus", "use_ipv6", "restore",
+                   "foreground"):
+        server_settings[option] = bool(server_settings[option])
+    # Debug implies foreground
+    if server_settings["debug"]:
+        server_settings["foreground"] = True
     # Now we have our good server settings in "server_settings"
     
     ##################################################################
@@ -2077,6 +2411,7 @@
     use_ipv6 = server_settings["use_ipv6"]
     stored_state_path = os.path.join(server_settings["statedir"],
                                      stored_state_file)
+    foreground = server_settings["foreground"]
     
     if debug:
         initlogger(debug, logging.DEBUG)
@@ -2089,12 +2424,14 @@
     
     if server_settings["servicename"] != "Mandos":
         syslogger.setFormatter(logging.Formatter
-                               ('Mandos (%s) [%%(process)d]:'
-                                ' %%(levelname)s: %%(message)s'
-                                % server_settings["servicename"]))
+                               ('Mandos ({0}) [%(process)d]:'
+                                ' %(levelname)s: %(message)s'
+                                .format(server_settings
+                                        ["servicename"])))
     
     # Parse config file with clients
-    client_config = configparser.SafeConfigParser(Client.client_defaults)
+    client_config = configparser.SafeConfigParser(Client
+                                                  .client_defaults)
     client_config.read(os.path.join(server_settings["configdir"],
                                     "clients.conf"))
     
@@ -2109,33 +2446,33 @@
                               use_ipv6=use_ipv6,
                               gnutls_priority=
                               server_settings["priority"],
-                              use_dbus=use_dbus)
-    if not debug:
+                              use_dbus=use_dbus,
+                              socketfd=(server_settings["socket"]
+                                        or None))
+    if not foreground:
         pidfilename = "/var/run/mandos.pid"
+        pidfile = None
         try:
             pidfile = open(pidfilename, "w")
-        except IOError:
-            logger.error("Could not open file %r", pidfilename)
+        except IOError as e:
+            logger.error("Could not open file %r", pidfilename,
+                         exc_info=e)
     
-    try:
-        uid = pwd.getpwnam("_mandos").pw_uid
-        gid = pwd.getpwnam("_mandos").pw_gid
-    except KeyError:
+    for name in ("_mandos", "mandos", "nobody"):
         try:
-            uid = pwd.getpwnam("mandos").pw_uid
-            gid = pwd.getpwnam("mandos").pw_gid
+            uid = pwd.getpwnam(name).pw_uid
+            gid = pwd.getpwnam(name).pw_gid
+            break
         except KeyError:
-            try:
-                uid = pwd.getpwnam("nobody").pw_uid
-                gid = pwd.getpwnam("nobody").pw_gid
-            except KeyError:
-                uid = 65534
-                gid = 65534
+            continue
+    else:
+        uid = 65534
+        gid = 65534
     try:
         os.setgid(gid)
         os.setuid(uid)
     except OSError as error:
-        if error[0] != errno.EPERM:
+        if error.errno != errno.EPERM:
             raise error
     
     if debug:
@@ -2153,21 +2490,23 @@
          .gnutls_global_set_log_function(debug_gnutls))
         
         # Redirect stdin so all checkers get /dev/null
-        null = os.open(os.path.devnull, os.O_NOCTTY | os.O_RDWR)
+        null = os.open(os.devnull, os.O_NOCTTY | os.O_RDWR)
         os.dup2(null, sys.stdin.fileno())
         if null > 2:
             os.close(null)
     
     # Need to fork before connecting to D-Bus
-    if not debug:
+    if not foreground:
         # Close all input and output, do double fork, etc.
         daemon()
     
+    # multiprocessing will use threads, so before we use gobject we
+    # need to inform gobject that threads will be used.
     gobject.threads_init()
     
     global main_loop
     # From the Avahi example code
-    DBusGMainLoop(set_as_default=True )
+    DBusGMainLoop(set_as_default=True)
     main_loop = gobject.MainLoop()
     bus = dbus.SystemBus()
     # End of Avahi example code
@@ -2179,7 +2518,7 @@
                             ("se.bsnet.fukt.Mandos", bus,
                              do_not_queue=True))
         except dbus.exceptions.NameExistsException as e:
-            logger.error(unicode(e) + ", disabling D-Bus")
+            logger.error("Disabling D-Bus:", exc_info=e)
             use_dbus = False
             server_settings["use_dbus"] = False
             tcp_server.use_dbus = False
@@ -2197,13 +2536,20 @@
     
     client_class = Client
     if use_dbus:
-        client_class = functools.partial(ClientDBusTransitional,
-                                         bus = bus)
+        client_class = functools.partial(ClientDBus, bus = bus)
     
     client_settings = Client.config_parser(client_config)
     old_client_settings = {}
     clients_data = {}
     
+    # This is used to redirect stdout and stderr for checker processes
+    global wnull
+    wnull = open(os.devnull, "w") # A writable /dev/null
+    # Only used if server is running in foreground but not in debug
+    # mode
+    if debug or not foreground:
+        wnull.close()
+    
     # Get client data and settings from last running state.
     if server_settings["restore"]:
         try:
@@ -2212,16 +2558,23 @@
                                                      (stored_state))
             os.remove(stored_state_path)
         except IOError as e:
-            logger.warning("Could not load persistent state: {0}"
-                           .format(e))
-            if e.errno != errno.ENOENT:
+            if e.errno == errno.ENOENT:
+                logger.warning("Could not load persistent state: {0}"
+                                .format(os.strerror(e.errno)))
+            else:
+                logger.critical("Could not load persistent state:",
+                                exc_info=e)
                 raise
         except EOFError as e:
             logger.warning("Could not load persistent state: "
-                           "EOFError: {0}".format(e))
+                           "EOFError:", exc_info=e)
     
     with PGPEngine() as pgp:
         for client_name, client in clients_data.iteritems():
+            # Skip removed clients
+            if client_name not in client_settings:
+                continue
+            
             # Decide which value to use after restoring saved state.
             # We have three different values: Old config file,
             # new config file, and saved state.
@@ -2241,28 +2594,31 @@
             
             # Clients who has passed its expire date can still be
             # enabled if its last checker was successful.  Clients
-            # whose checker failed before we stored its state is
-            # assumed to have failed all checkers during downtime.
+            # whose checker succeeded before we stored its state is
+            # assumed to have successfully run all checkers during
+            # downtime.
             if client["enabled"]:
                 if datetime.datetime.utcnow() >= client["expires"]:
                     if not client["last_checked_ok"]:
                         logger.warning(
                             "disabling client {0} - Client never "
-                            "performed a successfull checker"
-                            .format(client["name"]))
+                            "performed a successful checker"
+                            .format(client_name))
                         client["enabled"] = False
                     elif client["last_checker_status"] != 0:
                         logger.warning(
                             "disabling client {0} - Client "
                             "last checker failed with error code {1}"
-                            .format(client["name"],
+                            .format(client_name,
                                     client["last_checker_status"]))
                         client["enabled"] = False
                     else:
                         client["expires"] = (datetime.datetime
                                              .utcnow()
                                              + client["timeout"])
-                    
+                        logger.debug("Last checker succeeded,"
+                                     " keeping {0} enabled"
+                                     .format(client_name))
             try:
                 client["secret"] = (
                     pgp.decrypt(client["encrypted_secret"],
@@ -2274,47 +2630,54 @@
                              .format(client_name))
                 client["secret"] = (
                     client_settings[client_name]["secret"])
-
     
     # Add/remove clients based on new changes made to config
-    for client_name in set(old_client_settings) - set(client_settings):
+    for client_name in (set(old_client_settings)
+                        - set(client_settings)):
         del clients_data[client_name]
-    for client_name in set(client_settings) - set(old_client_settings):
+    for client_name in (set(client_settings)
+                        - set(old_client_settings)):
         clients_data[client_name] = client_settings[client_name]
-
-    # Create clients all clients
+    
+    # Create all client objects
     for client_name, client in clients_data.iteritems():
         tcp_server.clients[client_name] = client_class(
-            name = client_name, settings = client)
+            name = client_name, settings = client,
+            server_settings = server_settings)
     
     if not tcp_server.clients:
         logger.warning("No clients defined")
-        
-    if not debug:
-        try:
-            with pidfile:
-                pid = os.getpid()
-                pidfile.write(str(pid) + "\n".encode("utf-8"))
-            del pidfile
-        except IOError:
-            logger.error("Could not write to file %r with PID %d",
-                         pidfilename, pid)
-        except NameError:
-            # "pidfile" was never created
-            pass
+    
+    if not foreground:
+        if pidfile is not None:
+            try:
+                with pidfile:
+                    pid = os.getpid()
+                    pidfile.write(str(pid) + "\n".encode("utf-8"))
+            except IOError:
+                logger.error("Could not write to file %r with PID %d",
+                             pidfilename, pid)
+        del pidfile
         del pidfilename
-        signal.signal(signal.SIGINT, signal.SIG_IGN)
     
     signal.signal(signal.SIGHUP, lambda signum, frame: sys.exit())
     signal.signal(signal.SIGTERM, lambda signum, frame: sys.exit())
     
     if use_dbus:
-        class MandosDBusService(dbus.service.Object):
+        @alternate_dbus_interfaces({"se.recompile.Mandos":
+                                        "se.bsnet.fukt.Mandos"})
+        class MandosDBusService(DBusObjectWithProperties):
             """A D-Bus proxy object"""
             def __init__(self):
                 dbus.service.Object.__init__(self, bus, "/")
             _interface = "se.recompile.Mandos"
             
+            @dbus_interface_annotations(_interface)
+            def _foo(self):
+                return { "org.freedesktop.DBus.Property"
+                         ".EmitsChangedSignal":
+                             "false"}
+            
             @dbus.service.signal(_interface, signature="o")
             def ClientAdded(self, objpath):
                 "D-Bus signal"
@@ -2362,15 +2725,14 @@
             
             del _interface
         
-        class MandosDBusServiceTransitional(MandosDBusService):
-            __metaclass__ = AlternateDBusNamesMetaclass
-        mandos_dbus_service = MandosDBusServiceTransitional()
+        mandos_dbus_service = MandosDBusService()
     
     def cleanup():
         "Cleanup function; run on exit"
         service.cleanup()
         
         multiprocessing.active_children()
+        wnull.close()
         if not (tcp_server.clients or client_settings):
             return
         
@@ -2388,7 +2750,7 @@
                 # A list of attributes that can not be pickled
                 # + secret.
                 exclude = set(("bus", "changedstate", "secret",
-                               "checker"))
+                               "checker", "server_settings"))
                 for name, typ in (inspect.getmembers
                                   (dbus.service.Object)):
                     exclude.add(name)
@@ -2403,23 +2765,25 @@
                 del client_settings[client.name]["secret"]
         
         try:
-            tempfd, tempname = tempfile.mkstemp(suffix=".pickle",
-                                                prefix="clients-",
-                                                dir=os.path.dirname
-                                                (stored_state_path))
-            with os.fdopen(tempfd, "wb") as stored_state:
+            with (tempfile.NamedTemporaryFile
+                  (mode='wb', suffix=".pickle", prefix='clients-',
+                   dir=os.path.dirname(stored_state_path),
+                   delete=False)) as stored_state:
                 pickle.dump((clients, client_settings), stored_state)
+                tempname=stored_state.name
             os.rename(tempname, stored_state_path)
         except (IOError, OSError) as e:
-            logger.warning("Could not save persistent state: {0}"
-                           .format(e))
             if not debug:
                 try:
                     os.remove(tempname)
                 except NameError:
                     pass
-            if e.errno not in set((errno.ENOENT, errno.EACCES,
-                                   errno.EEXIST)):
+            if e.errno in (errno.ENOENT, errno.EACCES, errno.EEXIST):
+                logger.warning("Could not save persistent state: {0}"
+                               .format(os.strerror(e.errno)))
+            else:
+                logger.warning("Could not save persistent state:",
+                               exc_info=e)
                 raise e
         
         # Delete all clients, and settings from config
@@ -2453,11 +2817,11 @@
     service.port = tcp_server.socket.getsockname()[1]
     if use_ipv6:
         logger.info("Now listening on address %r, port %d,"
-                    " flowinfo %d, scope_id %d"
-                    % tcp_server.socket.getsockname())
+                    " flowinfo %d, scope_id %d",
+                    *tcp_server.socket.getsockname())
     else:                       # IPv4
-        logger.info("Now listening on address %r, port %d"
-                    % tcp_server.socket.getsockname())
+        logger.info("Now listening on address %r, port %d",
+                    *tcp_server.socket.getsockname())
     
     #service.interface = tcp_server.socket.getsockname()[3]
     
@@ -2466,7 +2830,7 @@
         try:
             service.activate()
         except dbus.exceptions.DBusException as error:
-            logger.critical("DBusException: %s", error)
+            logger.critical("D-Bus Exception", exc_info=error)
             cleanup()
             sys.exit(1)
         # End of Avahi example code
@@ -2479,7 +2843,7 @@
         logger.debug("Starting main loop")
         main_loop.run()
     except AvahiError as error:
-        logger.critical("AvahiError: %s", error)
+        logger.critical("Avahi Error", exc_info=error)
         cleanup()
         sys.exit(1)
     except KeyboardInterrupt:

=== modified file 'mandos-clients.conf.xml'
--- mandos-clients.conf.xml	2011-11-26 23:08:17 +0000
+++ mandos-clients.conf.xml	2012-06-23 00:58:49 +0000
@@ -3,7 +3,7 @@
 	"http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd" [
 <!ENTITY CONFNAME "mandos-clients.conf">
 <!ENTITY CONFPATH "<filename>/etc/mandos/clients.conf</filename>">
-<!ENTITY TIMESTAMP "2011-11-26">
+<!ENTITY TIMESTAMP "2012-06-23">
 <!ENTITY % common SYSTEM "common.ent">
 %common;
 ]>
@@ -36,6 +36,7 @@
       <year>2009</year>
       <year>2010</year>
       <year>2011</year>
+      <year>2012</year>
       <holder>Teddy Hogeborn</holder>
       <holder>Björn Påhlsson</holder>
     </copyright>
@@ -65,8 +66,13 @@
       ><refentrytitle>mandos</refentrytitle>
       <manvolnum>8</manvolnum></citerefentry>, read by it at startup.
       The file needs to list all clients that should be able to use
-      the service.  All clients listed will be regarded as enabled,
-      even if a client was disabled in a previous run of the server.
+      the service.  The settings in this file can be overridden by
+      runtime changes to the server, which it saves across restarts.
+      (See the section called <quote>PERSISTENT STATE</quote> in
+      <citerefentry><refentrytitle>mandos</refentrytitle><manvolnum
+      >8</manvolnum></citerefentry>.)  However, any <emphasis
+      >changes</emphasis> to this file (including adding and removing
+      clients) will, at startup, override changes done during runtime.
     </para>
     <para>
       The format starts with a <literal>[<replaceable>section
@@ -161,12 +167,12 @@
 	    This option is <emphasis>optional</emphasis>.
 	  </para>
 	  <para>
-	    This option allows you to override the default shell
-	    command that the server will use to check if the client is
-	    still up.  Any output of the command will be ignored, only
-	    the exit code is checked:  If the exit code of the command
-	    is zero, the client is considered up.  The command will be
-	    run using <quote><command><filename>/bin/sh</filename>
+	    This option overrides the default shell command that the
+	    server will use to check if the client is still up.  Any
+	    output of the command will be ignored, only the exit code
+	    is checked:  If the exit code of the command is zero, the
+	    client is considered up.  The command will be run using
+	    <quote><command><filename>/bin/sh</filename>
 	    <option>-c</option></command></quote>, so
 	    <varname>PATH</varname> will be searched.  The default
 	    value for the checker command is <quote><literal
@@ -329,17 +335,12 @@
 	    <option>extended_timeout</option> option.
 	  </para>
 	  <para>
-	    The <replaceable>TIME</replaceable> is specified as a
-	    space-separated number of values, each of which is a
-	    number and a one-character suffix.  The suffix must be one
-	    of <quote>d</quote>, <quote>s</quote>, <quote>m</quote>,
-	    <quote>h</quote>, and <quote>w</quote> for days, seconds,
-	    minutes, hours, and weeks, respectively.  The values are
-	    added together to give the total time value, so all of
-	    <quote><literal>330s</literal></quote>,
-	    <quote><literal>110s 110s 110s</literal></quote>, and
-	    <quote><literal>5m 30s</literal></quote> will give a value
-	    of five minutes and thirty seconds.
+	    The <replaceable>TIME</replaceable> is specified as an RFC
+	    3339 duration; for example
+	    <quote><literal>P1Y2M3DT4H5M6S</literal></quote> meaning
+	    one year, two months, three days, four hours, five
+	    minutes, and six seconds.  Some values can be omitted, see
+	    RFC 3339 Appendix A for details.
 	  </para>
 	</listitem>
       </varlistentry>
@@ -403,6 +404,7 @@
 	<quote><literal>approval_duration</literal></quote>,
 	<quote><literal>created</literal></quote>,
 	<quote><literal>enabled</literal></quote>,
+	<quote><literal>expires</literal></quote>,
 	<quote><literal>fingerprint</literal></quote>,
 	<quote><literal>host</literal></quote>,
 	<quote><literal>interval</literal></quote>,
@@ -458,8 +460,8 @@
     <informalexample>
       <programlisting>
 [DEFAULT]
-timeout = 5m
-interval = 2m
+timeout = PT5M
+interval = PT2M
 checker = fping -q -- %%(host)s
 
 # Client "foo"
@@ -482,15 +484,15 @@
         4T2zw4dxS5NswXWU0sVEXxjs6PYxuIiCTL7vdpx8QjBkrPWDrAbcMyBr2O
         QlnHIvPzEArRQLo=
 host = foo.example.org
-interval = 1m
+interval = PT1M
 
 # Client "bar"
 [bar]
 fingerprint = 3e393aeaefb84c7e89e2f547b3a107558fca3a27
 secfile = /etc/mandos/bar-secret
-timeout = 15m
+timeout = PT15M
 approved_by_default = False
-approval_delay = 30s
+approval_delay = PT30S
       </programlisting>
     </informalexample>
   </refsect1>
@@ -505,8 +507,24 @@
       <citerefentry><refentrytitle>mandos.conf</refentrytitle>
       <manvolnum>5</manvolnum></citerefentry>,
       <citerefentry><refentrytitle>mandos</refentrytitle>
+      <manvolnum>8</manvolnum></citerefentry>,
+      <citerefentry><refentrytitle>fping</refentrytitle>
       <manvolnum>8</manvolnum></citerefentry>
     </para>
+    <variablelist>
+      <varlistentry>
+	<term>
+	  RFC 3339: <citetitle>Date and Time on the Internet:
+	  Timestamps</citetitle>
+	</term>
+      <listitem>
+	<para>
+	  The time intervals are in the "duration" format, as
+	  specified in ABNF in Appendix A of RFC 3339.
+	</para>
+      </listitem>
+      </varlistentry>
+    </variablelist>
   </refsect1>
 </refentry>
 <!-- Local Variables: -->

=== modified file 'mandos-ctl'
--- mandos-ctl	2011-12-21 17:17:32 +0000
+++ mandos-ctl	2012-11-14 21:03:24 +0000
@@ -3,8 +3,8 @@
 # 
 # Mandos Monitor - Control and monitor the Mandos server
 # 
-# Copyright © 2008-2011 Teddy Hogeborn
-# Copyright © 2008-2011 Björn Påhlsson
+# Copyright © 2008-2012 Teddy Hogeborn
+# Copyright © 2008-2012 Björn Påhlsson
 # 
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,7 +17,8 @@
 #     GNU General Public License for more details.
 # 
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see
+# <http://www.gnu.org/licenses/>.
 # 
 # Contact the authors at <mandos@recompile.se>.
 # 
@@ -25,13 +26,18 @@
 from __future__ import (division, absolute_import, print_function,
                         unicode_literals)
 
+from future_builtins import *
+
 import sys
-import dbus
 import argparse
 import locale
 import datetime
 import re
 import os
+import collections
+import doctest
+
+import dbus
 
 locale.setlocale(locale.LC_ALL, "")
 
@@ -60,7 +66,7 @@
 server_path = "/"
 server_interface = domain + ".Mandos"
 client_interface = domain + ".Mandos.Client"
-version = "1.4.1"
+version = "1.6.0"
 
 def timedelta_to_milliseconds(td):
     """Convert a datetime.timedelta object to milliseconds"""
@@ -70,12 +76,112 @@
 
 def milliseconds_to_string(ms):
     td = datetime.timedelta(0, 0, 0, ms)
-    return ("%(days)s%(hours)02d:%(minutes)02d:%(seconds)02d"
-            % { "days": "%dT" % td.days if td.days else "",
-                "hours": td.seconds // 3600,
-                "minutes": (td.seconds % 3600) // 60,
-                "seconds": td.seconds % 60,
-                })
+    return ("{days}{hours:02}:{minutes:02}:{seconds:02}"
+            .format(days = "{0}T".format(td.days) if td.days else "",
+                    hours = td.seconds // 3600,
+                    minutes = (td.seconds % 3600) // 60,
+                    seconds = td.seconds % 60,
+                    ))
+
+
+def rfc3339_duration_to_delta(duration):
+    """Parse an RFC 3339 "duration" and return a datetime.timedelta
+    
+    >>> rfc3339_duration_to_delta("P7D")
+    datetime.timedelta(7)
+    >>> rfc3339_duration_to_delta("PT60S")
+    datetime.timedelta(0, 60)
+    >>> rfc3339_duration_to_delta("PT60M")
+    datetime.timedelta(0, 3600)
+    >>> rfc3339_duration_to_delta("PT24H")
+    datetime.timedelta(1)
+    >>> rfc3339_duration_to_delta("P1W")
+    datetime.timedelta(7)
+    >>> rfc3339_duration_to_delta("PT5M30S")
+    datetime.timedelta(0, 330)
+    >>> rfc3339_duration_to_delta("P1DT3M20S")
+    datetime.timedelta(1, 200)
+    """
+    
+    # Parsing an RFC 3339 duration with regular expressions is not
+    # possible - there would have to be multiple places for the same
+    # values, like seconds.  The current code, while more esoteric, is
+    # cleaner without depending on a parsing library.  If Python had a
+    # built-in library for parsing we would use it, but we'd like to
+    # avoid excessive use of external libraries.
+    
+    # New type for defining tokens, syntax, and semantics all-in-one
+    Token = collections.namedtuple("Token",
+                                   ("regexp", # To match token; if
+                                              # "value" is not None,
+                                              # must have a "group"
+                                              # containing digits
+                                    "value",  # datetime.timedelta or
+                                              # None
+                                    "followers")) # Tokens valid after
+                                                  # this token
+    # RFC 3339 "duration" tokens, syntax, and semantics; taken from
+    # the "duration" ABNF definition in RFC 3339, Appendix A.
+    token_end = Token(re.compile(r"$"), None, frozenset())
+    token_second = Token(re.compile(r"(\d+)S"),
+                         datetime.timedelta(seconds=1),
+                         frozenset((token_end,)))
+    token_minute = Token(re.compile(r"(\d+)M"),
+                         datetime.timedelta(minutes=1),
+                         frozenset((token_second, token_end)))
+    token_hour = Token(re.compile(r"(\d+)H"),
+                       datetime.timedelta(hours=1),
+                       frozenset((token_minute, token_end)))
+    token_time = Token(re.compile(r"T"),
+                       None,
+                       frozenset((token_hour, token_minute,
+                                  token_second)))
+    token_day = Token(re.compile(r"(\d+)D"),
+                      datetime.timedelta(days=1),
+                      frozenset((token_time, token_end)))
+    token_month = Token(re.compile(r"(\d+)M"),
+                        datetime.timedelta(weeks=4),
+                        frozenset((token_day, token_end)))
+    token_year = Token(re.compile(r"(\d+)Y"),
+                       datetime.timedelta(weeks=52),
+                       frozenset((token_month, token_end)))
+    token_week = Token(re.compile(r"(\d+)W"),
+                       datetime.timedelta(weeks=1),
+                       frozenset((token_end,)))
+    token_duration = Token(re.compile(r"P"), None,
+                           frozenset((token_year, token_month,
+                                      token_day, token_time,
+                                      token_week))),
+    # Define starting values
+    value = datetime.timedelta() # Value so far
+    found_token = None
+    followers = frozenset(token_duration,) # Following valid tokens
+    s = duration                # String left to parse
+    # Loop until end token is found
+    while found_token is not token_end:
+        # Search for any currently valid tokens
+        for token in followers:
+            match = token.regexp.match(s)
+            if match is not None:
+                # Token found
+                if token.value is not None:
+                    # Value found, parse digits
+                    factor = int(match.group(1), 10)
+                    # Add to value so far
+                    value += factor * token.value
+                # Strip token from string
+                s = token.regexp.sub("", s, 1)
+                # Go to found token
+                found_token = token
+                # Set valid next tokens
+                followers = found_token.followers
+                break
+        else:
+            # No currently valid tokens were found
+            raise ValueError("Invalid RFC 3339 duration")
+    # End token found
+    return value
+
 
 def string_to_delta(interval):
     """Parse a string and return a datetime.timedelta
@@ -93,8 +199,14 @@
     >>> string_to_delta("5m 30s")
     datetime.timedelta(0, 330)
     """
+    
+    try:
+        return rfc3339_duration_to_delta(interval)
+    except ValueError:
+        pass
+    
     value = datetime.timedelta(0)
-    regexp = re.compile("(\d+)([dsmhw]?)")
+    regexp = re.compile(r"(\d+)([dsmhw]?)")
     
     for num, suffix in regexp.findall(interval):
         if suffix == "d":
@@ -116,23 +228,25 @@
         if type(value) is dbus.Boolean:
             return "Yes" if value else "No"
         if keyword in ("Timeout", "Interval", "ApprovalDelay",
-                       "ApprovalDuration"):
+                       "ApprovalDuration", "ExtendedTimeout"):
             return milliseconds_to_string(value)
         return unicode(value)
     
     # Create format string to print table rows
-    format_string = " ".join("%%-%ds" %
-                             max(len(tablewords[key]),
-                                 max(len(valuetostring(client[key],
-                                                       key))
-                                     for client in
-                                     clients))
-                             for key in keywords)
+    format_string = " ".join("{{{key}:{width}}}".format(
+            width = max(len(tablewords[key]),
+                        max(len(valuetostring(client[key],
+                                              key))
+                            for client in
+                            clients)),
+            key = key) for key in keywords)
     # Print header line
-    print(format_string % tuple(tablewords[key] for key in keywords))
+    print(format_string.format(**tablewords))
     for client in clients:
-        print(format_string % tuple(valuetostring(client[key], key)
-                                    for key in keywords))
+        print(format_string.format(**dict((key,
+                                           valuetostring(client[key],
+                                                         key))
+                                          for key in keywords)))
 
 def has_actions(options):
     return any((options.enable,
@@ -157,7 +271,7 @@
 def main():
     parser = argparse.ArgumentParser()
     parser.add_argument("--version", action="version",
-                        version = "%%prog %s" % version,
+                        version = "%(prog)s {0}".format(version),
                         help="show version number and exit")
     parser.add_argument("-a", "--all", action="store_true",
                         help="Select all clients")
@@ -202,16 +316,22 @@
                         help="Approve any current client request")
     parser.add_argument("-D", "--deny", action="store_true",
                         help="Deny any current client request")
+    parser.add_argument("--check", action="store_true",
+                        help="Run self-test")
     parser.add_argument("client", nargs="*", help="Client name")
     options = parser.parse_args()
     
-    if has_actions(options) and not options.client and not options.all:
+    if has_actions(options) and not (options.client or options.all):
         parser.error("Options require clients names or --all.")
     if options.verbose and has_actions(options):
         parser.error("--verbose can only be used alone or with"
                      " --all.")
     if options.all and not has_actions(options):
         parser.error("--all requires an action.")
+
+    if options.check:
+        fail_count, test_count = doctest.testmod()
+        sys.exit(0 if fail_count == 0 else 1)
     
     try:
         bus = dbus.SystemBus()
@@ -256,8 +376,8 @@
                     clients[client_objc] = client
                     break
             else:
-                print("Client not found on server: %r" % name,
-                      file=sys.stderr)
+                print("Client not found on server: {0!r}"
+                      .format(name), file=sys.stderr)
                 sys.exit(1)
     
     if not has_actions(options) and clients:
@@ -277,18 +397,28 @@
     else:
         # Process each client in the list by all selected options
         for client in clients:
+            def set_client_prop(prop, value):
+                """Set a Client D-Bus property"""
+                client.Set(client_interface, prop, value,
+                           dbus_interface=dbus.PROPERTIES_IFACE)
+            def set_client_prop_ms(prop, value):
+                """Set a Client D-Bus property, converted
+                from a string to milliseconds."""
+                set_client_prop(prop,
+                                timedelta_to_milliseconds
+                                (string_to_delta(value)))
             if options.remove:
                 mandos_serv.RemoveClient(client.__dbus_object_path__)
             if options.enable:
-                client.Enable(dbus_interface=client_interface)
+                set_client_prop("Enabled", dbus.Boolean(True))
             if options.disable:
-                client.Disable(dbus_interface=client_interface)
+                set_client_prop("Enabled", dbus.Boolean(False))
             if options.bump_timeout:
-                client.CheckedOK(dbus_interface=client_interface)
+                set_client_prop("LastCheckedOK", "")
             if options.start_checker:
-                client.StartChecker(dbus_interface=client_interface)
+                set_client_prop("CheckerRunning", dbus.Boolean(True))
             if options.stop_checker:
-                client.StopChecker(dbus_interface=client_interface)
+                set_client_prop("CheckerRunning", dbus.Boolean(False))
             if options.is_enabled:
                 sys.exit(0 if client.Get(client_interface,
                                          "Enabled",
@@ -296,49 +426,29 @@
                                          dbus.PROPERTIES_IFACE)
                          else 1)
             if options.checker is not None:
-                client.Set(client_interface, "Checker",
-                           options.checker,
-                           dbus_interface=dbus.PROPERTIES_IFACE)
+                set_client_prop("Checker", options.checker)
             if options.host is not None:
-                client.Set(client_interface, "Host", options.host,
-                           dbus_interface=dbus.PROPERTIES_IFACE)
+                set_client_prop("Host", options.host)
             if options.interval is not None:
-                client.Set(client_interface, "Interval",
-                           timedelta_to_milliseconds
-                           (string_to_delta(options.interval)),
-                           dbus_interface=dbus.PROPERTIES_IFACE)
+                set_client_prop_ms("Interval", options.interval)
             if options.approval_delay is not None:
-                client.Set(client_interface, "ApprovalDelay",
-                           timedelta_to_milliseconds
-                           (string_to_delta(options.
-                                            approval_delay)),
-                           dbus_interface=dbus.PROPERTIES_IFACE)
+                set_client_prop_ms("ApprovalDelay",
+                                   options.approval_delay)
             if options.approval_duration is not None:
-                client.Set(client_interface, "ApprovalDuration",
-                           timedelta_to_milliseconds
-                           (string_to_delta(options.
-                                            approval_duration)),
-                           dbus_interface=dbus.PROPERTIES_IFACE)
+                set_client_prop_ms("ApprovalDuration",
+                                   options.approval_duration)
             if options.timeout is not None:
-                client.Set(client_interface, "Timeout",
-                           timedelta_to_milliseconds
-                           (string_to_delta(options.timeout)),
-                           dbus_interface=dbus.PROPERTIES_IFACE)
+                set_client_prop_ms("Timeout", options.timeout)
             if options.extended_timeout is not None:
-                client.Set(client_interface, "ExtendedTimeout",
-                           timedelta_to_milliseconds
-                           (string_to_delta(options.extended_timeout)),
-                           dbus_interface=dbus.PROPERTIES_IFACE)
+                set_client_prop_ms("ExtendedTimeout",
+                                   options.extended_timeout)
             if options.secret is not None:
-                client.Set(client_interface, "Secret",
-                           dbus.ByteArray(open(options.secret,
-                                               "rb").read()),
-                           dbus_interface=dbus.PROPERTIES_IFACE)
+                set_client_prop("Secret",
+                                dbus.ByteArray(options.secret.read()))
             if options.approved_by_default is not None:
-                client.Set(client_interface, "ApprovedByDefault",
-                           dbus.Boolean(options
-                                        .approved_by_default),
-                           dbus_interface=dbus.PROPERTIES_IFACE)
+                set_client_prop("ApprovedByDefault",
+                                dbus.Boolean(options
+                                             .approved_by_default))
             if options.approve:
                 client.Approve(dbus.Boolean(True),
                                dbus_interface=client_interface)

=== modified file 'mandos-ctl.xml'
--- mandos-ctl.xml	2011-10-05 16:00:56 +0000
+++ mandos-ctl.xml	2012-06-22 23:33:56 +0000
@@ -2,7 +2,7 @@
 <!DOCTYPE refentry PUBLIC "-//OASIS//DTD DocBook XML V4.5//EN"
 	"http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd" [
 <!ENTITY COMMANDNAME "mandos-ctl">
-<!ENTITY TIMESTAMP "2011-10-03">
+<!ENTITY TIMESTAMP "2012-06-22">
 <!ENTITY % common SYSTEM "common.ent">
 %common;
 ]>
@@ -33,6 +33,7 @@
     <copyright>
       <year>2010</year>
       <year>2011</year>
+      <year>2012</year>
       <holder>Teddy Hogeborn</holder>
       <holder>Björn Påhlsson</holder>
     </copyright>
@@ -194,6 +195,10 @@
 	<arg choice="plain"><option>-v</option></arg>
       </group>
     </cmdsynopsis>
+    <cmdsynopsis>
+      <command>&COMMANDNAME;</command>
+      <arg choice="plain"><option>--check</option></arg>
+    </cmdsynopsis>
   </refsynopsisdiv>
   
   <refsect1 id="description">
@@ -475,6 +480,15 @@
 	</listitem>
       </varlistentry>
       
+      <varlistentry>
+	<term><option>--check</option></term>
+	<listitem>
+	  <para>
+	    Run self-tests.  This includes any unit tests, etc.
+	  </para>
+	</listitem>
+      </varlistentry>
+      
     </variablelist>
   </refsect1>
   

=== modified file 'mandos-keygen'
--- mandos-keygen	2011-10-15 16:48:03 +0000
+++ mandos-keygen	2013-09-29 15:52:19 +0000
@@ -2,8 +2,8 @@
 # 
 # Mandos key generator - create a new OpenPGP key for a Mandos client
 # 
-# Copyright © 2008-2011 Teddy Hogeborn
-# Copyright © 2008-2011 Björn Påhlsson
+# Copyright © 2008-2012 Teddy Hogeborn
+# Copyright © 2008-2012 Björn Påhlsson
 # 
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -21,13 +21,13 @@
 # Contact the authors at <mandos@recompile.se>.
 # 
 
-VERSION="1.4.1"
+VERSION="1.6.0"
 
 KEYDIR="/etc/keys/mandos"
-KEYTYPE=DSA
-KEYLENGTH=2048
-SUBKEYTYPE=ELG-E
-SUBKEYLENGTH=2048
+KEYTYPE=RSA
+KEYLENGTH=4096
+SUBKEYTYPE=RSA
+SUBKEYLENGTH=4096
 KEYNAME="`hostname --fqdn 2>/dev/null || hostname`"
 KEYEMAIL=""
 KEYCOMMENT="Mandos client key"
@@ -60,13 +60,13 @@
   -v, --version         Show program's version number and exit
   -h, --help            Show this help message and exit
   -d DIR, --dir DIR     Target directory for key files
-  -t TYPE, --type TYPE  Key type.  Default is DSA.
+  -t TYPE, --type TYPE  Key type.  Default is RSA.
   -l BITS, --length BITS
-                        Key length in bits.  Default is 2048.
+                        Key length in bits.  Default is 4096.
   -s TYPE, --subtype TYPE
                         Subkey type.  Default is ELG-E.
   -L BITS, --sublength BITS
-                        Subkey length in bits.  Default is 2048.
+                        Subkey length in bits.  Default is 4096.
   -n NAME, --name NAME  Name of key.  Default is the FQDN.
   -e ADDRESS, --email ADDRESS
                         Email address of key.  Default is empty.
@@ -294,9 +294,11 @@
 	    cat "$PASSFILE"
 	else
 	    tty --quiet && stty -echo
-	    read -p "Enter passphrase: " first
+	    echo -n "Enter passphrase: "
+	    read first
 	    tty --quiet && echo >&2
-	    read -p "Repeat passphrase: " second
+	    echo -n "Repeat passphrase: "
+	    read second
 	    if tty --quiet; then
 		echo >&2
 		stty echo

=== modified file 'mandos-keygen.xml'
--- mandos-keygen.xml	2011-11-12 18:14:55 +0000
+++ mandos-keygen.xml	2013-09-29 15:52:19 +0000
@@ -2,7 +2,7 @@
 <!DOCTYPE refentry PUBLIC "-//OASIS//DTD DocBook XML V4.5//EN"
 	"http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd" [
 <!ENTITY COMMANDNAME "mandos-keygen">
-<!ENTITY TIMESTAMP "2011-10-03">
+<!ENTITY TIMESTAMP "2013-08-27">
 <!ENTITY % common SYSTEM "common.ent">
 %common;
 ]>
@@ -34,6 +34,7 @@
       <year>2008</year>
       <year>2009</year>
       <year>2011</year>
+      <year>2012</year>
       <holder>Teddy Hogeborn</holder>
       <holder>Björn Påhlsson</holder>
     </copyright>
@@ -226,7 +227,7 @@
 	<replaceable>TYPE</replaceable></option></term>
 	<listitem>
 	  <para>
-	    Key type.  Default is <quote>DSA</quote>.
+	    Key type.  Default is <quote>RSA</quote>.
 	  </para>
 	</listitem>
       </varlistentry>
@@ -238,7 +239,7 @@
 	<replaceable>BITS</replaceable></option></term>
 	<listitem>
 	  <para>
-	    Key length in bits.  Default is 2048.
+	    Key length in bits.  Default is 4096.
 	  </para>
 	</listitem>
       </varlistentry>
@@ -250,7 +251,7 @@
 	<replaceable>KEYTYPE</replaceable></option></term>
 	<listitem>
 	  <para>
-	    Subkey type.  Default is <quote>ELG-E</quote> (Elgamal
+	    Subkey type.  Default is <quote>RSA</quote> (Elgamal
 	    encryption-only).
 	  </para>
 	</listitem>
@@ -263,7 +264,7 @@
 	<replaceable>BITS</replaceable></option></term>
 	<listitem>
 	  <para>
-	    Subkey length in bits.  Default is 2048.
+	    Subkey length in bits.  Default is 4096.
 	  </para>
 	</listitem>
       </varlistentry>

=== modified file 'mandos-monitor'
--- mandos-monitor	2011-10-15 16:48:03 +0000
+++ mandos-monitor	2013-05-22 20:00:18 +0000
@@ -3,8 +3,8 @@
 # 
 # Mandos Monitor - Control and monitor the Mandos server
 # 
-# Copyright © 2009-2011 Teddy Hogeborn
-# Copyright © 2009-2011 Björn Påhlsson
+# Copyright © 2009-2012 Teddy Hogeborn
+# Copyright © 2009-2012 Björn Påhlsson
 # 
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,13 +17,18 @@
 #     GNU General Public License for more details.
 # 
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see
+# <http://www.gnu.org/licenses/>.
 # 
 # Contact the authors at <mandos@recompile.se>.
 # 
 
 from __future__ import (division, absolute_import, print_function,
                         unicode_literals)
+try:
+    from future_builtins import *
+except ImportError:
+    pass
 
 import sys
 import os
@@ -35,14 +40,18 @@
 import urwid
 
 from dbus.mainloop.glib import DBusGMainLoop
-import gobject
+try:
+    import gobject
+except ImportError:
+    from gi.repository import GObject as gobject
 
 import dbus
 
-import UserList
-
 import locale
 
+if sys.version_info[0] == 2:
+    str = unicode
+
 locale.setlocale(locale.LC_ALL, '')
 
 import logging
@@ -52,15 +61,7 @@
 domain = 'se.recompile'
 server_interface = domain + '.Mandos'
 client_interface = domain + '.Mandos.Client'
-version = "1.4.1"
-
-# Always run in monochrome mode
-urwid.curses_display.curses.has_colors = lambda : False
-
-# Urwid doesn't support blinking, but we want it.  Since we have no
-# use for underline on its own, we make underline also always blink.
-urwid.curses_display.curses.A_UNDERLINE |= (
-    urwid.curses_display.curses.A_BLINK)
+version = "1.6.0"
 
 def isoformat_to_datetime(iso):
     "Parse an ISO 8601 date string to a datetime.datetime()"
@@ -83,23 +84,26 @@
     properties and calls a hook function when any of them are
     changed.
     """
-    def __init__(self, proxy_object=None, *args, **kwargs):
+    def __init__(self, proxy_object=None, properties=None, **kwargs):
         self.proxy = proxy_object # Mandos Client proxy object
-        
-        self.properties = dict()
+        self.properties = dict() if properties is None else properties
         self.property_changed_match = (
             self.proxy.connect_to_signal("PropertyChanged",
-                                         self.property_changed,
+                                         self._property_changed,
                                          client_interface,
                                          byte_arrays=True))
         
-        self.properties.update(
-            self.proxy.GetAll(client_interface,
-                              dbus_interface = dbus.PROPERTIES_IFACE))
-
-        #XXX This breaks good super behaviour
-#        super(MandosClientPropertyCache, self).__init__(
-#            *args, **kwargs)
+        if properties is None:
+            self.properties.update(
+                self.proxy.GetAll(client_interface,
+                                  dbus_interface
+                                  = dbus.PROPERTIES_IFACE))
+        
+        super(MandosClientPropertyCache, self).__init__(**kwargs)
+    
+    def _property_changed(self, property, value):
+        """Helper which takes positional arguments"""
+        return self.property_changed(property=property, value=value)
     
     def property_changed(self, property=None, value=None):
         """This is called whenever we get a PropertyChanged signal
@@ -108,10 +112,8 @@
         # Update properties dict with new value
         self.properties[property] = value
     
-    def delete(self, *args, **kwargs):
+    def delete(self):
         self.property_changed_match.remove()
-        super(MandosClientPropertyCache, self).__init__(
-            *args, **kwargs)
 
 
 class MandosClientWidget(urwid.FlowWidget, MandosClientPropertyCache):
@@ -119,7 +121,7 @@
     """
     
     def __init__(self, server_proxy_object=None, update_hook=None,
-                 delete_hook=None, logger=None, *args, **kwargs):
+                 delete_hook=None, logger=None, **kwargs):
         # Called on update
         self.update_hook = update_hook
         # Called on delete
@@ -130,37 +132,15 @@
         self.logger = logger
         
         self._update_timer_callback_tag = None
-        self._update_timer_callback_lock = 0
-        self.last_checker_failed = False
         
         # The widget shown normally
         self._text_widget = urwid.Text("")
         # The widget shown when we have focus
         self._focus_text_widget = urwid.Text("")
-        super(MandosClientWidget, self).__init__(
-            update_hook=update_hook, delete_hook=delete_hook,
-            *args, **kwargs)
+        super(MandosClientWidget, self).__init__(**kwargs)
         self.update()
         self.opened = False
         
-        last_checked_ok = isoformat_to_datetime(self.properties
-                                                ["LastCheckedOK"])
-        if last_checked_ok is None:
-            self.last_checker_failed = True
-        else:
-            self.last_checker_failed = ((datetime.datetime.utcnow()
-                                         - last_checked_ok)
-                                        > datetime.timedelta
-                                        (milliseconds=
-                                         self.properties
-                                         ["Interval"]))
-        
-        if self.last_checker_failed:
-            self.using_timer(True)
-        
-        if self.need_approval:
-            self.using_timer(True)
-        
         self.match_objects = (
             self.proxy.connect_to_signal("CheckerCompleted",
                                          self.checker_completed,
@@ -182,87 +162,70 @@
                                          self.rejected,
                                          client_interface,
                                          byte_arrays=True))
-        #self.logger('Created client %s' % (self.properties["Name"]))
+        #self.logger('Created client {0}'
+        #            .format(self.properties["Name"]))
     
-    def property_changed(self, property=None, value=None):
-        super(self, MandosClientWidget).property_changed(property,
-                                                         value)
-        if property == "ApprovalPending":
-            using_timer(bool(value))
-        
     def using_timer(self, flag):
         """Call this method with True or False when timer should be
         activated or deactivated.
         """
-        old = self._update_timer_callback_lock
-        if flag:
-            self._update_timer_callback_lock += 1
-        else:
-            self._update_timer_callback_lock -= 1
-        if old == 0 and self._update_timer_callback_lock:
+        if flag and self._update_timer_callback_tag is None:
             # Will update the shown timer value every second
             self._update_timer_callback_tag = (gobject.timeout_add
                                                (1000,
                                                 self.update_timer))
-        elif old and self._update_timer_callback_lock == 0:
+        elif not (flag or self._update_timer_callback_tag is None):
             gobject.source_remove(self._update_timer_callback_tag)
             self._update_timer_callback_tag = None
     
     def checker_completed(self, exitstatus, condition, command):
         if exitstatus == 0:
-            if self.last_checker_failed:
-                self.last_checker_failed = False
-                self.using_timer(False)
-            #self.logger('Checker for client %s (command "%s")'
-            #            ' was successful'
-            #            % (self.properties["Name"], command))
             self.update()
             return
         # Checker failed
-        if not self.last_checker_failed:
-            self.last_checker_failed = True
-            self.using_timer(True)
         if os.WIFEXITED(condition):
-            self.logger('Checker for client %s (command "%s")'
-                        ' failed with exit code %s'
-                        % (self.properties["Name"], command,
-                           os.WEXITSTATUS(condition)))
+            self.logger('Checker for client {0} (command "{1}")'
+                        ' failed with exit code {2}'
+                        .format(self.properties["Name"], command,
+                                os.WEXITSTATUS(condition)))
         elif os.WIFSIGNALED(condition):
-            self.logger('Checker for client %s (command "%s")'
-                        ' was killed by signal %s'
-                        % (self.properties["Name"], command,
-                           os.WTERMSIG(condition)))
+            self.logger('Checker for client {0} (command "{1}") was'
+                        ' killed by signal {2}'
+                        .format(self.properties["Name"], command,
+                                os.WTERMSIG(condition)))
         elif os.WCOREDUMP(condition):
-            self.logger('Checker for client %s (command "%s")'
+            self.logger('Checker for client {0} (command "{1}")'
                         ' dumped core'
-                        % (self.properties["Name"], command))
+                        .format(self.properties["Name"], command))
         else:
-            self.logger('Checker for client %s completed'
-                        ' mysteriously')
+            self.logger('Checker for client {0} completed'
+                        ' mysteriously'
+                        .format(self.properties["Name"]))
         self.update()
     
     def checker_started(self, command):
-        #self.logger('Client %s started checker "%s"'
-        #            % (self.properties["Name"], unicode(command)))
+        """Server signals that a checker started. This could be useful
+           to log in the future. """
+        #self.logger('Client {0} started checker "{1}"'
+        #            .format(self.properties["Name"],
+        #                    str(command)))
         pass
     
     def got_secret(self):
-        self.last_checker_failed = False
-        self.logger('Client %s received its secret'
-                    % self.properties["Name"])
+        self.logger('Client {0} received its secret'
+                    .format(self.properties["Name"]))
     
     def need_approval(self, timeout, default):
         if not default:
-            message = 'Client %s needs approval within %s seconds'
+            message = 'Client {0} needs approval within {1} seconds'
         else:
-            message = 'Client %s will get its secret in %s seconds'
-        self.logger(message
-                    % (self.properties["Name"], timeout/1000))
-        self.using_timer(True)
+            message = 'Client {0} will get its secret in {1} seconds'
+        self.logger(message.format(self.properties["Name"],
+                                   timeout/1000))
     
     def rejected(self, reason):
-        self.logger('Client %s was rejected; reason: %s'
-                    % (self.properties["Name"], reason))
+        self.logger('Client {0} was rejected; reason: {1}'
+                    .format(self.properties["Name"], reason))
     
     def selectable(self):
         """Make this a "selectable" widget.
@@ -290,14 +253,14 @@
                           "bold-underline-blink":
                               "bold-underline-blink-standout",
                           }
-
+        
         # Rebuild focus and non-focus widgets using current properties
-
+        
         # Base part of a client. Name!
-        base = ('%(name)s: '
-                      % {"name": self.properties["Name"]})
+        base = '{name}: '.format(name=self.properties["Name"])
         if not self.properties["Enabled"]:
             message = "DISABLED"
+            self.using_timer(False)
         elif self.properties["ApprovalPending"]:
             timeout = datetime.timedelta(milliseconds
                                          = self.properties
@@ -305,31 +268,36 @@
             last_approval_request = isoformat_to_datetime(
                 self.properties["LastApprovalRequest"])
             if last_approval_request is not None:
-                timer = timeout - (datetime.datetime.utcnow()
-                                   - last_approval_request)
+                timer = max(timeout - (datetime.datetime.utcnow()
+                                       - last_approval_request),
+                            datetime.timedelta())
             else:
                 timer = datetime.timedelta()
             if self.properties["ApprovedByDefault"]:
-                message = "Approval in %s. (d)eny?"
+                message = "Approval in {0}. (d)eny?"
             else:
-                message = "Denial in %s. (a)pprove?"
-            message = message % unicode(timer).rsplit(".", 1)[0]
-        elif self.last_checker_failed:
-            # When checker has failed, print a timer until client expires
+                message = "Denial in {0}. (a)pprove?"
+            message = message.format(str(timer).rsplit(".", 1)[0])
+            self.using_timer(True)
+        elif self.properties["LastCheckerStatus"] != 0:
+            # When checker has failed, show timer until client expires
             expires = self.properties["Expires"]
             if expires == "":
                 timer = datetime.timedelta(0)
             else:
-                expires = datetime.datetime.strptime(expires,
-                                                     '%Y-%m-%dT%H:%M:%S.%f')
-                timer = expires - datetime.datetime.utcnow()
+                expires = (datetime.datetime.strptime
+                           (expires, '%Y-%m-%dT%H:%M:%S.%f'))
+                timer = max(expires - datetime.datetime.utcnow(),
+                            datetime.timedelta())
             message = ('A checker has failed! Time until client'
-                       ' gets disabled: %s'
-                           % unicode(timer).rsplit(".", 1)[0])
+                       ' gets disabled: {0}'
+                       .format(str(timer).rsplit(".", 1)[0]))
+            self.using_timer(True)
         else:
             message = "enabled"
-        self._text = "%s%s" % (base, message)
-            
+            self.using_timer(False)
+        self._text = "{0}{1}".format(base, message)
+        
         if not urwid.supports_unicode():
             self._text = self._text.encode("ascii", "replace")
         textlist = [("normal", self._text)]
@@ -352,7 +320,7 @@
         self.update()
         return True             # Keep calling this
     
-    def delete(self, *args, **kwargs):
+    def delete(self, **kwargs):
         if self._update_timer_callback_tag is not None:
             gobject.source_remove(self._update_timer_callback_tag)
             self._update_timer_callback_tag = None
@@ -361,7 +329,7 @@
         self.match_objects = ()
         if self.delete_hook is not None:
             self.delete_hook(self)
-        return super(MandosClientWidget, self).delete(*args, **kwargs)
+        return super(MandosClientWidget, self).delete(**kwargs)
     
     def render(self, maxcolrow, focus=False):
         """Render differently if we have focus.
@@ -409,14 +377,13 @@
         else:
             return key
     
-    def property_changed(self, property=None, value=None,
-                         *args, **kwargs):
+    def property_changed(self, property=None, **kwargs):
         """Call self.update() if old value is not new value.
         This overrides the method from MandosClientPropertyCache"""
-        property_name = unicode(property)
+        property_name = str(property)
         old_value = self.properties.get(property_name)
         super(MandosClientWidget, self).property_changed(
-            property=property, value=value, *args, **kwargs)
+            property=property, **kwargs)
         if self.properties.get(property_name) != old_value:
             self.update()
 
@@ -426,8 +393,8 @@
     "down" key presses, thus not allowing any containing widgets to
     use them as an excuse to shift focus away from this widget.
     """
-    def keypress(self, maxcolrow, key):
-        ret = super(ConstrainedListBox, self).keypress(maxcolrow, key)
+    def keypress(self, *args, **kwargs):
+        ret = super(ConstrainedListBox, self).keypress(*args, **kwargs)
         if ret in ("up", "down"):
             return
         return ret
@@ -446,20 +413,21 @@
                 ("normal",
                  "default", "default", None),
                 ("bold",
-                 "default", "default", "bold"),
+                 "bold", "default", "bold"),
                 ("underline-blink",
-                 "default", "default", "underline"),
+                 "underline,blink", "default", "underline,blink"),
                 ("standout",
-                 "default", "default", "standout"),
+                 "standout", "default", "standout"),
                 ("bold-underline-blink",
-                 "default", "default", ("bold", "underline")),
+                 "bold,underline,blink", "default", "bold,underline,blink"),
                 ("bold-standout",
-                 "default", "default", ("bold", "standout")),
+                 "bold,standout", "default", "bold,standout"),
                 ("underline-blink-standout",
-                 "default", "default", ("underline", "standout")),
+                 "underline,blink,standout", "default",
+                 "underline,blink,standout"),
                 ("bold-underline-blink-standout",
-                 "default", "default", ("bold", "underline",
-                                          "standout")),
+                 "bold,underline,blink,standout", "default",
+                 "bold,underline,blink,standout"),
                 ))
         
         if urwid.supports_unicode():
@@ -497,53 +465,11 @@
         
         self.busname = domain + '.Mandos'
         self.main_loop = gobject.MainLoop()
-        self.bus = dbus.SystemBus()
-        mandos_dbus_objc = self.bus.get_object(
-            self.busname, "/", follow_name_owner_changes=True)
-        self.mandos_serv = dbus.Interface(mandos_dbus_objc,
-                                          dbus_interface
-                                          = server_interface)
-        try:
-            mandos_clients = (self.mandos_serv
-                              .GetAllClientsWithProperties())
-        except dbus.exceptions.DBusException:
-            mandos_clients = dbus.Dictionary()
-        
-        (self.mandos_serv
-         .connect_to_signal("ClientRemoved",
-                            self.find_and_remove_client,
-                            dbus_interface=server_interface,
-                            byte_arrays=True))
-        (self.mandos_serv
-         .connect_to_signal("ClientAdded",
-                            self.add_new_client,
-                            dbus_interface=server_interface,
-                            byte_arrays=True))
-        (self.mandos_serv
-         .connect_to_signal("ClientNotFound",
-                            self.client_not_found,
-                            dbus_interface=server_interface,
-                            byte_arrays=True))
-        for path, client in mandos_clients.iteritems():
-            client_proxy_object = self.bus.get_object(self.busname,
-                                                      path)
-            self.add_client(MandosClientWidget(server_proxy_object
-                                               =self.mandos_serv,
-                                               proxy_object
-                                               =client_proxy_object,
-                                               properties=client,
-                                               update_hook
-                                               =self.refresh,
-                                               delete_hook
-                                               =self.remove_client,
-                                               logger
-                                               =self.log_message),
-                            path=path)
     
     def client_not_found(self, fingerprint, address):
-        self.log_message(("Client with address %s and fingerprint %s"
-                          " could not be found" % (address,
-                                                    fingerprint)))
+        self.log_message("Client with address {0} and fingerprint"
+                         " {1} could not be found"
+                         .format(address, fingerprint))
     
     def rebuild(self):
         """This rebuilds the User Interface.
@@ -559,10 +485,10 @@
                                                      self.divider)))
         if self.log_visible:
             self.uilist.append(self.logbox)
-            pass
         self.topwidget = urwid.Pile(self.uilist)
     
     def log_message(self, message):
+        """Log message formatted with timestamp"""
         timestamp = datetime.datetime.now().isoformat()
         self.log_message_raw(timestamp + ": " + message)
     
@@ -581,7 +507,7 @@
         self.log_visible = not self.log_visible
         self.rebuild()
         #self.log_message("Log visibility changed to: "
-        #                 + unicode(self.log_visible))
+        #                 + str(self.log_visible))
     
     def change_log_display(self):
         """Change type of log display.
@@ -603,8 +529,8 @@
             client = self.clients_dict[path]
         except KeyError:
             # not found?
-            self.log_message("Unknown client %r (%r) removed", name,
-                             path)
+            self.log_message("Unknown client {0!r} ({1!r}) removed"
+                             .format(name, path))
             return
         client.delete()
     
@@ -627,7 +553,7 @@
         if path is None:
             path = client.proxy.object_path
         self.clients_dict[path] = client
-        self.clients.sort(None, lambda c: c.properties["Name"])
+        self.clients.sort(key=lambda c: c.properties["Name"])
         self.refresh()
     
     def remove_client(self, client, path=None):
@@ -635,11 +561,6 @@
         if path is None:
             path = client.proxy.object_path
         del self.clients_dict[path]
-        if not self.clients_dict:
-            # Work around bug in Urwid 0.9.8.3 - if a SimpleListWalker
-            # is completely emptied, we need to recreate it.
-            self.clients = urwid.SimpleListWalker([])
-            self.rebuild()
         self.refresh()
     
     def refresh(self):
@@ -649,6 +570,52 @@
     
     def run(self):
         """Start the main loop and exit when it's done."""
+        self.bus = dbus.SystemBus()
+        mandos_dbus_objc = self.bus.get_object(
+            self.busname, "/", follow_name_owner_changes=True)
+        self.mandos_serv = dbus.Interface(mandos_dbus_objc,
+                                          dbus_interface
+                                          = server_interface)
+        try:
+            mandos_clients = (self.mandos_serv
+                              .GetAllClientsWithProperties())
+            if not mandos_clients:
+                self.log_message_raw(("bold", "Note: Server has no clients."))
+        except dbus.exceptions.DBusException:
+            self.log_message_raw(("bold", "Note: No Mandos server running."))
+            mandos_clients = dbus.Dictionary()
+        
+        (self.mandos_serv
+         .connect_to_signal("ClientRemoved",
+                            self.find_and_remove_client,
+                            dbus_interface=server_interface,
+                            byte_arrays=True))
+        (self.mandos_serv
+         .connect_to_signal("ClientAdded",
+                            self.add_new_client,
+                            dbus_interface=server_interface,
+                            byte_arrays=True))
+        (self.mandos_serv
+         .connect_to_signal("ClientNotFound",
+                            self.client_not_found,
+                            dbus_interface=server_interface,
+                            byte_arrays=True))
+        for path, client in mandos_clients.items():
+            client_proxy_object = self.bus.get_object(self.busname,
+                                                      path)
+            self.add_client(MandosClientWidget(server_proxy_object
+                                               =self.mandos_serv,
+                                               proxy_object
+                                               =client_proxy_object,
+                                               properties=client,
+                                               update_hook
+                                               =self.refresh,
+                                               delete_hook
+                                               =self.remove_client,
+                                               logger
+                                               =self.log_message),
+                            path=path)
+        
         self.refresh()
         self._input_callback_tag = (gobject.io_add_watch
                                     (sys.stdin.fileno(),
@@ -751,7 +718,7 @@
     ui.run()
 except KeyboardInterrupt:
     ui.screen.stop()
-except Exception, e:
-    ui.log_message(unicode(e))
+except Exception as e:
+    ui.log_message(str(e))
     ui.screen.stop()
     raise

=== modified file 'mandos-monitor.xml'
--- mandos-monitor.xml	2011-10-05 16:00:56 +0000
+++ mandos-monitor.xml	2011-12-31 23:05:34 +0000
@@ -2,7 +2,7 @@
 <!DOCTYPE refentry PUBLIC "-//OASIS//DTD DocBook XML V4.5//EN"
 	"http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd" [
 <!ENTITY COMMANDNAME "mandos-monitor">
-<!ENTITY TIMESTAMP "2011-10-03">
+<!ENTITY TIMESTAMP "2012-01-01">
 <!ENTITY % common SYSTEM "common.ent">
 %common;
 ]>
@@ -33,6 +33,7 @@
     <copyright>
       <year>2010</year>
       <year>2011</year>
+      <year>2012</year>
       <holder>Teddy Hogeborn</holder>
       <holder>Björn Påhlsson</holder>
     </copyright>

=== modified file 'mandos-options.xml'
--- mandos-options.xml	2011-11-26 22:22:20 +0000
+++ mandos-options.xml	2013-06-23 15:13:06 +0000
@@ -49,8 +49,9 @@
   <para id="priority">
     GnuTLS priority string for the <acronym>TLS</acronym> handshake.
     The default is <quote><literal
-    >SECURE256:!CTYPE-X.509:+CTYPE-OPENPGP</literal></quote>.  See
-    <citerefentry><refentrytitle>gnutls_priority_init</refentrytitle>
+    >SECURE256:!CTYPE-X.509:+CTYPE-OPENPGP:+SIGN-RSA-SHA224</literal></quote>.
+    See <citerefentry><refentrytitle
+    >gnutls_priority_init</refentrytitle>
     <manvolnum>3</manvolnum></citerefentry> for the syntax.
     <emphasis>Warning</emphasis>: changing this may make the
     <acronym>TLS</acronym> handshake fail, making server-client
@@ -87,8 +88,8 @@
   </para>
   
   <para id="restore">
-    This option controls whether the server will restore any state from
-    last time it ran. Default is to try restore last state.
+    This option controls whether the server will restore its state
+    from the last time it ran.  Default is to restore last state.
   </para>
   
   <para id="statedir">
@@ -97,4 +98,17 @@
     class="directory">/var/lib/mandos</filename></quote>.
   </para>
   
+  <para id="socket">
+    If this option is used, the server will not create a new network
+    socket, but will instead use the supplied file descriptor.  By
+    default, the server will create a new network socket.
+  </para>
+  
+  <para id="foreground">
+    This option will make the server run in the foreground and not
+    write a PID file.  The default is to <emphasis>not</emphasis> run
+    in the foreground, except in <option>debug</option> mode, which
+    implies this option.
+  </para>
+  
 </section>

=== modified file 'mandos.conf'
--- mandos.conf	2011-11-26 22:22:20 +0000
+++ mandos.conf	2013-06-23 15:13:06 +0000
@@ -23,7 +23,7 @@
 ;debug = False
 
 # GnuTLS priority for the TLS handshake.  See gnutls_priority_init(3).
-;priority = SECURE256:!CTYPE-X.509:+CTYPE-OPENPGP
+;priority = SECURE256:!CTYPE-X.509:+CTYPE-OPENPGP:+SIGN-RSA-SHA224
 
 # Zeroconf service name.  You need to change this if you for some
 # reason want to run more than one server on the same *host*.
@@ -42,3 +42,6 @@
 
 # The directory where state is saved
 ;statedir = /var/lib/mandos
+
+# Whether to run in the foreground
+;foreground = False

=== modified file 'mandos.conf.xml'
--- mandos.conf.xml	2011-11-26 22:22:20 +0000
+++ mandos.conf.xml	2012-05-26 22:21:17 +0000
@@ -3,7 +3,7 @@
 	"http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd" [
 <!ENTITY CONFNAME "mandos.conf">
 <!ENTITY CONFPATH "<filename>/etc/mandos/mandos.conf</filename>">
-<!ENTITY TIMESTAMP "2011-11-26">
+<!ENTITY TIMESTAMP "2012-05-26">
 <!ENTITY % common SYSTEM "common.ent">
 %common;
 ]>
@@ -35,6 +35,7 @@
       <year>2008</year>
       <year>2009</year>
       <year>2011</year>
+      <year>2012</year>
       <holder>Teddy Hogeborn</holder>
       <holder>Björn Påhlsson</holder>
     </copyright>
@@ -173,6 +174,14 @@
 	</listitem>
       </varlistentry>
       
+      <varlistentry>
+	<term><option>socket<literal> = </literal><replaceable
+	>NUMBER</replaceable></option></term>
+	<listitem>
+	  <xi:include href="mandos-options.xml" xpointer="socket"/>
+	</listitem>
+      </varlistentry>
+      
     </variablelist>
   </refsect1>
   

=== modified file 'mandos.lsm'
--- mandos.lsm	2011-10-15 16:48:03 +0000
+++ mandos.lsm	2012-06-17 22:26:40 +0000
@@ -1,7 +1,7 @@
 Begin4
 Title:		Mandos
-Version:	1.4.1
-Entered-date:	2011-10-15
+Version:	1.6.0
+Entered-date:	2012-06-18
 Description:	The Mandos system allows computers to have encrypted
 root file systems and at the same time be capable of remote and/or
 unattended reboots.
@@ -12,9 +12,9 @@
 Maintained-by:	teddy@recompile.se (Teddy Hogeborn),
 		belorn@recompile.se (Björn Påhlsson)
 Primary-site: http://www.recompile.se/mandos
-	      138K mandos_1.4.1.orig.tar.gz
+	      150K mandos_1.6.0.orig.tar.gz
 Alternate-site: ftp://ftp.recompile.se/pub/mandos
-		138K mandos_1.4.1.orig.tar.gz
+		150K mandos_1.6.0.orig.tar.gz
 Platforms:	Requires GCC, GNU libC, Avahi, GnuPG, Python 2.6, and
 various other libraries.  While made for Debian GNU/Linux, it is
 probably portable to other distributions, but not other Unixes.

=== modified file 'mandos.xml'
--- mandos.xml	2011-12-21 00:29:33 +0000
+++ mandos.xml	2012-06-17 14:55:31 +0000
@@ -2,7 +2,7 @@
 <!DOCTYPE refentry PUBLIC "-//OASIS//DTD DocBook XML V4.5//EN"
 "http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd" [
 <!ENTITY COMMANDNAME "mandos">
-<!ENTITY TIMESTAMP "2011-11-26">
+<!ENTITY TIMESTAMP "2012-06-17">
 <!ENTITY % common SYSTEM "common.ent">
 %common;
 ]>
@@ -35,6 +35,7 @@
       <year>2009</year>
       <year>2010</year>
       <year>2011</year>
+      <year>2012</year>
       <holder>Teddy Hogeborn</holder>
       <holder>Björn Påhlsson</holder>
     </copyright>
@@ -99,6 +100,11 @@
       <sbr/>
       <arg><option>--statedir
       <replaceable>DIRECTORY</replaceable></option></arg>
+      <sbr/>
+      <arg><option>--socket
+      <replaceable>FD</replaceable></option></arg>
+      <sbr/>
+      <arg><option>--foreground</option></arg>
     </cmdsynopsis>
     <cmdsynopsis>
       <command>&COMMANDNAME;</command>
@@ -285,6 +291,9 @@
 	<term><option>--no-restore</option></term>
 	<listitem>
 	  <xi:include href="mandos-options.xml" xpointer="restore"/>
+	  <para>
+	    See also <xref linkend="persistent_state"/>.
+	  </para>
 	</listitem>
       </varlistentry>
       
@@ -295,6 +304,23 @@
 	  <xi:include href="mandos-options.xml" xpointer="statedir"/>
 	</listitem>
       </varlistentry>
+      
+      <varlistentry>
+	<term><option>--socket
+	<replaceable>FD</replaceable></option></term>
+	<listitem>
+	  <xi:include href="mandos-options.xml" xpointer="socket"/>
+	</listitem>
+      </varlistentry>
+      
+      <varlistentry>
+	<term><option>--foreground</option></term>
+	<listitem>
+	  <xi:include href="mandos-options.xml"
+		      xpointer="foreground"/>
+	</listitem>
+      </varlistentry>
+      
     </variablelist>
   </refsect1>
   
@@ -377,9 +403,7 @@
       extended timeout, checker program, and interval between checks
       can be configured both globally and per client; see
       <citerefentry><refentrytitle>mandos-clients.conf</refentrytitle>
-      <manvolnum>5</manvolnum></citerefentry>.  A client successfully
-      receiving its password will also be treated as a successful
-      checker run.
+      <manvolnum>5</manvolnum></citerefentry>.
     </para>
   </refsect1>
   
@@ -413,6 +437,18 @@
     </para>
   </refsect1>
   
+  <refsect1 id="persistent_state">
+    <title>PERSISTENT STATE</title>
+    <para>
+      Client settings, initially read from
+      <filename>clients.conf</filename>, are persistent across
+      restarts, and run-time changes will override settings in
+      <filename>clients.conf</filename>.  However, if a setting is
+      <emphasis>changed</emphasis> (or a client added, or removed) in
+      <filename>clients.conf</filename>, this will take precedence.
+    </para>
+  </refsect1>
+  
   <refsect1 id="dbus_interface">
     <title>D-BUS INTERFACE</title>
     <para>
@@ -535,9 +571,6 @@
       There is no fine-grained control over logging and debug output.
     </para>
     <para>
-      Debug mode is conflated with running in the foreground.
-    </para>
-    <para>
       This server does not check the expire time of clients’ OpenPGP
       keys.
     </para>

=== modified file 'network-hooks.d/bridge'
--- network-hooks.d/bridge	2011-12-31 13:25:58 +0000
+++ network-hooks.d/bridge	2012-06-13 22:06:57 +0000
@@ -6,6 +6,9 @@
 # configuration file(s) should be copied into the
 # /etc/mandos/network-hooks.d directory.
 # 
+# Copyright © 2012 Teddy Hogeborn
+# Copyright © 2012 Björn Påhlsson
+# 
 # Copying and distribution of this file, with or without modification,
 # are permitted in any medium without royalty provided the copyright
 # notice and this notice are preserved.  This file is offered as-is,
@@ -16,7 +19,8 @@
 CONFIG="$MANDOSNETHOOKDIR/bridge.conf"
 
 addrtoif(){
-    grep -liFe "$1" /sys/class/net/*/address | sed -e 's,.*/\([^/]*\)/[^/]*,\1,'
+    grep -liFe "$1" /sys/class/net/*/address \
+	| sed -e 's,.*/\([^/]*\)/[^/]*,\1,' -e "/^${BRIDGE}\$/d"
 }
 
 # Read config file, which must set "BRIDGE", "PORT_ADDRESSES", and
@@ -29,46 +33,55 @@
     exit
 fi
 
-if [ -n "$DEVICE" -a "$DEVICE" != "$BRIDGE" ]; then
-    exit
+if [ -n "$DEVICE" ]; then
+    case "$DEVICE" in
+	*,"$BRIDGE"|*,"$BRIDGE",*|"$BRIDGE",*|"$BRIDGE") :;;
+	*) exit;;
+    esac
 fi
 
-for b in /sbin/brctl /usr/sbin/brctl; do
+brctl="/sbin/brctl"
+for b in "$brctl" /usr/sbin/brctl; do
     if [ -e "$b" ]; then
 	brctl="$b"
 	break
     fi
 done
 
-case "$1" in
-    start)
-	"$brctl" addbr "$BRIDGE"
-	for address in $PORT_ADDRESSES; do
-	    interface=`addrtoif "$address"`
-	    "$brctl" addif "$BRIDGE" "$interface"
-	    ip link set dev "$interface" up
-	done
-	ip link set dev "$BRIDGE" up
-	sleep "$DELAY"
-	if [ -n "$IPADDRS" ]; then
-            for ipaddr in $IPADDRS; do
-		ip addr add "$ipaddr" dev "$BRIDGE"
-	    done
-	fi
-	if [ -n "$ROUTES" ]; then
-            for route in $ROUTES; do
-		ip route add "$route" dev "$BRIDGE"
-	    done
-	fi
-	;;
-    stop)
-	ip link set dev "$BRIDGE" down
-	for address in $PORT_ADDRESSES; do
-	    interface=`addrtoif "$address"`
-	    ip link set dev "$interface" down
-	    "$brctl" delif "$BRIDGE" "$interface"
-	done
-	"$brctl" delbr "$BRIDGE"
+do_start(){
+    "$brctl" addbr "$BRIDGE"
+    for address in $PORT_ADDRESSES; do
+	interface=`addrtoif "$address"`
+	"$brctl" addif "$BRIDGE" "$interface"
+	ip link set dev "$interface" up
+    done
+    ip link set dev "$BRIDGE" up
+    sleep "${DELAY%%.*}"
+    if [ -n "$IPADDRS" ]; then
+        for ipaddr in $IPADDRS; do
+	    ip addr add "$ipaddr" dev "$BRIDGE"
+	done
+    fi
+    if [ -n "$ROUTES" ]; then
+        for route in $ROUTES; do
+	    ip route add "$route" dev "$BRIDGE"
+	done
+    fi
+}
+
+do_stop(){
+    ip link set dev "$BRIDGE" down
+    for address in $PORT_ADDRESSES; do
+	interface=`addrtoif "$address"`
+	ip link set dev "$interface" down
+	"$brctl" delif "$BRIDGE" "$interface"
+    done
+    "$brctl" delbr "$BRIDGE"
+}
+
+case "${MODE:-$1}" in
+    start|stop)
+	do_"${MODE:-$1}"
 	;;
     files)
 	echo /bin/ip

=== modified file 'network-hooks.d/openvpn'
--- network-hooks.d/openvpn	2011-12-02 16:52:50 +0000
+++ network-hooks.d/openvpn	2012-06-13 22:06:57 +0000
@@ -6,6 +6,9 @@
 # configuration file(s) should be copied into the
 # /etc/mandos/network-hooks.d directory.
 # 
+# Copyright © 2012 Teddy Hogeborn
+# Copyright © 2012 Björn Påhlsson
+# 
 # Copying and distribution of this file, with or without modification,
 # are permitted in any medium without royalty provided the copyright
 # notice and this notice are preserved.  This file is offered as-is,
@@ -16,7 +19,9 @@
 CONFIG="openvpn.conf"
 
 # Extract the "dev" setting from the config file
-VPNDEVICE="`sed -n -e 's/#.*//' -e 's/^[[:space:]]*dev[[:space:]]\+//p' \"$MANDOSNETHOOKDIR/$CONFIG\"`"
+VPNDEVICE=`sed -n -e 's/[[:space:]]#.*//' \
+    -e 's/^[[:space:]]*dev[[:space:]]\+//p' \
+    "$MANDOSNETHOOKDIR/$CONFIG"`
 
 PIDFILE=/run/openvpn-mandos.pid
 
@@ -26,22 +31,31 @@
 fi
 
 # Exit if DEVICE is set and it doesn't match the VPN interface
-if [ -n "$DEVICE" -a "$DEVICE" = "${DEVICE#$VPNDEVICE}" ]; then
-    exit
+if [ -n "$DEVICE" ]; then
+    case "$DEVICE" in
+	*,"$VPNDEVICE"*|"$VPNDEVICE"*) :;;
+	*) exit;;
+    esac
 fi
 
 openvpn=/usr/sbin/openvpn
 
-case "$1" in
-    start)
-	"$openvpn" --cd "$MANDOSNETHOOKDIR" --daemon 'openvpn(Mandos)' --writepid "$PIDFILE" --config "$CONFIG"
-	sleep "$DELAY"
-	;;
-    stop)
-	PID="`cat \"$PIDFILE\"`"
-	if [ "$PID" -gt 0 ]; then
-	    kill "$PID"
-	fi
+do_start(){
+    "$openvpn" --cd "$MANDOSNETHOOKDIR" --daemon 'openvpn(Mandos)' \
+	--writepid "$PIDFILE" --config "$CONFIG"
+     sleep "$DELAY"
+}
+
+do_stop(){
+    PID="`cat \"$PIDFILE\"`"
+    if [ "$PID" -gt 0 ]; then
+	kill "$PID"
+    fi
+}
+
+case "${MODE:-$1}" in
+    start|stop)
+	do_"${MODE:-$1}"
 	;;
     files)
 	echo "$openvpn"

=== modified file 'network-hooks.d/wireless'
--- network-hooks.d/wireless	2011-12-31 13:25:58 +0000
+++ network-hooks.d/wireless	2012-06-13 22:06:57 +0000
@@ -6,6 +6,9 @@
 # configuration file(s) should be copied into the
 # /etc/mandos/network-hooks.d directory.
 # 
+# Copyright © 2012 Teddy Hogeborn
+# Copyright © 2012 Björn Påhlsson
+# 
 # Copying and distribution of this file, with or without modification,
 # are permitted in any medium without royalty provided the copyright
 # notice and this notice are preserved.  This file is offered as-is,
@@ -21,7 +24,8 @@
 CONFIG="$MANDOSNETHOOKDIR/wireless.conf"
 
 addrtoif(){
-    grep -liFe "$1" /sys/class/net/*/address | sed -e 's,.*/\([^/]*\)/[^/]*,\1,'
+    grep -liFe "$1" /sys/class/net/*/address \
+	| sed -e 's,.*/\([^/]*\)/[^/]*,\1,'
 }
 
 # Read config file
@@ -31,7 +35,7 @@
     exit
 fi
 
-ifkeys="`env | sed -n -e 's/^ADDRESS_\([^=]*\)=.*/\1/p' \"$CONFIG\" | sort -u`"
+ifkeys=`sed -n -e 's/^ADDRESS_\([^=]*\)=.*/\1/p' "$CONFIG" | sort -u`
 
 # Exit if DEVICE is set and is not any of the wireless interfaces
 if [ -n "$DEVICE" ]; then
@@ -39,9 +43,11 @@
 	for KEY in $ifkeys; do
 	    ADDRESS=`eval 'echo "$ADDRESS_'"$KEY"\"`
 	    INTERFACE=`addrtoif "$ADDRESS"`
-	    if [ "$INTERFACE" = "$DEVICE" ]; then
-		break 2
-	    fi
+	    
+	    case "$DEVICE" in
+		*,"$INTERFACE"|*,"$INTERFACE",*|"$INTERFACE",*|"$INTERFACE")
+		    break 2;;
+	    esac
 	done
 	exit
     done
@@ -68,75 +74,81 @@
     WPAS_OPTIONS="-P$PIDFILE $WPAS_OPTIONS"
 fi
 
-case "${MODE:-$1}" in
-    start)
-	mkdir -m u=rwx,go= -p "$CTRLDIR"
-	"$wpa_supplicant" -B -g "$CTRL" -p "$CTRLDIR" $WPAS_OPTIONS
-        for KEY in $ifkeys; do
-	    ADDRESS=`eval 'echo "$ADDRESS_'"$KEY"\"`
-	    INTERFACE=`addrtoif "$ADDRESS"`
-	    DRIVER=`eval 'echo "$WPA_DRIVER_'"$KEY"\"`
-	    IFDELAY=`eval 'echo "$DELAY_'"$KEY"\"`
-	    "$wpa_cli" -g "$CTRL" interface_add "$INTERFACE" "" \
-		"${DRIVER:-wext}" "$CTRLDIR" > /dev/null \
-		| sed -e '/^OK$/d'
-	    NETWORK=`"$wpa_cli" -p "$CTRLDIR" -i "$INTERFACE" add_network`
-	    eval wpa_interface_"$KEY"
-	    "$wpa_cli" -p "$CTRLDIR" -i "$INTERFACE" enable_network \
-		"$NETWORK" | sed -e '/^OK$/d'
-	    sleep "${IFDELAY:-$DELAY}" &
-	    sleep=$!
-	    while :; do
-		kill -0 $sleep 2>/dev/null || break
-		STATE=`"$wpa_cli" -p "$CTRLDIR" -i "$INTERFACE" status | sed -n -e 's/^wpa_state=//p'`
-		if [ "$STATE" = COMPLETED ]; then
-		    while :; do
-			kill -0 $sleep 2>/dev/null || break 2
-			UP=`cat /sys/class/net/"$INTERFACE"/operstate`
-			if [ "$UP" = up ]; then
-			    kill $sleep 2>/dev/null
-			    break 2
-			fi
-			sleep 1
-		    done
-		fi
-		sleep 1
-	    done &
-	    wait $sleep || :
-	    IPADDRS=`eval 'echo "$IPADDRS_'"$KEY"\"`
-	    if [ -n "$IPADDRS" ]; then
-		if [ "$IPADDRS" = dhcp ]; then
-		    ipconfig -c dhcp -d "$INTERFACE" || :
-		    #dhclient "$INTERFACE"
-		else
-		    for ipaddr in $IPADDRS; do
-			"$ip" addr add "$ipaddr" dev "$INTERFACE"
-		    done
-		fi
-	    fi
-	    ROUTES=`eval 'echo "$ROUTES_'"$KEY"\"`
-	    if [ -n "$ROUTES" ]; then
-		for route in $ROUTES; do
-		    "$ip" route add "$route" dev "$BRIDGE"
-		done
-	    fi
-        done
-	;;
-    stop)
-	"$wpa_cli" -g "$CTRL" terminate 2>&1 | sed -e '/^OK$/d'
-        for KEY in $ifkeys; do
-	    ADDRESS=`eval 'echo "$ADDRESS_'"$KEY"\"`
-	    INTERFACE=`addrtoif "$ADDRESS"`
-	    "$ip" addr show scope global permanent dev "$INTERFACE" \
-		| while read type addr rest; do
+do_start(){
+    mkdir -m u=rwx,go= -p "$CTRLDIR"
+    "$wpa_supplicant" -B -g "$CTRL" -p "$CTRLDIR" $WPAS_OPTIONS
+    for KEY in $ifkeys; do
+	ADDRESS=`eval 'echo "$ADDRESS_'"$KEY"\"`
+	INTERFACE=`addrtoif "$ADDRESS"`
+	DRIVER=`eval 'echo "$WPA_DRIVER_'"$KEY"\"`
+	IFDELAY=`eval 'echo "$DELAY_'"$KEY"\"`
+	"$wpa_cli" -g "$CTRL" interface_add "$INTERFACE" "" \
+	    "${DRIVER:-wext}" "$CTRLDIR" > /dev/null \
+	    | sed -e '/^OK$/d'
+        NETWORK=`"$wpa_cli" -p "$CTRLDIR" -i "$INTERFACE" add_network`
+	eval wpa_interface_"$KEY"
+	"$wpa_cli" -p "$CTRLDIR" -i "$INTERFACE" enable_network \
+	    "$NETWORK" | sed -e '/^OK$/d'
+	sleep "${IFDELAY:-$DELAY}" &
+	sleep=$!
+	while :; do
+	    kill -0 $sleep 2>/dev/null || break
+	    STATE=`"$wpa_cli" -p "$CTRLDIR" -i "$INTERFACE" status \
+		| sed -n -e 's/^wpa_state=//p'`
+	    if [ "$STATE" = COMPLETED ]; then
+		while :; do
+		    kill -0 $sleep 2>/dev/null || break 2
+		    UP=`cat /sys/class/net/"$INTERFACE"/operstate`
+		    if [ "$UP" = up ]; then
+			kill $sleep 2>/dev/null
+			break 2
+		    fi
+		    sleep 1
+		done
+	    fi
+	    sleep 1
+	done &
+	wait $sleep || :
+	IPADDRS=`eval 'echo "$IPADDRS_'"$KEY"\"`
+	if [ -n "$IPADDRS" ]; then
+	    if [ "$IPADDRS" = dhcp ]; then
+		ipconfig -c dhcp -d "$INTERFACE" || :
+		#dhclient "$INTERFACE"
+	    else
+		for ipaddr in $IPADDRS; do
+		    "$ip" addr add "$ipaddr" dev "$INTERFACE"
+		done
+	    fi
+	fi
+	ROUTES=`eval 'echo "$ROUTES_'"$KEY"\"`
+	if [ -n "$ROUTES" ]; then
+	    for route in $ROUTES; do
+		"$ip" route add "$route" dev "$INTERFACE"
+	    done
+	fi
+    done
+}
+
+do_stop(){
+    "$wpa_cli" -g "$CTRL" terminate 2>&1 | sed -e '/^OK$/d'
+    for KEY in $ifkeys; do
+	ADDRESS=`eval 'echo "$ADDRESS_'"$KEY"\"`
+	INTERFACE=`addrtoif "$ADDRESS"`
+	"$ip" addr show scope global permanent dev "$INTERFACE" \
+	    | while read type addr rest; do
 		case "$type" in
 		    inet|inet6)
 			"$ip" addr del "$addr" dev "$INTERFACE"
 			;;
 		esac
 	    done
-	    "$ip" link set dev "$INTERFACE" down
-	done
+	"$ip" link set dev "$INTERFACE" down
+    done
+}
+
+case "${MODE:-$1}" in
+    start|stop)
+	do_"${MODE:-$1}"
 	;;
     files)
 	echo "$wpa_supplicant"

=== modified file 'plugin-runner.c'
--- plugin-runner.c	2011-12-25 00:40:09 +0000
+++ plugin-runner.c	2013-08-27 21:47:35 +0000
@@ -2,8 +2,8 @@
 /*
  * Mandos plugin runner - Run Mandos plugins
  *
- * Copyright © 2008-2011 Teddy Hogeborn
- * Copyright © 2008-2011 Björn Påhlsson
+ * Copyright © 2008-2012 Teddy Hogeborn
+ * Copyright © 2008-2012 Björn Påhlsson
  * 
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -426,7 +426,7 @@
     errno = 0;
     switch(key){
       char *tmp;
-      intmax_t tmpmax;
+      intmax_t tmp_id;
     case 'g': 			/* --global-options */
       {
 	char *plugin_option;
@@ -505,24 +505,24 @@
       /* This is already done by parse_opt_config_file() */
       break;
     case 130:			/* --userid */
-      tmpmax = strtoimax(arg, &tmp, 10);
+      tmp_id = strtoimax(arg, &tmp, 10);
       if(errno != 0 or tmp == arg or *tmp != '\0'
-	 or tmpmax != (uid_t)tmpmax){
+	 or tmp_id != (uid_t)tmp_id){
 	argp_error(state, "Bad user ID number: \"%s\", using %"
 		   PRIdMAX, arg, (intmax_t)uid);
 	break;
       }
-      uid = (uid_t)tmpmax;
+      uid = (uid_t)tmp_id;
       break;
     case 131:			/* --groupid */
-      tmpmax = strtoimax(arg, &tmp, 10);
+      tmp_id = strtoimax(arg, &tmp, 10);
       if(errno != 0 or tmp == arg or *tmp != '\0'
-	 or tmpmax != (gid_t)tmpmax){
+	 or tmp_id != (gid_t)tmp_id){
 	argp_error(state, "Bad group ID number: \"%s\", using %"
 		   PRIdMAX, arg, (intmax_t)gid);
 	break;
       }
-      gid = (gid_t)tmpmax;
+      gid = (gid_t)tmp_id;
       break;
     case 132:			/* --debug */
       debug = true;
@@ -771,7 +771,7 @@
   }
   
   /* Lower permissions */
-  setgid(gid);
+  ret = setgid(gid);
   if(ret == -1){
     error(0, errno, "setgid");
   }

=== modified file 'plugin-runner.xml'
--- plugin-runner.xml	2011-10-05 16:56:06 +0000
+++ plugin-runner.xml	2011-12-31 23:05:34 +0000
@@ -2,7 +2,7 @@
 <!DOCTYPE refentry PUBLIC "-//OASIS//DTD DocBook XML V4.5//EN"
 	"http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd" [
 <!ENTITY COMMANDNAME "plugin-runner">
-<!ENTITY TIMESTAMP "2011-10-05">
+<!ENTITY TIMESTAMP "2012-01-01">
 <!ENTITY % common SYSTEM "common.ent">
 %common;
 ]>
@@ -33,7 +33,7 @@
     <copyright>
       <year>2008</year>
       <year>2009</year>
-      <year>2011</year>
+      <year>2012</year>
       <holder>Teddy Hogeborn</holder>
       <holder>Björn Påhlsson</holder>
     </copyright>

=== modified file 'plugins.d/askpass-fifo.c'
--- plugins.d/askpass-fifo.c	2011-12-24 23:17:02 +0000
+++ plugins.d/askpass-fifo.c	2011-12-31 23:05:34 +0000
@@ -2,8 +2,8 @@
 /*
  * Askpass-FIFO - Read a password from a FIFO and output it
  * 
- * Copyright © 2008-2011 Teddy Hogeborn
- * Copyright © 2008-2011 Björn Påhlsson
+ * Copyright © 2008-2012 Teddy Hogeborn
+ * Copyright © 2008-2012 Björn Påhlsson
  * 
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as

=== modified file 'plugins.d/askpass-fifo.xml'
--- plugins.d/askpass-fifo.xml	2011-10-05 16:00:56 +0000
+++ plugins.d/askpass-fifo.xml	2011-12-31 23:05:34 +0000
@@ -2,7 +2,7 @@
 <!DOCTYPE refentry PUBLIC "-//OASIS//DTD DocBook XML V4.5//EN"
 	"http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd" [
 <!ENTITY COMMANDNAME "askpass-fifo">
-<!ENTITY TIMESTAMP "2011-10-03">
+<!ENTITY TIMESTAMP "2012-01-01">
 <!ENTITY % common SYSTEM "../common.ent">
 %common;
 ]>
@@ -34,6 +34,7 @@
       <year>2008</year>
       <year>2009</year>
       <year>2011</year>
+      <year>2012</year>
       <holder>Teddy Hogeborn</holder>
       <holder>Björn Påhlsson</holder>
     </copyright>

=== modified file 'plugins.d/mandos-client.c'
--- plugins.d/mandos-client.c	2011-12-30 18:48:49 +0000
+++ plugins.d/mandos-client.c	2013-10-05 19:34:40 +0000
@@ -9,8 +9,8 @@
  * "browse_callback", and parts of "main".
  * 
  * Everything else is
- * Copyright © 2008-2011 Teddy Hogeborn
- * Copyright © 2008-2011 Björn Påhlsson
+ * Copyright © 2008-2012 Teddy Hogeborn
+ * Copyright © 2008-2012 Björn Påhlsson
  * 
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -41,7 +41,7 @@
 
 #include <stdio.h>		/* fprintf(), stderr, fwrite(),
 				   stdout, ferror(), remove() */
-#include <stdint.h> 		/* uint16_t, uint32_t */
+#include <stdint.h> 		/* uint16_t, uint32_t, intptr_t */
 #include <stddef.h>		/* NULL, size_t, ssize_t */
 #include <stdlib.h> 		/* free(), EXIT_SUCCESS, srand(),
 				   strtof(), abort() */
@@ -61,7 +61,6 @@
 				 */
 #include <inttypes.h>		/* PRIu16, PRIdMAX, intmax_t,
 				   strtoimax() */
-#include <assert.h>		/* assert() */
 #include <errno.h>		/* perror(), errno,
 				   program_invocation_short_name */
 #include <time.h>		/* nanosleep(), time(), sleep() */
@@ -88,6 +87,10 @@
 #include <sys/wait.h>		/* waitpid(), WIFEXITED(),
 				   WEXITSTATUS(), WTERMSIG() */
 #include <grp.h>		/* setgroups() */
+#include <argz.h>		/* argz_add_sep(), argz_next(),
+				   argz_delete(), argz_append(),
+				   argz_stringify(), argz_add(),
+				   argz_count() */
 
 #ifdef __linux__
 #include <sys/klog.h> 		/* klogctl() */
@@ -135,11 +138,13 @@
 static const char sys_class_net[] = "/sys/class/net";
 char *connect_to = NULL;
 const char *hookdir = HOOKDIR;
+uid_t uid = 65534;
+gid_t gid = 65534;
 
 /* Doubly linked list that need to be circularly linked when used */
 typedef struct server{
   const char *ip;
-  uint16_t port;
+  in_port_t port;
   AvahiIfIndex if_index;
   int af;
   struct timespec last_seen;
@@ -149,7 +154,6 @@
 
 /* Used for passing in values through the Avahi callback functions */
 typedef struct {
-  AvahiSimplePoll *simple_poll;
   AvahiServer *server;
   gnutls_certificate_credentials_t cred;
   unsigned int dh_bits;
@@ -157,13 +161,12 @@
   const char *priority;
   gpgme_ctx_t ctx;
   server *current_server;
+  char *interfaces;
+  size_t interfaces_size;
 } mandos_context;
 
-/* global context so signal handler can reach it*/
-mandos_context mc = { .simple_poll = NULL, .server = NULL,
-		      .dh_bits = 1024, .priority = "SECURE256"
-		      ":!CTYPE-X.509:+CTYPE-OPENPGP",
-		      .current_server = NULL };
+/* global so signal handler can reach it*/
+AvahiSimplePoll *simple_poll;
 
 sig_atomic_t quit_now = 0;
 int signal_received = 0;
@@ -184,7 +187,7 @@
   
   TEMP_FAILURE_RETRY(fprintf(stream, "Mandos plugin %s: ",
 			     program_invocation_short_name));
-  return TEMP_FAILURE_RETRY(vfprintf(stream, format, ap));
+  return (int)TEMP_FAILURE_RETRY(vfprintf(stream, format, ap));
 }
 
 /*
@@ -205,8 +208,8 @@
 }
 
 /* Add server to set of servers to retry periodically */
-bool add_server(const char *ip, uint16_t port, AvahiIfIndex if_index,
-		int af){
+bool add_server(const char *ip, in_port_t port, AvahiIfIndex if_index,
+		int af, server **current_server){
   int ret;
   server *new_server = malloc(sizeof(server));
   if(new_server == NULL){
@@ -222,18 +225,18 @@
     return false;
   }
   /* Special case of first server */
-  if (mc.current_server == NULL){
+  if(*current_server == NULL){
     new_server->next = new_server;
     new_server->prev = new_server;
-    mc.current_server = new_server;
+    *current_server = new_server;
   /* Place the new server last in the list */
   } else {
-    new_server->next = mc.current_server;
-    new_server->prev = mc.current_server->prev;
+    new_server->next = *current_server;
+    new_server->prev = (*current_server)->prev;
     new_server->prev->next = new_server;
-    mc.current_server->prev = new_server;
+    (*current_server)->prev = new_server;
   }
-  ret = clock_gettime(CLOCK_MONOTONIC, &mc.current_server->last_seen);
+  ret = clock_gettime(CLOCK_MONOTONIC, &(*current_server)->last_seen);
   if(ret == -1){
     perror_plus("clock_gettime");
     return false;
@@ -245,11 +248,10 @@
  * Initialize GPGME.
  */
 static bool init_gpgme(const char *seckey, const char *pubkey,
-		       const char *tempdir){
+		       const char *tempdir, mandos_context *mc){
   gpgme_error_t rc;
   gpgme_engine_info_t engine_info;
   
-  
   /*
    * Helper function to insert pub and seckey to the engine keyring.
    */
@@ -271,7 +273,7 @@
       return false;
     }
     
-    rc = gpgme_op_import(mc.ctx, pgp_data);
+    rc = gpgme_op_import(mc->ctx, pgp_data);
     if(rc != GPG_ERR_NO_ERROR){
       fprintf_plus(stderr, "bad gpgme_op_import: %s: %s\n",
 		   gpgme_strsource(rc), gpgme_strerror(rc));
@@ -321,7 +323,7 @@
   }
   
   /* Create new GPGME "context" */
-  rc = gpgme_new(&(mc.ctx));
+  rc = gpgme_new(&(mc->ctx));
   if(rc != GPG_ERR_NO_ERROR){
     fprintf_plus(stderr, "Mandos plugin mandos-client: "
 		 "bad gpgme_new: %s: %s\n", gpgme_strsource(rc),
@@ -342,7 +344,8 @@
  */
 static ssize_t pgp_packet_decrypt(const char *cryptotext,
 				  size_t crypto_size,
-				  char **plaintext){
+				  char **plaintext,
+				  mandos_context *mc){
   gpgme_data_t dh_crypto, dh_plain;
   gpgme_error_t rc;
   ssize_t ret;
@@ -374,14 +377,14 @@
   
   /* Decrypt data from the cryptotext data buffer to the plaintext
      data buffer */
-  rc = gpgme_op_decrypt(mc.ctx, dh_crypto, dh_plain);
+  rc = gpgme_op_decrypt(mc->ctx, dh_crypto, dh_plain);
   if(rc != GPG_ERR_NO_ERROR){
     fprintf_plus(stderr, "bad gpgme_op_decrypt: %s: %s\n",
 		 gpgme_strsource(rc), gpgme_strerror(rc));
     plaintext_length = -1;
     if(debug){
       gpgme_decrypt_result_t result;
-      result = gpgme_op_decrypt_result(mc.ctx);
+      result = gpgme_op_decrypt_result(mc->ctx);
       if(result == NULL){
 	fprintf_plus(stderr, "gpgme_op_decrypt_result failed\n");
       } else {
@@ -465,8 +468,7 @@
 }
 
 static const char * safer_gnutls_strerror(int value){
-  const char *ret = gnutls_strerror(value); /* Spurious warning from
-					       -Wunreachable-code */
+  const char *ret = gnutls_strerror(value);
   if(ret == NULL)
     ret = "(unknown)";
   return ret;
@@ -479,7 +481,8 @@
 }
 
 static int init_gnutls_global(const char *pubkeyfilename,
-			      const char *seckeyfilename){
+			      const char *seckeyfilename,
+			      mandos_context *mc){
   int ret;
   
   if(debug){
@@ -502,7 +505,7 @@
   }
   
   /* OpenPGP credentials */
-  ret = gnutls_certificate_allocate_credentials(&mc.cred);
+  ret = gnutls_certificate_allocate_credentials(&mc->cred);
   if(ret != GNUTLS_E_SUCCESS){
     fprintf_plus(stderr, "GnuTLS memory error: %s\n",
 		 safer_gnutls_strerror(ret));
@@ -518,7 +521,7 @@
   }
   
   ret = gnutls_certificate_set_openpgp_key_file
-    (mc.cred, pubkeyfilename, seckeyfilename,
+    (mc->cred, pubkeyfilename, seckeyfilename,
      GNUTLS_OPENPGP_FMT_BASE64);
   if(ret != GNUTLS_E_SUCCESS){
     fprintf_plus(stderr,
@@ -530,33 +533,34 @@
   }
   
   /* GnuTLS server initialization */
-  ret = gnutls_dh_params_init(&mc.dh_params);
+  ret = gnutls_dh_params_init(&mc->dh_params);
   if(ret != GNUTLS_E_SUCCESS){
     fprintf_plus(stderr, "Error in GnuTLS DH parameter"
 		 " initialization: %s\n",
 		 safer_gnutls_strerror(ret));
     goto globalfail;
   }
-  ret = gnutls_dh_params_generate2(mc.dh_params, mc.dh_bits);
+  ret = gnutls_dh_params_generate2(mc->dh_params, mc->dh_bits);
   if(ret != GNUTLS_E_SUCCESS){
     fprintf_plus(stderr, "Error in GnuTLS prime generation: %s\n",
 		 safer_gnutls_strerror(ret));
     goto globalfail;
   }
   
-  gnutls_certificate_set_dh_params(mc.cred, mc.dh_params);
+  gnutls_certificate_set_dh_params(mc->cred, mc->dh_params);
   
   return 0;
   
  globalfail:
   
-  gnutls_certificate_free_credentials(mc.cred);
+  gnutls_certificate_free_credentials(mc->cred);
   gnutls_global_deinit();
-  gnutls_dh_params_deinit(mc.dh_params);
+  gnutls_dh_params_deinit(mc->dh_params);
   return -1;
 }
 
-static int init_gnutls_session(gnutls_session_t *session){
+static int init_gnutls_session(gnutls_session_t *session,
+			       mandos_context *mc){
   int ret;
   /* GnuTLS session creation */
   do {
@@ -574,7 +578,7 @@
   {
     const char *err;
     do {
-      ret = gnutls_priority_set_direct(*session, mc.priority, &err);
+      ret = gnutls_priority_set_direct(*session, mc->priority, &err);
       if(quit_now){
 	gnutls_deinit(*session);
 	return -1;
@@ -591,7 +595,7 @@
   
   do {
     ret = gnutls_credentials_set(*session, GNUTLS_CRD_CERTIFICATE,
-				 mc.cred);
+				 mc->cred);
     if(quit_now){
       gnutls_deinit(*session);
       return -1;
@@ -607,7 +611,7 @@
   /* ignore client certificate if any. */
   gnutls_certificate_server_set_request(*session, GNUTLS_CERT_IGNORE);
   
-  gnutls_dh_set_prime_bits(*session, mc.dh_bits);
+  gnutls_dh_set_prime_bits(*session, mc->dh_bits);
   
   return 0;
 }
@@ -617,9 +621,9 @@
 		      __attribute__((unused)) const char *txt){}
 
 /* Called when a Mandos server is found */
-static int start_mandos_communication(const char *ip, uint16_t port,
+static int start_mandos_communication(const char *ip, in_port_t port,
 				      AvahiIfIndex if_index,
-				      int af){
+				      int af, mandos_context *mc){
   int ret, tcp_sd = -1;
   ssize_t sret;
   union {
@@ -655,14 +659,46 @@
     return -1;
   }
   
-  ret = init_gnutls_session(&session);
+  /* If the interface is specified and we have a list of interfaces */
+  if(if_index != AVAHI_IF_UNSPEC and mc->interfaces != NULL){
+    /* Check if the interface is one of the interfaces we are using */
+    bool match = false;
+    {
+      char *interface = NULL;
+      while((interface=argz_next(mc->interfaces, mc->interfaces_size,
+				 interface))){
+	if(if_nametoindex(interface) == (unsigned int)if_index){
+	  match = true;
+	  break;
+	}
+      }
+    }
+    if(not match){
+      /* This interface does not match any in the list, so we don't
+	 connect to the server */
+      if(debug){
+	char interface[IF_NAMESIZE];
+	if(if_indextoname((unsigned int)if_index, interface) == NULL){
+	  perror_plus("if_indextoname");
+	} else {
+	  fprintf_plus(stderr, "Skipping server on non-used interface"
+		       " \"%s\"\n",
+		       if_indextoname((unsigned int)if_index,
+				      interface));
+	}
+      }
+      return -1;
+    }
+  }
+  
+  ret = init_gnutls_session(&session, mc);
   if(ret != 0){
     return -1;
   }
   
   if(debug){
     fprintf_plus(stderr, "Setting up a TCP connection to %s, port %"
-		 PRIu16 "\n", ip, port);
+		 PRIuMAX "\n", ip, (uintmax_t)port);
   }
   
   tcp_sd = socket(pf, SOCK_STREAM, 0);
@@ -699,10 +735,7 @@
     goto mandos_end;
   }
   if(af == AF_INET6){
-    to.in6.sin6_port = htons(port); /* Spurious warnings from
-				       -Wconversion and
-				       -Wunreachable-code */
-    
+    to.in6.sin6_port = htons(port);    
     if(IN6_IS_ADDR_LINKLOCAL /* Spurious warnings from */
        (&to.in6.sin6_addr)){ /* -Wstrict-aliasing=2 or lower and
 				-Wunreachable-code*/
@@ -732,12 +765,12 @@
       if(if_indextoname((unsigned int)if_index, interface) == NULL){
 	perror_plus("if_indextoname");
       } else {
-	fprintf_plus(stderr, "Connection to: %s%%%s, port %" PRIu16
-		     "\n", ip, interface, port);
+	fprintf_plus(stderr, "Connection to: %s%%%s, port %" PRIuMAX
+		     "\n", ip, interface, (uintmax_t)port);
       }
     } else {
-      fprintf_plus(stderr, "Connection to: %s, port %" PRIu16 "\n",
-		   ip, port);
+      fprintf_plus(stderr, "Connection to: %s, port %" PRIuMAX "\n",
+		   ip, (uintmax_t)port);
     }
     char addrstr[(INET_ADDRSTRLEN > INET6_ADDRSTRLEN) ?
 		 INET_ADDRSTRLEN : INET6_ADDRSTRLEN] = "";
@@ -821,8 +854,11 @@
     goto mandos_end;
   }
   
-  /* Spurious warning from -Wint-to-pointer-cast */
-  gnutls_transport_set_ptr(session, (gnutls_transport_ptr_t) tcp_sd);
+  /* This casting via intptr_t is to eliminate warning about casting
+     an int to a pointer type.  This is exactly how the GnuTLS Guile
+     function "set-session-transport-fd!" does it. */
+  gnutls_transport_set_ptr(session,
+			   (gnutls_transport_ptr_t)(intptr_t)tcp_sd);
   
   if(quit_now){
     errno = EINTR;
@@ -933,7 +969,7 @@
   if(buffer_length > 0){
     ssize_t decrypted_buffer_size;
     decrypted_buffer_size = pgp_packet_decrypt(buffer, buffer_length,
-					       &decrypted_buffer);
+					       &decrypted_buffer, mc);
     if(decrypted_buffer_size >= 0){
       
       written = 0;
@@ -1000,8 +1036,10 @@
 			     AVAHI_GCC_UNUSED AvahiStringList *txt,
 			     AVAHI_GCC_UNUSED AvahiLookupResultFlags
 			     flags,
-			     AVAHI_GCC_UNUSED void* userdata){
-  assert(r);
+			     void* mc){
+  if(r == NULL){
+    return;
+  }
   
   /* Called whenever a service has been resolved successfully or
      timed out */
@@ -1016,7 +1054,8 @@
     fprintf_plus(stderr, "(Avahi Resolver) Failed to resolve service "
 		 "'%s' of type '%s' in domain '%s': %s\n", name, type,
 		 domain,
-		 avahi_strerror(avahi_server_errno(mc.server)));
+		 avahi_strerror(avahi_server_errno
+				(((mandos_context*)mc)->server)));
     break;
     
   case AVAHI_RESOLVER_FOUND:
@@ -1028,13 +1067,16 @@
 		     PRIdMAX ") on port %" PRIu16 "\n", name,
 		     host_name, ip, (intmax_t)interface, port);
       }
-      int ret = start_mandos_communication(ip, port, interface,
-					   avahi_proto_to_af(proto));
+      int ret = start_mandos_communication(ip, (in_port_t)port,
+					   interface,
+					   avahi_proto_to_af(proto),
+					   mc);
       if(ret == 0){
-	avahi_simple_poll_quit(mc.simple_poll);
+	avahi_simple_poll_quit(simple_poll);
       } else {
-	if(not add_server(ip, port, interface,
-			  avahi_proto_to_af(proto))){
+	if(not add_server(ip, (in_port_t)port, interface,
+			  avahi_proto_to_af(proto),
+			  &((mandos_context*)mc)->current_server)){
 	  fprintf_plus(stderr, "Failed to add server \"%s\" to server"
 		       " list\n", name);
 	}
@@ -1053,8 +1095,10 @@
 			    const char *domain,
 			    AVAHI_GCC_UNUSED AvahiLookupResultFlags
 			    flags,
-			    AVAHI_GCC_UNUSED void* userdata){
-  assert(b);
+			    void* mc){
+  if(b == NULL){
+    return;
+  }
   
   /* Called whenever a new services becomes available on the LAN or
      is removed from the LAN */
@@ -1068,8 +1112,9 @@
   case AVAHI_BROWSER_FAILURE:
     
     fprintf_plus(stderr, "(Avahi browser) %s\n",
-		 avahi_strerror(avahi_server_errno(mc.server)));
-    avahi_simple_poll_quit(mc.simple_poll);
+		 avahi_strerror(avahi_server_errno
+				(((mandos_context*)mc)->server)));
+    avahi_simple_poll_quit(simple_poll);
     return;
     
   case AVAHI_BROWSER_NEW:
@@ -1078,12 +1123,14 @@
        the callback function is called the Avahi server will free the
        resolver for us. */
     
-    if(avahi_s_service_resolver_new(mc.server, interface, protocol,
-				    name, type, domain, protocol, 0,
-				    resolve_callback, NULL) == NULL)
+    if(avahi_s_service_resolver_new(((mandos_context*)mc)->server,
+				    interface, protocol, name, type,
+				    domain, protocol, 0,
+				    resolve_callback, mc) == NULL)
       fprintf_plus(stderr, "Avahi: Failed to resolve service '%s':"
 		   " %s\n", name,
-		   avahi_strerror(avahi_server_errno(mc.server)));
+		   avahi_strerror(avahi_server_errno
+				  (((mandos_context*)mc)->server)));
     break;
     
   case AVAHI_BROWSER_REMOVE:
@@ -1108,25 +1155,30 @@
   signal_received = sig;
   int old_errno = errno;
   /* set main loop to exit */
-  if(mc.simple_poll != NULL){
-    avahi_simple_poll_quit(mc.simple_poll);
+  if(simple_poll != NULL){
+    avahi_simple_poll_quit(simple_poll);
   }
   errno = old_errno;
 }
 
 bool get_flags(const char *ifname, struct ifreq *ifr){
   int ret;
+  error_t ret_errno;
   
   int s = socket(PF_INET6, SOCK_DGRAM, IPPROTO_IP);
   if(s < 0){
+    ret_errno = errno;
     perror_plus("socket");
+    errno = ret_errno;
     return false;
   }
   strcpy(ifr->ifr_name, ifname);
   ret = ioctl(s, SIOCGIFFLAGS, ifr);
   if(ret == -1){
     if(debug){
+      ret_errno = errno;
       perror_plus("ioctl SIOCGIFFLAGS");
+      errno = ret_errno;
     }
     return false;
   }
@@ -1201,46 +1253,35 @@
 }
 
 /* 
- * This function determines if a directory entry in /sys/class/net
- * corresponds to an acceptable network device which is up.
- * (This function is passed to scandir(3) as a filter function.)
- */
-int up_interface(const struct dirent *if_entry){
-  if(if_entry->d_name[0] == '.'){
-    return 0;
-  }
-  
-  struct ifreq ifr;
-  if(not get_flags(if_entry->d_name, &ifr)){
-    if(debug){
-      fprintf_plus(stderr, "Failed to get flags for interface "
-		   "\"%s\"\n", if_entry->d_name);
-    }
-    return 0;
-  }
-  
-  /* Reject down interfaces */
-  if(not (ifr.ifr_flags & IFF_UP)){
-    if(debug){
-      fprintf_plus(stderr, "Rejecting down interface \"%s\"\n",
-		   if_entry->d_name);
-    }
-    return 0;
-  }
-  
-  /* Reject non-running interfaces */
-  if(not (ifr.ifr_flags & IFF_RUNNING)){
-    if(debug){
-      fprintf_plus(stderr, "Rejecting non-running interface \"%s\"\n",
-		   if_entry->d_name);
-    }
-    return 0;
-  }
-  
-  if(not good_flags(if_entry->d_name, &ifr)){
-    return 0;
-  }
-  return 1;
+ * This function determines if a network interface is up.
+ */
+bool interface_is_up(const char *interface){
+  struct ifreq ifr;
+  if(not get_flags(interface, &ifr)){
+    if(debug){
+      fprintf_plus(stderr, "Failed to get flags for interface "
+		   "\"%s\"\n", interface);
+    }
+    return false;
+  }
+  
+  return (bool)(ifr.ifr_flags & IFF_UP);
+}
+
+/* 
+ * This function determines if a network interface is running
+ */
+bool interface_is_running(const char *interface){
+  struct ifreq ifr;
+  if(not get_flags(interface, &ifr)){
+    if(debug){
+      fprintf_plus(stderr, "Failed to get flags for interface "
+		   "\"%s\"\n", interface);
+    }
+    return false;
+  }
+  
+  return (bool)(ifr.ifr_flags & IFF_RUNNING);
 }
 
 int notdotentries(const struct dirent *direntry){
@@ -1315,14 +1356,15 @@
   return 1;
 }
 
-int avahi_loop_with_timeout(AvahiSimplePoll *s, int retry_interval){
+int avahi_loop_with_timeout(AvahiSimplePoll *s, int retry_interval,
+			    mandos_context *mc){
   int ret;
   struct timespec now;
   struct timespec waited_time;
   intmax_t block_time;
   
   while(true){
-    if(mc.current_server == NULL){
+    if(mc->current_server == NULL){
       if (debug){
 	fprintf_plus(stderr, "Wait until first server is found."
 		     " No timeout!\n");
@@ -1342,9 +1384,9 @@
       /* Calculating in ms how long time between now and server
 	 who we visted longest time ago. Now - last seen.  */
       waited_time.tv_sec = (now.tv_sec
-			    - mc.current_server->last_seen.tv_sec);
+			    - mc->current_server->last_seen.tv_sec);
       waited_time.tv_nsec = (now.tv_nsec
-			     - mc.current_server->last_seen.tv_nsec);
+			     - mc->current_server->last_seen.tv_nsec);
       /* total time is 10s/10,000ms.
 	 Converting to s from ms by dividing by 1,000,
 	 and ns to ms by dividing by 1,000,000. */
@@ -1358,21 +1400,21 @@
       }
       
       if(block_time <= 0){
-	ret = start_mandos_communication(mc.current_server->ip,
-					 mc.current_server->port,
-					 mc.current_server->if_index,
-					 mc.current_server->af);
+	ret = start_mandos_communication(mc->current_server->ip,
+					 mc->current_server->port,
+					 mc->current_server->if_index,
+					 mc->current_server->af, mc);
 	if(ret == 0){
-	  avahi_simple_poll_quit(mc.simple_poll);
+	  avahi_simple_poll_quit(s);
 	  return 0;
 	}
 	ret = clock_gettime(CLOCK_MONOTONIC,
-			    &mc.current_server->last_seen);
+			    &mc->current_server->last_seen);
 	if(ret == -1){
 	  perror_plus("clock_gettime");
 	  return -1;
 	}
-	mc.current_server = mc.current_server->next;
+	mc->current_server = mc->current_server->next;
 	block_time = 0; 	/* Call avahi to find new Mandos
 				   servers, but don't block */
       }
@@ -1387,6 +1429,58 @@
   }
 }
 
+/* Set effective uid to 0, return errno */
+error_t raise_privileges(void){
+  error_t old_errno = errno;
+  error_t ret_errno = 0;
+  if(seteuid(0) == -1){
+    ret_errno = errno;
+    perror_plus("seteuid");
+  }
+  errno = old_errno;
+  return ret_errno;
+}
+
+/* Set effective and real user ID to 0.  Return errno. */
+error_t raise_privileges_permanently(void){
+  error_t old_errno = errno;
+  error_t ret_errno = raise_privileges();
+  if(ret_errno != 0){
+    errno = old_errno;
+    return ret_errno;
+  }
+  if(setuid(0) == -1){
+    ret_errno = errno;
+    perror_plus("seteuid");
+  }
+  errno = old_errno;
+  return ret_errno;
+}
+
+/* Set effective user ID to unprivileged saved user ID */
+error_t lower_privileges(void){
+  error_t old_errno = errno;
+  error_t ret_errno = 0;
+  if(seteuid(uid) == -1){
+    ret_errno = errno;
+    perror_plus("seteuid");
+  }
+  errno = old_errno;
+  return ret_errno;
+}
+
+/* Lower privileges permanently */
+error_t lower_privileges_permanently(void){
+  error_t old_errno = errno;
+  error_t ret_errno = 0;
+  if(setuid(uid) == -1){
+    ret_errno = errno;
+    perror_plus("setuid");
+  }
+  errno = old_errno;
+  return ret_errno;
+}
+
 bool run_network_hooks(const char *mode, const char *interface,
 		       const float delay){
   struct dirent **direntries;
@@ -1395,7 +1489,14 @@
   int numhooks = scandir(hookdir, &direntries, runnable_hook,
 			 alphasort);
   if(numhooks == -1){
-    perror_plus("scandir");
+    if(errno == ENOENT){
+      if(debug){
+	fprintf_plus(stderr, "Network hook directory \"%s\" not"
+		     " found\n", hookdir);
+      }
+    } else {
+      perror_plus("scandir");
+    }
   } else {
     int devnull = open("/dev/null", O_RDONLY);
     for(int i = 0; i < numhooks; i++){
@@ -1414,17 +1515,7 @@
       if(hook_pid == 0){
 	/* Child */
 	/* Raise privileges */
-	errno = 0;
-	ret = seteuid(0);
-	if(ret == -1){
-	  perror_plus("seteuid");
-	}
-	/* Raise privileges even more */
-	errno = 0;
-	ret = setuid(0);
-	if(ret == -1){
-	  perror_plus("setuid");
-	}
+	raise_privileges_permanently();
 	/* Set group */
 	errno = 0;
 	ret = setgid(0);
@@ -1523,24 +1614,198 @@
   return true;
 }
 
+error_t bring_up_interface(const char *const interface,
+			   const float delay){
+  int sd = -1;
+  error_t old_errno = errno;
+  error_t ret_errno = 0;
+  int ret, ret_setflags;
+  struct ifreq network;
+  unsigned int if_index = if_nametoindex(interface);
+  if(if_index == 0){
+    fprintf_plus(stderr, "No such interface: \"%s\"\n", interface);
+    errno = old_errno;
+    return ENXIO;
+  }
+  
+  if(quit_now){
+    errno = old_errno;
+    return EINTR;
+  }
+  
+  if(not interface_is_up(interface)){
+    if(not get_flags(interface, &network) and debug){
+      ret_errno = errno;
+      fprintf_plus(stderr, "Failed to get flags for interface "
+		   "\"%s\"\n", interface);
+      return ret_errno;
+    }
+    network.ifr_flags |= IFF_UP;
+    
+    sd = socket(PF_INET6, SOCK_DGRAM, IPPROTO_IP);
+    if(sd < 0){
+      ret_errno = errno;
+      perror_plus("socket");
+      errno = old_errno;
+      return ret_errno;
+    }
+  
+    if(quit_now){
+      close(sd);
+      errno = old_errno;
+      return EINTR;
+    }
+    
+    if(debug){
+      fprintf_plus(stderr, "Bringing up interface \"%s\"\n",
+		   interface);
+    }
+    
+    /* Raise priviliges */
+    raise_privileges();
+    
+#ifdef __linux__
+    /* Lower kernel loglevel to KERN_NOTICE to avoid KERN_INFO
+       messages about the network interface to mess up the prompt */
+    int ret_linux = klogctl(8, NULL, 5);
+    bool restore_loglevel = true;
+    if(ret_linux == -1){
+      restore_loglevel = false;
+      perror_plus("klogctl");
+    }
+#endif	/* __linux__ */
+    ret_setflags = ioctl(sd, SIOCSIFFLAGS, &network);
+    ret_errno = errno;
+#ifdef __linux__
+    if(restore_loglevel){
+      ret_linux = klogctl(7, NULL, 0);
+      if(ret_linux == -1){
+	perror_plus("klogctl");
+      }
+    }
+#endif	/* __linux__ */
+    
+    /* Lower privileges */
+    lower_privileges();
+    
+    /* Close the socket */
+    ret = (int)TEMP_FAILURE_RETRY(close(sd));
+    if(ret == -1){
+      perror_plus("close");
+    }
+    
+    if(ret_setflags == -1){
+      errno = ret_errno;
+      perror_plus("ioctl SIOCSIFFLAGS +IFF_UP");
+      errno = old_errno;
+      return ret_errno;
+    }
+  } else if(debug){
+    fprintf_plus(stderr, "Interface \"%s\" is already up; good\n",
+		 interface);
+  }
+  
+  /* Sleep checking until interface is running.
+     Check every 0.25s, up to total time of delay */
+  for(int i=0; i < delay * 4; i++){
+    if(interface_is_running(interface)){
+      break;
+    }
+    struct timespec sleeptime = { .tv_nsec = 250000000 };
+    ret = nanosleep(&sleeptime, NULL);
+    if(ret == -1 and errno != EINTR){
+      perror_plus("nanosleep");
+    }
+  }
+  
+  errno = old_errno;
+  return 0;
+}
+
+error_t take_down_interface(const char *const interface){
+  int sd = -1;
+  error_t old_errno = errno;
+  error_t ret_errno = 0;
+  int ret, ret_setflags;
+  struct ifreq network;
+  unsigned int if_index = if_nametoindex(interface);
+  if(if_index == 0){
+    fprintf_plus(stderr, "No such interface: \"%s\"\n", interface);
+    errno = old_errno;
+    return ENXIO;
+  }
+  if(interface_is_up(interface)){
+    if(not get_flags(interface, &network) and debug){
+      ret_errno = errno;
+      fprintf_plus(stderr, "Failed to get flags for interface "
+		   "\"%s\"\n", interface);
+      return ret_errno;
+    }
+    network.ifr_flags &= ~(short)IFF_UP; /* clear flag */
+    
+    sd = socket(PF_INET6, SOCK_DGRAM, IPPROTO_IP);
+    if(sd < 0){
+      ret_errno = errno;
+      perror_plus("socket");
+      errno = old_errno;
+      return ret_errno;
+    }
+    
+    if(debug){
+      fprintf_plus(stderr, "Taking down interface \"%s\"\n",
+		   interface);
+    }
+    
+    /* Raise priviliges */
+    raise_privileges();
+    
+    ret_setflags = ioctl(sd, SIOCSIFFLAGS, &network);
+    ret_errno = errno;
+    
+    /* Lower privileges */
+    lower_privileges();
+    
+    /* Close the socket */
+    ret = (int)TEMP_FAILURE_RETRY(close(sd));
+    if(ret == -1){
+      perror_plus("close");
+    }
+    
+    if(ret_setflags == -1){
+      errno = ret_errno;
+      perror_plus("ioctl SIOCSIFFLAGS -IFF_UP");
+      errno = old_errno;
+      return ret_errno;
+    }
+  } else if(debug){
+    fprintf_plus(stderr, "Interface \"%s\" is already down; odd\n",
+		 interface);
+  }
+  
+  errno = old_errno;
+  return 0;
+}
+
 int main(int argc, char *argv[]){
+  mandos_context mc = { .server = NULL, .dh_bits = 1024,
+			.priority = "SECURE256:!CTYPE-X.509:"
+			"+CTYPE-OPENPGP", .current_server = NULL, 
+			.interfaces = NULL, .interfaces_size = 0 };
   AvahiSServiceBrowser *sb = NULL;
-  int error;
+  error_t ret_errno;
   int ret;
   intmax_t tmpmax;
   char *tmp;
   int exitcode = EXIT_SUCCESS;
-  const char *interface = "";
-  struct ifreq network;
-  int sd = -1;
-  bool take_down_interface = false;
-  uid_t uid;
-  gid_t gid;
+  char *interfaces_to_take_down = NULL;
+  size_t interfaces_to_take_down_size = 0;
   char tempdir[] = "/tmp/mandosXXXXXX";
   bool tempdir_created = false;
   AvahiIfIndex if_index = AVAHI_IF_UNSPEC;
   const char *seckey = PATHDIR "/" SECKEY;
   const char *pubkey = PATHDIR "/" PUBKEY;
+  char *interfaces_hooks = NULL;
+  size_t interfaces_hooks_size = 0;
   
   bool gnutls_initialized = false;
   bool gpgme_initialized = false;
@@ -1637,7 +1902,11 @@
 	connect_to = arg;
 	break;
       case 'i':			/* --interface */
-	interface = arg;
+	ret_errno = argz_add_sep(&mc.interfaces, &mc.interfaces_size,
+				 arg, (int)',');
+	if(ret_errno != 0){
+	  argp_error(state, "%s", strerror(ret_errno));
+	}
 	break;
       case 's':			/* --seckey */
 	seckey = arg;
@@ -1721,11 +1990,7 @@
        <http://bugs.debian.org/633582> */
     
     /* Re-raise priviliges */
-    errno = 0;
-    ret = seteuid(0);
-    if(ret == -1){
-      perror_plus("seteuid");
-    } else {
+    if(raise_privileges() == 0){
       struct stat st;
       
       if(strcmp(seckey, PATHDIR "/" SECKEY) == 0){
@@ -1771,61 +2036,56 @@
       }
     
       /* Lower privileges */
-      errno = 0;
-      ret = seteuid(uid);
-      if(ret == -1){
-	perror_plus("seteuid");
+      lower_privileges();
+    }
+  }
+  
+  /* Remove invalid interface names (except "none") */
+  {
+    char *interface = NULL;
+    while((interface = argz_next(mc.interfaces, mc.interfaces_size,
+				 interface))){
+      if(strcmp(interface, "none") != 0
+	 and if_nametoindex(interface) == 0){
+	if(interface[0] != '\0'){
+	  fprintf_plus(stderr, "Not using nonexisting interface"
+		       " \"%s\"\n", interface);
+	}
+	argz_delete(&mc.interfaces, &mc.interfaces_size, interface);
+	interface = NULL;
       }
     }
   }
   
   /* Run network hooks */
-  if(not run_network_hooks("start", interface, delay)){
-    goto end;
+  {
+    if(mc.interfaces != NULL){
+      interfaces_hooks = malloc(mc.interfaces_size);
+      if(interfaces_hooks == NULL){
+	perror_plus("malloc");
+	goto end;
+      }
+      memcpy(interfaces_hooks, mc.interfaces, mc.interfaces_size);
+      interfaces_hooks_size = mc.interfaces_size;
+      argz_stringify(interfaces_hooks, interfaces_hooks_size,
+		     (int)',');
+    }
+    if(not run_network_hooks("start", interfaces_hooks != NULL ?
+			     interfaces_hooks : "", delay)){
+      goto end;
+    }
   }
   
   if(not debug){
     avahi_set_log_function(empty_log);
   }
   
-  if(interface[0] == '\0'){
-    struct dirent **direntries;
-    /* First look for interfaces that are up */
-    ret = scandir(sys_class_net, &direntries, up_interface,
-		  alphasort);
-    if(ret == 0){
-      /* No up interfaces, look for any good interfaces */
-      free(direntries);
-      ret = scandir(sys_class_net, &direntries, good_interface,
-		    alphasort);
-    }
-    if(ret >= 1){
-      /* Pick the first interface returned */
-      interface = strdup(direntries[0]->d_name);
-      if(debug){
-	fprintf_plus(stderr, "Using interface \"%s\"\n", interface);
-      }
-      if(interface == NULL){
-	perror_plus("malloc");
-	free(direntries);
-	exitcode = EXIT_FAILURE;
-	goto end;
-      }
-      free(direntries);
-    } else {
-      free(direntries);
-      fprintf_plus(stderr, "Could not find a network interface\n");
-      exitcode = EXIT_FAILURE;
-      goto end;
-    }
-  }
-  
   /* Initialize Avahi early so avahi_simple_poll_quit() can be called
      from the signal handler */
   /* Initialize the pseudo-RNG for Avahi */
   srand((unsigned int) time(NULL));
-  mc.simple_poll = avahi_simple_poll_new();
-  if(mc.simple_poll == NULL){
+  simple_poll = avahi_simple_poll_new();
+  if(simple_poll == NULL){
     fprintf_plus(stderr,
 		 "Avahi: Failed to create simple poll object.\n");
     exitcode = EX_UNAVAILABLE;
@@ -1895,148 +2155,88 @@
     }
   }
   
-  /* If the interface is down, bring it up */
-  if(strcmp(interface, "none") != 0){
-    if_index = (AvahiIfIndex) if_nametoindex(interface);
-    if(if_index == 0){
-      fprintf_plus(stderr, "No such interface: \"%s\"\n", interface);
-      exitcode = EX_UNAVAILABLE;
-      goto end;
-    }
-    
-    if(quit_now){
-      goto end;
-    }
-    
-    /* Re-raise priviliges */
-    errno = 0;
-    ret = seteuid(0);
-    if(ret == -1){
-      perror_plus("seteuid");
-    }
-    
-#ifdef __linux__
-    /* Lower kernel loglevel to KERN_NOTICE to avoid KERN_INFO
-       messages about the network interface to mess up the prompt */
-    ret = klogctl(8, NULL, 5);
-    bool restore_loglevel = true;
-    if(ret == -1){
-      restore_loglevel = false;
-      perror_plus("klogctl");
-    }
-#endif	/* __linux__ */
-    
-    sd = socket(PF_INET6, SOCK_DGRAM, IPPROTO_IP);
-    if(sd < 0){
-      perror_plus("socket");
-      exitcode = EX_OSERR;
-#ifdef __linux__
-      if(restore_loglevel){
-	ret = klogctl(7, NULL, 0);
-	if(ret == -1){
-	  perror_plus("klogctl");
-	}
-      }
-#endif	/* __linux__ */
-      /* Lower privileges */
-      errno = 0;
-      ret = seteuid(uid);
-      if(ret == -1){
-	perror_plus("seteuid");
-      }
-      goto end;
-    }
-    strcpy(network.ifr_name, interface);
-    ret = ioctl(sd, SIOCGIFFLAGS, &network);
-    if(ret == -1){
-      perror_plus("ioctl SIOCGIFFLAGS");
-#ifdef __linux__
-      if(restore_loglevel){
-	ret = klogctl(7, NULL, 0);
-	if(ret == -1){
-	  perror_plus("klogctl");
-	}
-      }
-#endif	/* __linux__ */
-      exitcode = EX_OSERR;
-      /* Lower privileges */
-      errno = 0;
-      ret = seteuid(uid);
-      if(ret == -1){
-	perror_plus("seteuid");
-      }
-      goto end;
-    }
-    if((network.ifr_flags & IFF_UP) == 0){
-      network.ifr_flags |= IFF_UP;
-      take_down_interface = true;
-      ret = ioctl(sd, SIOCSIFFLAGS, &network);
-      if(ret == -1){
-	take_down_interface = false;
-	perror_plus("ioctl SIOCSIFFLAGS +IFF_UP");
-	exitcode = EX_OSERR;
-#ifdef __linux__
-	if(restore_loglevel){
-	  ret = klogctl(7, NULL, 0);
-	  if(ret == -1){
-	    perror_plus("klogctl");
+  /* If no interfaces were specified, make a list */
+  if(mc.interfaces == NULL){
+    struct dirent **direntries;
+    /* Look for any good interfaces */
+    ret = scandir(sys_class_net, &direntries, good_interface,
+		  alphasort);
+    if(ret >= 1){
+      /* Add all found interfaces to interfaces list */
+      for(int i = 0; i < ret; ++i){
+	ret_errno = argz_add(&mc.interfaces, &mc.interfaces_size,
+			     direntries[i]->d_name);
+	if(ret_errno != 0){
+	  perror_plus("argz_add");
+	  continue;
+	}
+	if(debug){
+	  fprintf_plus(stderr, "Will use interface \"%s\"\n",
+		       direntries[i]->d_name);
+	}
+      }
+      free(direntries);
+    } else {
+      free(direntries);
+      fprintf_plus(stderr, "Could not find a network interface\n");
+      exitcode = EXIT_FAILURE;
+      goto end;
+    }
+  }
+  
+  /* Bring up interfaces which are down, and remove any "none"s */
+  {
+    char *interface = NULL;
+    while((interface = argz_next(mc.interfaces, mc.interfaces_size,
+				 interface))){
+      /* If interface name is "none", stop bringing up interfaces.
+	 Also remove all instances of "none" from the list */
+      if(strcmp(interface, "none") == 0){
+	argz_delete(&mc.interfaces, &mc.interfaces_size,
+		    interface);
+	interface = NULL;
+	while((interface = argz_next(mc.interfaces,
+				     mc.interfaces_size, interface))){
+	  if(strcmp(interface, "none") == 0){
+	    argz_delete(&mc.interfaces, &mc.interfaces_size,
+			interface);
+	    interface = NULL;
 	  }
 	}
-#endif	/* __linux__ */
-	/* Lower privileges */
-	errno = 0;
-	ret = seteuid(uid);
-	if(ret == -1){
-	  perror_plus("seteuid");
-	}
-	goto end;
-      }
-    }
-    /* Sleep checking until interface is running.
-       Check every 0.25s, up to total time of delay */
-    for(int i=0; i < delay * 4; i++){
-      ret = ioctl(sd, SIOCGIFFLAGS, &network);
-      if(ret == -1){
-	perror_plus("ioctl SIOCGIFFLAGS");
-      } else if(network.ifr_flags & IFF_RUNNING){
 	break;
       }
-      struct timespec sleeptime = { .tv_nsec = 250000000 };
-      ret = nanosleep(&sleeptime, NULL);
-      if(ret == -1 and errno != EINTR){
-	perror_plus("nanosleep");
-      }
-    }
-    if(not take_down_interface){
-      /* We won't need the socket anymore */
-      ret = (int)TEMP_FAILURE_RETRY(close(sd));
-      if(ret == -1){
-	perror_plus("close");
-      }
-    }
-#ifdef __linux__
-    if(restore_loglevel){
-      /* Restores kernel loglevel to default */
-      ret = klogctl(7, NULL, 0);
-      if(ret == -1){
-	perror_plus("klogctl");
-      }
-    }
-#endif	/* __linux__ */
-    /* Lower privileges */
-    errno = 0;
-    /* Lower privileges */
-    ret = seteuid(uid);
-    if(ret == -1){
-      perror_plus("seteuid");
-    }
+      bool interface_was_up = interface_is_up(interface);
+      ret = bring_up_interface(interface, delay);
+      if(not interface_was_up){
+	if(ret != 0){
+	  errno = ret;
+	  perror_plus("Failed to bring up interface");
+	} else {
+	  ret_errno = argz_add(&interfaces_to_take_down,
+			       &interfaces_to_take_down_size,
+			       interface);
+	}
+      }
+    }
+    if(debug and (interfaces_to_take_down == NULL)){
+      fprintf_plus(stderr, "No interfaces were brought up\n");
+    }
+  }
+  
+  /* If we only got one interface, explicitly use only that one */
+  if(argz_count(mc.interfaces, mc.interfaces_size) == 1){
+    if(debug){
+      fprintf_plus(stderr, "Using only interface \"%s\"\n",
+		   mc.interfaces);
+    }
+    if_index = (AvahiIfIndex)if_nametoindex(mc.interfaces);
   }
   
   if(quit_now){
     goto end;
   }
   
-  ret = init_gnutls_global(pubkey, seckey);
+  ret = init_gnutls_global(pubkey, seckey, &mc);
   if(ret == -1){
     fprintf_plus(stderr, "init_gnutls_global failed\n");
     exitcode = EX_UNAVAILABLE;
@@ -2059,7 +2259,7 @@
     goto end;
   }
   
-  if(not init_gpgme(pubkey, seckey, tempdir)){
+  if(not init_gpgme(pubkey, seckey, tempdir, &mc)){
     fprintf_plus(stderr, "init_gpgme failed\n");
     exitcode = EX_UNAVAILABLE;
     goto end;
@@ -2075,6 +2275,7 @@
     /* Connect directly, do not use Zeroconf */
     /* (Mainly meant for debugging) */
     char *address = strrchr(connect_to, ':');
+    
     if(address == NULL){
       fprintf_plus(stderr, "No colon in address\n");
       exitcode = EX_USAGE;
@@ -2085,21 +2286,21 @@
       goto end;
     }
     
-    uint16_t port;
+    in_port_t port;
     errno = 0;
     tmpmax = strtoimax(address+1, &tmp, 10);
     if(errno != 0 or tmp == address+1 or *tmp != '\0'
-       or tmpmax != (uint16_t)tmpmax){
+       or tmpmax != (in_port_t)tmpmax){
       fprintf_plus(stderr, "Bad port number\n");
       exitcode = EX_USAGE;
       goto end;
     }
-  
+    
     if(quit_now){
       goto end;
     }
     
-    port = (uint16_t)tmpmax;
+    port = (in_port_t)tmpmax;
     *address = '\0';
     /* Colon in address indicates IPv6 */
     int af;
@@ -2121,7 +2322,8 @@
     }
     
     while(not quit_now){
-      ret = start_mandos_communication(address, port, if_index, af);
+      ret = start_mandos_communication(address, port, if_index, af,
+				       &mc);
       if(quit_now or ret == 0){
 	break;
       }
@@ -2129,7 +2331,7 @@
 	fprintf_plus(stderr, "Retrying in %d seconds\n",
 		     (int)retry_interval);
       }
-      sleep((int)retry_interval);
+      sleep((unsigned int)retry_interval);
     }
     
     if (not quit_now){
@@ -2153,9 +2355,8 @@
     config.publish_domain = 0;
     
     /* Allocate a new server */
-    mc.server = avahi_server_new(avahi_simple_poll_get
-				 (mc.simple_poll), &config, NULL,
-				 NULL, &error);
+    mc.server = avahi_server_new(avahi_simple_poll_get(simple_poll),
+				 &config, NULL, NULL, &ret_errno);
     
     /* Free the Avahi configuration data */
     avahi_server_config_free(&config);
@@ -2164,7 +2365,7 @@
   /* Check if creating the Avahi server object succeeded */
   if(mc.server == NULL){
     fprintf_plus(stderr, "Failed to create Avahi server: %s\n",
-		 avahi_strerror(error));
+		 avahi_strerror(ret_errno));
     exitcode = EX_UNAVAILABLE;
     goto end;
   }
@@ -2176,7 +2377,8 @@
   /* Create the Avahi service browser */
   sb = avahi_s_service_browser_new(mc.server, if_index,
 				   AVAHI_PROTO_UNSPEC, "_mandos._tcp",
-				   NULL, 0, browse_callback, NULL);
+				   NULL, 0, browse_callback,
+				   (void *)&mc);
   if(sb == NULL){
     fprintf_plus(stderr, "Failed to create service browser: %s\n",
 		 avahi_strerror(avahi_server_errno(mc.server)));
@@ -2194,8 +2396,8 @@
     fprintf_plus(stderr, "Starting Avahi loop search\n");
   }
 
-  ret = avahi_loop_with_timeout(mc.simple_poll,
-				(int)(retry_interval * 1000));
+  ret = avahi_loop_with_timeout(simple_poll,
+				(int)(retry_interval * 1000), &mc);
   if(debug){
     fprintf_plus(stderr, "avahi_loop_with_timeout exited %s\n",
 		 (ret == 0) ? "successfully" : "with error");
@@ -2208,14 +2410,16 @@
   }
   
   /* Cleanup things */
+  free(mc.interfaces);
+  
   if(sb != NULL)
     avahi_s_service_browser_free(sb);
   
   if(mc.server != NULL)
     avahi_server_free(mc.server);
   
-  if(mc.simple_poll != NULL)
-    avahi_simple_poll_free(mc.simple_poll);
+  if(simple_poll != NULL)
+    avahi_simple_poll_free(simple_poll);
   
   if(gnutls_initialized){
     gnutls_certificate_free_credentials(mc.cred);
@@ -2238,41 +2442,37 @@
     }
   }
   
-  /* Run network hooks */
-  run_network_hooks("stop", interface, delay);
-  
   /* Re-raise priviliges */
   {
-    errno = 0;
-    ret = seteuid(0);
-    if(ret == -1){
-      perror_plus("seteuid");
-    }
-    
-    /* Take down the network interface */
-    if(take_down_interface and geteuid() == 0){
-      ret = ioctl(sd, SIOCGIFFLAGS, &network);
-      if(ret == -1){
-	perror_plus("ioctl SIOCGIFFLAGS");
-      } else if(network.ifr_flags & IFF_UP){
-	network.ifr_flags &= ~(short)IFF_UP; /* clear flag */
-	ret = ioctl(sd, SIOCSIFFLAGS, &network);
-	if(ret == -1){
-	  perror_plus("ioctl SIOCSIFFLAGS -IFF_UP");
+    raise_privileges();
+    
+    /* Run network hooks */
+    run_network_hooks("stop", interfaces_hooks != NULL ?
+		      interfaces_hooks : "", delay);
+    
+    /* Take down the network interfaces which were brought up */
+    {
+      char *interface = NULL;
+      while((interface=argz_next(interfaces_to_take_down,
+				 interfaces_to_take_down_size,
+				 interface))){
+	ret_errno = take_down_interface(interface);
+	if(ret_errno != 0){
+	  errno = ret_errno;
+	  perror_plus("Failed to take down interface");
 	}
       }
-      ret = (int)TEMP_FAILURE_RETRY(close(sd));
-      if(ret == -1){
-	perror_plus("close");
+      if(debug and (interfaces_to_take_down == NULL)){
+	fprintf_plus(stderr, "No interfaces needed to be taken"
+		     " down\n");
       }
     }
-  }
-  /* Lower privileges permanently */
-  errno = 0;
-  ret = setuid(uid);
-  if(ret == -1){
-    perror_plus("setuid");
-  }
+    
+    lower_privileges_permanently();
+  }
+  
+  free(interfaces_to_take_down);
+  free(interfaces_hooks);
   
   /* Removes the GPGME temp directory and all files inside */
   if(tempdir_created){

=== modified file 'plugins.d/mandos-client.xml'
--- plugins.d/mandos-client.xml	2011-12-27 03:56:39 +0000
+++ plugins.d/mandos-client.xml	2013-06-23 15:13:06 +0000
@@ -2,7 +2,7 @@
 <!DOCTYPE refentry PUBLIC "-//OASIS//DTD DocBook XML V4.5//EN"
 	"http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd" [
 <!ENTITY COMMANDNAME "mandos-client">
-<!ENTITY TIMESTAMP "2011-11-27">
+<!ENTITY TIMESTAMP "2013-06-21">
 <!ENTITY % common SYSTEM "../common.ent">
 %common;
 ]>
@@ -33,7 +33,7 @@
     <copyright>
       <year>2008</year>
       <year>2009</year>
-      <year>2011</year>
+      <year>2012</year>
       <holder>Teddy Hogeborn</holder>
       <holder>Björn Påhlsson</holder>
     </copyright>
@@ -64,11 +64,13 @@
 	><replaceable>PORT</replaceable></option></arg>
       </group>
       <sbr/>
-      <group>
+      <group rep='repeat'>
 	<arg choice="plain"><option>--interface
-	<replaceable>NAME</replaceable></option></arg>
-	<arg choice="plain"><option>-i
-	<replaceable>NAME</replaceable></option></arg>
+	<replaceable>NAME</replaceable><arg rep='repeat'
+	>,<replaceable>NAME</replaceable></arg></option></arg>
+	<arg choice="plain"><option>-i <replaceable>NAME</replaceable
+	><arg rep='repeat'>,<replaceable>NAME</replaceable></arg
+	></option></arg>
       </group>
       <sbr/>
       <group>
@@ -137,10 +139,10 @@
       communicates with <citerefentry><refentrytitle
       >mandos</refentrytitle><manvolnum>8</manvolnum></citerefentry>
       to get a password.  In slightly more detail, this client program
-      brings up a network interface, uses the interface’s IPv6
-      link-local address to get network connectivity, uses Zeroconf to
-      find servers on the local network, and communicates with servers
-      using TLS with an OpenPGP key to ensure authenticity and
+      brings up network interfaces, uses the interfaces’ IPv6
+      link-local addresses to get network connectivity, uses Zeroconf
+      to find servers on the local network, and communicates with
+      servers using TLS with an OpenPGP key to ensure authenticity and
       confidentiality.  This client program keeps running, trying all
       servers on the network, until it receives a satisfactory reply
       or a TERM signal.  After all servers have been tried, all
@@ -148,23 +150,21 @@
       will wait indefinitely for new servers to appear.
     </para>
     <para>
-      The network interface is selected like this: If an interface is
-      specified using the <option>--interface</option> option, that
-      interface is used.  Otherwise, <command>&COMMANDNAME;</command>
-      will choose any interface that is up and running and is not a
-      loopback interface, is not a point-to-point interface, is
-      capable of broadcasting and does not have the NOARP flag (see
+      The network interfaces are selected like this: If any interfaces
+      are specified using the <option>--interface</option> option,
+      those interface are used.  Otherwise,
+      <command>&COMMANDNAME;</command> will use all interfaces that
+      are not loopback interfaces, are not point-to-point interfaces,
+      are capable of broadcasting and do not have the NOARP flag (see
       <citerefentry><refentrytitle>netdevice</refentrytitle>
       <manvolnum>7</manvolnum></citerefentry>).  (If the
       <option>--connect</option> option is used, point-to-point
-      interfaces and non-broadcast interfaces are accepted.)  If no
-      acceptable interfaces are found, re-run the check but without
-      the <quote>up and running</quote> requirement, and manually take
-      the selected interface up (and later take it down on program
-      exit).
+      interfaces and non-broadcast interfaces are accepted.)  If any
+      used interfaces are not up and running, they are first taken up
+      (and later taken down again on program exit).
     </para>
     <para>
-      Before a network interface is selected, all <quote>network
+      Before network interfaces are selected, all <quote>network
       hooks</quote> are run; see <xref linkend="network-hooks"/>.
     </para>
     <para>
@@ -226,37 +226,42 @@
       
       <varlistentry>
 	<term><option>--interface=<replaceable
-	>NAME</replaceable></option></term>
+	>NAME</replaceable><arg rep='repeat'>,<replaceable
+	>NAME</replaceable></arg></option></term>
 	<term><option>-i
-	<replaceable>NAME</replaceable></option></term>
+	<replaceable>NAME</replaceable><arg rep='repeat'>,<replaceable
+	>NAME</replaceable></arg></option></term>
 	<listitem>
 	  <para>
-	    Network interface that will be brought up and scanned for
-	    Mandos servers to connect to.  The default is the empty
-	    string, which will automatically choose an appropriate
-	    interface.
+	    Comma separated list of network interfaces that will be
+	    brought up and scanned for Mandos servers to connect to.
+	    The default is the empty string, which will automatically
+	    use all appropriate interfaces.
 	  </para>
 	  <para>
-	    If the <option>--connect</option> option is used, this
-	    specifies the interface to use to connect to the address
-	    given.
+	    If the <option>--connect</option> option is used, and
+	    exactly one interface name is specified (except
+	    <quote><literal>none</literal></quote>), this specifies
+	    the interface to use to connect to the address given.
 	  </para>
 	  <para>
 	    Note that since this program will normally run in the
 	    initial RAM disk environment, the interface must be an
 	    interface which exists at that stage.  Thus, the interface
-	    can not be a pseudo-interface such as <quote>br0</quote>
-	    or <quote>tun0</quote>; such interfaces will not exist
-	    until much later in the boot process, and can not be used
-	    by this program, unless created by a <quote>network
-	    hook</quote> — see <xref linkend="network-hooks"/>.
+	    can normally not be a pseudo-interface such as
+	    <quote>br0</quote> or <quote>tun0</quote>; such interfaces
+	    will not exist until much later in the boot process, and
+	    can not be used by this program, unless created by a
+	    <quote>network hook</quote> — see <xref
+	    linkend="network-hooks"/>.
 	  </para>
 	  <para>
 	    <replaceable>NAME</replaceable> can be the string
-	    <quote><literal>none</literal></quote>; this will not use
-	    any specific interface, and will not bring up an interface
-	    on startup.  This is not recommended, and only meant for
-	    advanced users.
+	    <quote><literal>none</literal></quote>; this will make
+	    <command>&COMMANDNAME;</command> not bring up
+	    <emphasis>any</emphasis> interfaces specified
+	    <emphasis>after</emphasis> this string.  This is not
+	    recommended, and only meant for advanced users.
 	  </para>
 	</listitem>
       </varlistentry>
@@ -314,7 +319,7 @@
 	>SECONDS</replaceable></option></term>
 	<listitem>
 	  <para>
-	    After bringing the network interface up, the program waits
+	    After bringing a network interface up, the program waits
 	    for the interface to arrive in a <quote>running</quote>
 	    state before proceeding.  During this time, the kernel log
 	    level will be lowered to reduce clutter on the system
@@ -507,7 +512,7 @@
 	      It is not necessary to print any non-executable files
 	      already in the network hook directory, these will be
 	      copied implicitly if they otherwise satisfy the name
-	      requirement.
+	      requirements.
 	    </para>
 	  </listitem>
 	</varlistentry>
@@ -547,11 +552,12 @@
 	  <term><envar>DEVICE</envar></term>
 	  <listitem>
 	    <para>
-	      The network interface, as specified to
+	      The network interfaces, as specified to
 	      <command>&COMMANDNAME;</command> by the
-	      <option>--interface</option> option.  If this is not the
-	      interface a hook will bring up, there is no reason for a
-	      hook to continue.
+	      <option>--interface</option> option, combined to one
+	      string and separated by commas.  If this is set, and
+	      does not contain the interface a hook will bring up,
+	      there is no reason for a hook to continue.
 	    </para>
 	  </listitem>
 	</varlistentry>
@@ -660,8 +666,8 @@
     </para>
     <informalexample>
       <para>
-	Normal invocation needs no options, if the network interface
-	is <quote>eth0</quote>:
+	Normal invocation needs no options, if the network interfaces
+	can be automatically determined:
       </para>
       <para>
 	<userinput>&COMMANDNAME;</userinput>
@@ -669,8 +675,8 @@
     </informalexample>
     <informalexample>
       <para>
-	Search for Mandos servers (and connect to them) using another
-	interface:
+	Search for Mandos servers (and connect to them) using one
+	specific interface:
       </para>
       <para>
 	<!-- do not wrap this line -->
@@ -840,7 +846,7 @@
 	      <para>
 		This client uses IPv6 link-local addresses, which are
 		immediately usable since a link-local addresses is
-		automatically assigned to a network interfaces when it
+		automatically assigned to a network interface when it
 		is brought up.
 	      </para>
 	    </listitem>

=== modified file 'plugins.d/password-prompt.c'
--- plugins.d/password-prompt.c	2011-12-25 00:40:09 +0000
+++ plugins.d/password-prompt.c	2013-10-05 19:34:40 +0000
@@ -2,8 +2,8 @@
 /*
  * Password-prompt - Read a password from the terminal and print it
  * 
- * Copyright © 2008-2011 Teddy Hogeborn
- * Copyright © 2008-2011 Björn Påhlsson
+ * Copyright © 2008-2012 Teddy Hogeborn
+ * Copyright © 2008-2012 Björn Påhlsson
  * 
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -79,7 +79,7 @@
   
   TEMP_FAILURE_RETRY(fprintf(stream, "Mandos plugin %s: ",
 			     program_invocation_short_name));
-  return TEMP_FAILURE_RETRY(vfprintf(stream, format, ap));
+  return (int)TEMP_FAILURE_RETRY(vfprintf(stream, format, ap));
 }
 
 /* Function to use when printing errors */
@@ -124,13 +124,13 @@
     int ret;
     int cl_fd;
     {
-      uintmax_t maxvalue;
+      uintmax_t proc_id;
       char *tmp;
       errno = 0;
-      maxvalue = strtoumax(proc_entry->d_name, &tmp, 10);
+      proc_id = strtoumax(proc_entry->d_name, &tmp, 10);
       
       if(errno != 0 or *tmp != '\0'
-	 or maxvalue != (uintmax_t)((pid_t)maxvalue)){
+	 or proc_id != (uintmax_t)((pid_t)proc_id)){
 	return 0;
       }
     }
@@ -514,6 +514,7 @@
 	switch(e){
 	case EBADF:
 	  status = EX_UNAVAILABLE;
+	  break;
 	case EIO:
 	case EINVAL:
 	default:

=== modified file 'plugins.d/password-prompt.xml'
--- plugins.d/password-prompt.xml	2011-10-05 16:00:56 +0000
+++ plugins.d/password-prompt.xml	2011-12-31 23:05:34 +0000
@@ -2,7 +2,7 @@
 <!DOCTYPE refentry PUBLIC "-//OASIS//DTD DocBook XML V4.5//EN"
 	"http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd" [
 <!ENTITY COMMANDNAME "password-prompt">
-<!ENTITY TIMESTAMP "2011-10-03">
+<!ENTITY TIMESTAMP "2012-01-01">
 <!ENTITY % common SYSTEM "../common.ent">
 %common;
 ]>
@@ -34,6 +34,7 @@
       <year>2008</year>
       <year>2009</year>
       <year>2011</year>
+      <year>2012</year>
       <holder>Teddy Hogeborn</holder>
       <holder>Björn Påhlsson</holder>
     </copyright>

=== modified file 'plugins.d/plymouth.c'
--- plugins.d/plymouth.c	2011-12-25 00:40:09 +0000
+++ plugins.d/plymouth.c	2013-08-27 21:47:35 +0000
@@ -2,8 +2,8 @@
 /*
  * Plymouth - Read a password from Plymouth and output it
  * 
- * Copyright © 2010-2011 Teddy Hogeborn
- * Copyright © 2010-2011 Björn Påhlsson
+ * Copyright © 2010-2012 Teddy Hogeborn
+ * Copyright © 2010-2012 Björn Påhlsson
  * 
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -156,7 +156,7 @@
 
 __attribute__((nonnull (2, 3)))
 bool exec_and_wait(pid_t *pid_return, const char *path,
-		   const char **argv, bool interruptable,
+		   const char * const *argv, bool interruptable,
 		   bool daemonize){
   int status;
   int ret;
@@ -218,13 +218,13 @@
 int is_plymouth(const struct dirent *proc_entry){
   int ret;
   {
-    uintmax_t maxvalue;
+    uintmax_t proc_id;
     char *tmp;
     errno = 0;
-    maxvalue = strtoumax(proc_entry->d_name, &tmp, 10);
+    proc_id = strtoumax(proc_entry->d_name, &tmp, 10);
 
     if(errno != 0 or *tmp != '\0'
-       or maxvalue != (uintmax_t)((pid_t)maxvalue)){
+       or proc_id != (uintmax_t)((pid_t)proc_id)){
       return 0;
     }
   }
@@ -265,36 +265,36 @@
 
 pid_t get_pid(void){
   int ret;
-  uintmax_t maxvalue = 0;
+  uintmax_t proc_id = 0;
   FILE *pidfile = fopen(plymouth_pid, "r");
   /* Try the new pid file location */
   if(pidfile != NULL){
-    ret = fscanf(pidfile, "%" SCNuMAX, &maxvalue);
+    ret = fscanf(pidfile, "%" SCNuMAX, &proc_id);
     if(ret != 1){
-      maxvalue = 0;
+      proc_id = 0;
     }
     fclose(pidfile);
   }
   /* Try the old pid file location */
-  if(maxvalue == 0){
+  if(proc_id == 0){
     pidfile = fopen(plymouth_pid, "r");
     if(pidfile != NULL){
-      ret = fscanf(pidfile, "%" SCNuMAX, &maxvalue);
+      ret = fscanf(pidfile, "%" SCNuMAX, &proc_id);
       if(ret != 1){
-	maxvalue = 0;
+	proc_id = 0;
       }
       fclose(pidfile);
     }
   }
   /* Look for a plymouth process */
-  if(maxvalue == 0){
+  if(proc_id == 0){
     struct dirent **direntries = NULL;
     ret = scandir("/proc", &direntries, is_plymouth, alphasort);
     if (ret == -1){
       error_plus(0, errno, "scandir");
     }
     if (ret > 0){
-      ret = sscanf(direntries[0]->d_name, "%" SCNuMAX, &maxvalue);
+      ret = sscanf(direntries[0]->d_name, "%" SCNuMAX, &proc_id);
       if (ret < 0){
 	error_plus(0, errno, "sscanf");
       }
@@ -304,15 +304,15 @@
     free(direntries);
   }
   pid_t pid;
-  pid = (pid_t)maxvalue;
-  if((uintmax_t)pid == maxvalue){
+  pid = (pid_t)proc_id;
+  if((uintmax_t)pid == proc_id){
     return pid;
   }
   
   return 0;
 }
 
-const char **getargv(pid_t pid){
+const char * const * getargv(pid_t pid){
   int cl_fd;
   char *cmdline_filename;
   ssize_t sret;
@@ -379,7 +379,7 @@
     return NULL;
   }
   argz_extract(cmdline, cmdline_len, argv); /* Create argv */
-  return (const char **)argv;
+  return (const char * const *)argv;
 }
 
 int main(__attribute__((unused))int argc,
@@ -460,7 +460,7 @@
   }
   kill_and_wait(plymouth_command_pid);
   
-  const char **plymouthd_argv;
+  const char * const *plymouthd_argv;
   pid_t pid = get_pid();
   if(pid == 0){
     error_plus(0, 0, "plymouthd pid not found");

=== modified file 'plugins.d/plymouth.xml'
--- plugins.d/plymouth.xml	2011-11-12 18:14:55 +0000
+++ plugins.d/plymouth.xml	2011-12-31 23:05:34 +0000
@@ -2,7 +2,7 @@
 <!DOCTYPE refentry PUBLIC "-//OASIS//DTD DocBook XML V4.5//EN"
 	"http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd" [
 <!ENTITY COMMANDNAME "plymouth">
-<!ENTITY TIMESTAMP "2011-10-03">
+<!ENTITY TIMESTAMP "2012-01-01">
 <!ENTITY % common SYSTEM "../common.ent">
 %common;
 ]>
@@ -33,6 +33,7 @@
     <copyright>
       <year>2010</year>
       <year>2011</year>
+      <year>2012</year>
       <holder>Teddy Hogeborn</holder>
       <holder>Björn Påhlsson</holder>
     </copyright>

=== modified file 'plugins.d/splashy.c'
--- plugins.d/splashy.c	2011-12-24 23:17:02 +0000
+++ plugins.d/splashy.c	2011-12-31 23:05:34 +0000
@@ -2,8 +2,8 @@
 /*
  * Splashy - Read a password from splashy and output it
  * 
- * Copyright © 2008-2011 Teddy Hogeborn
- * Copyright © 2008-2011 Björn Påhlsson
+ * Copyright © 2008-2012 Teddy Hogeborn
+ * Copyright © 2008-2012 Björn Påhlsson
  * 
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as

=== modified file 'plugins.d/splashy.xml'
--- plugins.d/splashy.xml	2011-10-05 16:00:56 +0000
+++ plugins.d/splashy.xml	2011-12-31 23:05:34 +0000
@@ -2,7 +2,7 @@
 <!DOCTYPE refentry PUBLIC "-//OASIS//DTD DocBook XML V4.5//EN"
 	"http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd" [
 <!ENTITY COMMANDNAME "splashy">
-<!ENTITY TIMESTAMP "2011-10-03">
+<!ENTITY TIMESTAMP "2012-01-01">
 <!ENTITY % common SYSTEM "../common.ent">
 %common;
 ]>
@@ -33,7 +33,7 @@
     <copyright>
       <year>2008</year>
       <year>2009</year>
-      <year>2011</year>
+      <year>2012</year>
       <holder>Teddy Hogeborn</holder>
       <holder>Björn Påhlsson</holder>
     </copyright>

=== modified file 'plugins.d/usplash.c'
--- plugins.d/usplash.c	2011-12-24 23:17:02 +0000
+++ plugins.d/usplash.c	2011-12-31 23:05:34 +0000
@@ -2,8 +2,8 @@
 /*
  * Usplash - Read a password from usplash and output it
  * 
- * Copyright © 2008-2011 Teddy Hogeborn
- * Copyright © 2008-2011 Björn Påhlsson
+ * Copyright © 2008-2012 Teddy Hogeborn
+ * Copyright © 2008-2012 Björn Påhlsson
  * 
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as

=== modified file 'plugins.d/usplash.xml'
--- plugins.d/usplash.xml	2011-11-12 18:14:55 +0000
+++ plugins.d/usplash.xml	2011-12-31 23:05:34 +0000
@@ -2,7 +2,7 @@
 <!DOCTYPE refentry PUBLIC "-//OASIS//DTD DocBook XML V4.5//EN"
 	"http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd" [
 <!ENTITY COMMANDNAME "usplash">
-<!ENTITY TIMESTAMP "2011-10-03">
+<!ENTITY TIMESTAMP "2012-01-01">
 <!ENTITY % common SYSTEM "../common.ent">
 %common;
 ]>
@@ -34,6 +34,7 @@
       <year>2008</year>
       <year>2009</year>
       <year>2011</year>
+      <year>2012</year>
       <holder>Teddy Hogeborn</holder>
       <holder>Björn Påhlsson</holder>
     </copyright>

